/* -------------------------------------------------------------------- */
/* File:     curves.c 

   Contents: Curve fitting functions

   Uses:     nlib.c, linear.c */
/* -------------------------------------------------------------------- */


/* -------------------------------------------------------------------- */
float piece (float a,vector x,vector y,int n) {
/* -------------------------------------------------------------------- */

/* Description: Perform a piecewise-linear interpolation at x.

   On entry:    a = evaluation point
                x = n by 1 vector of strictly increasing independent
                    variables
                y = n by 1 vector of dependent variables 
                 n = number of sample points (n >= 2)

   Returns:     The interpolated or extrapolated value a x = a. */

   int		k = 1;
   float	em = epsilon(),
		sigma,f;

/* Determine segment index */

   if (n < 2)
      nerror (100,"piece",4,(float) n,""); 
   while ((a > x[k+1]) && (k < n-1))
      k++; 
      
/* Interpolate */

   if (fabs(x[k+1] - x[k]) < em) {
      nerror (309,"piece",0,0,"");
      exit (1); }
   sigma = (a - x[k])/(x[k+1] - x[k]);
   f = (1 - sigma)*y[k] + sigma*y[k+1];
   return f; }

/* -------------------------------------------------------------------- */
float spline (float a,vector x,vector y,vector c,int n,int ic) {
/* -------------------------------------------------------------------- */

/* Description: Evaluate a cubic spline function at x.

   On entry:    a  = evaluation point
                x  = n by 1 vector of strictly increasing independent
                     variables
                y  = n by 1 vector of dependent variables 

                c  = n by 1 vector containing the spline coefficients,
                n  = number of sample points (n >= 3)
                ic = initial condition code defined as follows:

                    0  =>  use previously computed c
                    1  =>  compute vector c
 
   On exit:    spline = natural cubic spline function evaluated at a

   Notes:       1. spline is initialized using ic <> 0 and then 
                   subsequent calls use ic = 1.
                2. spline uses the function tridec in linear.c to
                   compute the cubic spline coefficients. */   
            
   int		k = 1,
		    i;
   float	s,b0,b1,p;
   vector	h,w,v;
   matrix	T;

   if (n < 3)
      nerror (100,"spline",5,(float) n,"");
   if (ic) {

/* Compute spline coefficients */
   
      h = vec (n-1,"");			
      w = vec (n-1,"");
      v = vec (n,"");			
      T = mat (3,n,"");			

/* Initialize right-hand side v */

      for (i = 1; i < n; i++)
      { 
         h[i] = x[i+1] - x[i];
         if (h[i] > 0)
            w[i] = (y[i+1] - y[i])/h[i];
         else
         {
            printf ("\nThe independent variables in spline must be");
            printf ("\nstrictly increasing.\n");
            return 0;
         }
      }
      v[1]   = 0;
      v[n] = 0;
      for (i = 2; i < n; i++) 
         v[i] = 6*(w[i] - w[i-1]);

/* Solve equations */

      for (i = 1; i < n; i++)
      {
      	 if (i > 1)
         {
      	    T[1][i] = h[i];
      	    T[2][i] = 2*(h[i-1] + h[i]);
         }
         if (i < n-1)
            T[3][i] = h[i];
      }
      T[2][1] = T[2][n] = 1;
      tridec (T,v,n,c);
      freemat (T,3);
      freevec (v);
      freevec (w);
      freevec (h);
   }

/* Evaluate spline function at a */

   while ((a > x[k+1]) && (k < n-1)) k++;    /* segment k */
   p = x[k+1] - x[k];			
   b0 = (6*y[k+1] - p*p*c[k+1])/(6*p); 
   b1 = (6*y[k]   - p*p*c[k])/(6*p); 
   s = b0*(a - x[k]) + b1*(x[k+1] - a); 
   s += (float) (pow(a-x[k],3)*c[k+1] + pow(x[k+1]-a,3)*c[k])/(6*p);
   return s; }

/* -------------------------------------------------------------------- */
float polyfit (float a,vector x,vector y,vector w,matrix A,int n,
               int m,int ic) 
/* -------------------------------------------------------------------- */
{
/* Description: Evaluate the least-squares polynomial of degree
                m-1 at x.

   On entry:    a  = evaluation point
                x  = n by 1 vector of strictly increasing independent
                     variables
                y  = n by 1 vector of dependent variables     
                w  = n by 1 vector containing the weights
                A  = 3 by (m+1) matrix containing least-squares coefficients 
                     in row 1, and orthogonal polynomial coefficients
                     in rows 2 and 3
                n  = number of data samples (n >= 1)
                m  = polynomial degree (0 <= m < n) 
                ic = initiai condition code defined as follows:

                     0 =>  use previously computed A
                     1 =>  compute matrix A

   Returned:    the least-squares polynomial evaluated at x

   Notes:       When m = n, the least-squares polynomial is the
                interpolating polynomial.  polyfit is initialized
                using ic <> 0, and then subsequent calls use ic = 1. */
 
   int		i,k;
   float	f = 0,
		s = 1;

   if      (n < 1)
      nerror (100,"polyfit",6,(float) n,"");
   else if ((m < 0) || (m >= n))
      nerror (100,"polyfit",7,(float) m,"");
   if (ic)
   {

/* Compute orthogonal polynomial coefficients b and c */

      for (k = 1; k <= m+1; k++)
      {
         A[3][k] = 0;
         if (k > 1)
         {
            for (i = 1; i <= n; i++) 
               A[3][k] += w[i]*x[i]*orthpoly(x[i],A,m,k-1)*
                          orthpoly(x[i],A,m,k-2);    
            A[3][k] /= s;
         }
         A[2][k] = 0;
         s = 0;
         for (i = 1; i <= n; i++)
         {
            A[2][k] += w[i]*x[i]*orthpoly(x[i],A,m,k-1)*
                       orthpoly(x[i],A,m,k-1);
            s += w[i]*orthpoly(x[i],A,m,k-1)*orthpoly(x[i],A,m,k-1);
         }
         A[2][k] /= s;
      }

/* Compute least-squares polynomial coefficients: a */

      for (i = 1; i <= m+1; i++)
      {
         s = 0; 	
         A[1][i] = 0;
         for (k = 1; k <= n; k++)
         {
            A[1][i] += y[k]*orthpoly(x[k],A,m,i-1);
            s += w[k]*orthpoly(x[k],A,m,i-1)*orthpoly(x[k],A,m,i-1);
         }
         A[1][i] /= s;
      }
   }

/* Evaluate least-squares polynomial */

   for (i = 1; i <= m+1; i++)
      f += A[1][i]*orthpoly(a,A,m,i-1);
   return f;
}

/* -------------------------------------------------------------------- */
float bilin (float a,float b,vector x,vector y,matrix Z,int m,int n) {
/* -------------------------------------------------------------------- */

/* Description: Perform a bilinear interpolation of the surface f(x,y)
                evaluated at the point (a,b).

   On entry:	a = x coordinate of point of evaluation
                b = y coordinate of point of evaluation
                x = m by 1 vector containing independent variables
                y = n by 1 vector containing independent variables
                Z = m by n matrix containing dependent variables

                       Z[k][j] = f(x[k],y[j])

                m = number of x values (m >= 2)
                n = number of y values (n >= 2)

   Returned:    The estimated value of f(x,y) at x = a and y = b
                 
   Notes:       The independent variables in the vectors x and y
                must be strictly increasing.  If (a,b) does not lie
                in the data range x[1] <= x <= x[m] and y[1] <= y <=
                y[n], then extrapolation using the nearest grid
                element is used. */

   int		k = 1,
		j = 1;
   float	em = epsilon(),
		u,v,f;

/* Initialize and determine grid indices */

   if (m < 2)
      nerror (100,"bilin",6,(float) m,""); 
   if (n < 2)
      nerror (100,"bilin",7,(float) n,"");
   while ((a > x[k+1]) && (k < m-1))
      k++; 
   while ((b > y[j+1]) && (j < n-1))
      j++; 
      
/* Interpolate */

   if ((fabs(x[k+1] - x[k]) < em) || (fabs(y[j+1] - y[j]) < em)) {
      nerror (309,"bilin",0,0,"");
      exit (1); }
   u = (a - x[k])/(x[k+1] - x[k]);
   v = (b - y[j])/(y[j+1] - y[j]);
   f = u*v*Z[k+1][j+1] + (1-u)*v*Z[k][j+1] + u*(1-v)*Z[k+1][j] +
       (1-u)*(1-v)*Z[k][j];
       
/*
   printf ("\n(k,j) (x[k],y[j]) (u,v) f = (%i,%i) (%g,%g) (%g,%g) %g",
           k,j,x[k],y[j],u,v,f);
   if (getch() == 'x') exit(1);
*/

   return f; }

/* -------------------------------------------------------------------- */
void smooth (vector x,vector y,int n,int m,vector z)
/* -------------------------------------------------------------------- */
{
/* Description: Smooth the data in a vectors using a least-squares
                polynomial.

   On entry:    x = n by 1 vector of strictly increasing independent
                    variables
                y = n by 1 vector of dependent variables
                n = number of points
                m = degree of least-squares polynomial fitted to y  

   On exit:     z = n by 1 vector containing smoothed version of y
                    obtained by evaluating least squares polynomial
                    at x. */
                    

   int         i;                 
   vector      w;                  
   matrix      A;


/* Initialize */

   if (n < 1)
      nerror (100,"smooth",3,(float) n,"");
   if ((m < 0) || (m >= n))
      nerror (100,"smooth",4,(float) m,"");
   w = vec (n,"");
   for (i = 1; i <= n; i++)
      w[i] = 1;
   A = mat (3,m+1,"");         
                
/* Find least-squares fit */

   polyfit (0,x,y,w,A,n,m,1);
   for (i = 1; i <= n; i++)
      z[i] = polyfit (x[i],x,y,w,A,n,m,0);

/* Finalize */

   freevec (w);
   freemat (A,3);
}   
   
/* -------------------------------------------------------------------- */
float orthpoly (float x,matrix A,int m,int k) {
/* -------------------------------------------------------------------- */

/* Description: Evaluate the kth orthogonal polynomial at x.

   On entry:    x = point of evaluation
		A = 3 by (m+1) matrix A containing the coefficients of 
                    the orthogonal polynomials in columns 2 and 3
                m = maximum polynomial degree
                k = polynomial degree (1 <= k <= m) 

   Notes:       A is computed by calling polyfit with ic <> 0;  The
                orthogonal polynomials are computed as follows:
 
                   orthpoly[1]   = 1
                   orthpoly[2]   = x - A[2][1]
                   orthpoly[j+1] = (x - A[2][j])*orthpoly[j] -
                                    A[3][j]*orthpoly[j-1] , j >= 2 */
            
   int	   j;		   	
   float   q;
   vector  p;	

   if      (m < 0)
      nerror (100,"orthpoly",3,(float) m,"");
   else if ((k < 0) || (k > m)) {
      nerror (100,"orthpoly",4,(float) k,"");
      shownum ("m = ",(float) m); }
      
   switch (k) {
      case 0:
         return 1;
      case 1:
         return (x - A[2][1]);
      default:
         p = vec (k+1,"");
         p[1] = 1;
         p[2] = x - A[2][1];
         for (j = 2; j <= k; j++)
            p[j+1] = (x - A[2][j])*p[j] - A[3][j]*p[j-1];
         q = p[k+1];
         freevec (p);
         return q; } }



