/* -------------------------------------------------------------------- */
/* File:        draw.c

   Description: This file contains NLIB functions that write data
                into Matlab executable M-files.  These M-files can be
                run from Matlab to produce plots and perform additional
                Matlab processing. */
/* -------------------------------------------------------------------- */

                       
/* -------------------------------------------------------------------- */
void  putstr0  (char *s,char *fname,int mode) {
/* -------------------------------------------------------------------- */

/* Description: Write string s to specified file.

   On entry:    s     = string to be written
                fname = file name
                mode  = writing mode (0 = append, 1 = replace) */

   FILE	       *f;	             

/* Check arguments */
   
   if (mode)
      f = fopen (fname,"w");
   else
      f = fopen (fname,"a");   
   if (f == NULL) 
      nerror (102,"putstr0",0,0,fname);
      
/* Write string */

   fprintf (f,"%s\n",s);
   fclose (f); }

/* -------------------------------------------------------------------- */
void  putstr1  (char *title,char *xaxis,char *yaxis,char *fname) {
/* -------------------------------------------------------------------- */

/* Description: Append title and axis labels to specified file.

   On entry:    title = plot title
                xaxis = x-axis label
                yaxis = y-axis label
                fname = file name */

   FILE	       *f;	             
   char        s[81];

/* Check arguments */
   
   f = fopen (fname,"a");
   if (f == NULL) 
      nerror (102,"putstr1",0,0,fname);

/* Write strings */

   if (strlen(title) > 0) {
      sprintf (s,"title('%s')\n",title);
      fprintf (f,"%s",s); }
   if (strlen(xaxis) > 0) {  
      sprintf (s,"xlabel('%s')\n",xaxis);
      fprintf (f,"%s",s); }
   if (strlen(yaxis) > 0) {
      sprintf (s,"ylabel('%s')\n",yaxis);
      fprintf (f,"%s",s); }
   fclose (f); }


/* -------------------------------------------------------------------- */
void  putstr2  (char *title,char *xaxis,char *yaxis,
                char *zaxis,char *fname) {
/* -------------------------------------------------------------------- */

/* Description: Append title and axis labels to specified file.

   On entry:    title = plot title
                xaxis = x-axis label
                yaxis = y-axis label
                zaxis = z-axis label
                fname = file name */

   FILE	       *f;	             
   char        s[81];


/* Check arguments */

   f = fopen (fname,"a");
   if (f == NULL) 
      nerror (102,"putstr2",0,0,fname);

/* Write strings */

   if (strlen(title) > 0) {
      sprintf (s,"title('%s')\n",title);
      fprintf (f,"%s",s); }
   if (strlen(xaxis) > 0) {  
      sprintf (s,"xlabel('%s')\n",xaxis);
      fprintf (f,"%s",s); }
   if (strlen(yaxis) > 0) {
      sprintf (s,"ylabel('%s')\n",yaxis);
      fprintf (f,"%s",s); }
   if (strlen(zaxis) > 0) {
      sprintf (s,"zlabel('%s')\n",zaxis);
      fprintf (f,"%s",s); }
   fclose (f); }


/* -------------------------------------------------------------------- */
void addlabel (float x,float y,char *label,char *fname)
/* -------------------------------------------------------------------- */
{
/* Description: Add the string label to the graph in file fname
                at location (x,y).

   On entry:    x     = x coordinate of start of string
                y     = y coordinate of start of string
                label = string to be written to graph
                fname = file name of plot file */
                
   char        r[81],s[81];
   FILE	       *f;	             
   
/* Initialize */

   add_m (fname,r);
   f = fopen (r,"a");   
   if (f == NULL) 
      nerror (102,"addlabel",0,0,fname);
      
/* Write string */

   sprintf (s,"text(%g,%g,'%s')\n",x,y,label);
   fprintf (f,"%s",s);
   fclose (f);
}

/* -------------------------------------------------------------------- */
void 	graphvec (vector y,int n,char *title,char *xaxis,char *yaxis,
                  char *fname) {
/* -------------------------------------------------------------------- */

/* Description: Create a Matlab M-file which graphs the n by 1
                vector y and labels the graph.

   On entry:    y     = n by 1 vector to be graphed
                n     = number of components of y
                title = graph title
                xaxis = x axis label
                yaxis = y axis label
                fname = Matlab M-file name */

   char r[81];                

/* Check arguments */

   if (n < 2)
      nerror (100,"graphvec",2,(float) n,"");
   add_m (fname,r);
      
/* Create M-file */

   putstr0 ("clear y",r,1);
   putvec (y,n,"y",r,0);
   putstr0 ("plot(y)",r,0);      
   putstr1 (title,xaxis,yaxis,r);
   putstr0 ("box on",r,0);
   printf ("\nMatlab plot file %s created.\n",r); 
}


/* -------------------------------------------------------------------- */
void 	graphmat (matrix Y,int m,int n,char *title,char *xaxis,
                  char *yaxis,char *fname) {
/* -------------------------------------------------------------------- */

/* Description: Create a Matlab M-file which graphs the rows of m by n
                matrix Y and labels the graph.


   On entry:    Y     = m by n matrix whose rows are to be graphed
                m     = number of rows of Y 
                n     = number of columns of Y
                title = graph title
                xaxis = x axis label
                yaxis = y axis label
                fname = Matlab M-file name
 
   Notes:       Y is transposed in the M-file */

   char         r[81]; 
   matrix	Y0;

/* Check arguments */

   if (m < 1)
      nerror (100,"graphmat",2,(float) m,"");
   if (n < 2)
      nerror (100,"graphmat",3,(float) n,"");
   add_m (fname,r);
      
/* Create M-file */

   putstr0 ("clear Y",r,1);
   Y0 = mat (n,m,"");
   transpose (Y,m,n,Y0);
   putmat (Y0,n,m,"Y",r,0);
   putstr0 ("plot(Y)",r,0);
   putstr1 (title,xaxis,yaxis,r);
   putstr0 ("box on",r,0);
   printf ("\nMatlab plot file %s created.\n",r); 
   freemat (Y0,n); }
   

/* -------------------------------------------------------------------- */
void 	graphfun (float a,float b,char *title,char *xaxis,
                  char *yaxis,float (*f)(float x),char *fname) {
/* -------------------------------------------------------------------- */

/* Description: Create a Matlab M-file which graphs the function f(x)
                over the interval [a,b] and labels the graph.

   On entry:    a     = lower limit of plot range
                b     = upper limit of plot range
                title = graph title
                xaxis = x axis label
                yaxis = y axis label
                f     = name of user-supplied function to be
                        graphed.  When f is called, it must
                        return the value f(x).
                fname = Matlab M-file name */

   char         r[81];             
   int		i,n=301;
   vector	x = vec(n,""),
		y = vec(n,"");

/* Initialize */		

   add_m (fname,r);
		
/* Create M-file */		
		
   for (i = 1; i <= n; i++) {
      x[i] = a + (i-1)*(b-a)/(n-1);
      y[i] = f(x[i]); }
   putstr0 ("clear x y",r,1);   
   putvec (x,n,"x",r,0);
   putvec (y,n,"y",r,0);
   putstr0 ("plot(x,y)",r,0);      
   putstr1 (title,xaxis,yaxis,r);
   putstr0 ("box on",r,0);
   printf ("\nMatlab plot file %s created.\n",r); 
   freevec (x);
   freevec (y); }

   
/* -------------------------------------------------------------------- */
void 	graphxy (vector x,vector y,int n, char *title,
                 char *xaxis,char *yaxis,char *fname) {
/* -------------------------------------------------------------------- */


/* Description: Create a Matlab M-file which graphs the n by 1
                vector y vs. the n by 1 vector x and labels the graph.

   On entry:    x     = n by 1 vector of independent variables
                y     = n by 1 vector of dependent variables
                n     = number of components of x and y
                title = graph title
                xaxis = x axis label
                yaxis = y axis label
                fname = Matlab M-file name */

   char      r[81];
                
/* Check arguments */

   if (n < 2)
      nerror (100,"graphxy",3,(float) n,"");
   add_m (fname,r);
      
/* Create M-file */
      
   putstr0 ("clear x y",r,1);
   putvec (x,n,"x",r,0);
   putvec (y,n,"y",r,0);
   putstr0 ("plot(x,y)",r,0);      
   putstr1 (title,xaxis,yaxis,r);
   putstr0 ("box on",r,0);
   printf ("\nMatlab plot file %s created.\n",r); 
}


/* -------------------------------------------------------------------- */
void 	graphxm (vector x,matrix Y,int m,int n, char *title,
                 char *xaxis,char *yaxis,char *fname) {
/* -------------------------------------------------------------------- */

/* Description: Create a Matlab M-file which graphs the rows of the
                m by n matrix Y vs. the n by 1 vector x and labels
                the graph.

   On entry:    x     = n by 1 matrix of independent variables
                Y     = m by n matrix of dependent variables
                m     = number of rows of Y
                n     = number of columns of Y and components of x 
                title = graph title
                xaxis = x axis label
                yaxis = y axis label
                fname = Matlab M-file name 

   Notes:       Y is transposed in the M-file. */

   char         r[81];
   matrix	Y0;

/* Check arguments */
   
   if (m < 1)
      nerror (100,"graphxm",3,(float) m,"");
   if (n < 2)
      nerror (100,"graphxm",4,(float) n,"");
   add_m (fname,r);
      
/* Create M-file */

   putstr0 ("clear x Y",r,1);
   Y0 = mat (n,m,"");
   transpose (Y,m,n,Y0);
   putvec (x,n,"x",r,0);
   putmat (Y0,n,m,"Y",r,0);
   putstr0 ("plot(x,Y)",r,0);
   putstr1 (title,xaxis,yaxis,r);
   putstr0 ("box on",r,0);
   printf ("\nMatlab plot file %s created.\n",r); 
   freemat (Y0,n);
}

/* -------------------------------------------------------------------- */
void 	graphmm (matrix X,matrix Y,int m,int n,char *title,
                 char *xaxis,char *yaxis,char *fname,int dm) 
/* -------------------------------------------------------------------- */
{
/* Description: Create a Matlab M-file which graphs the rows of the
                m by n matrix Y vs. the rows of the m by n matrix X
                and labels the graph.

   On entry:    X     = m by n matrix of independent variables
                Y     = m by n matrix of dependent variables
                m     = number of rows of X and Y
                n     = number of columns of X and Y
                title = graph title
                xaxis = x axis label
                yaxis = y axis label
                fname = Matlab M-file name
                dm    = display mode. Normally, the points are
                        connected with solid straight line segments.
                        If dm != 0, then the last row of Y is
                        graphed using isolated plot symbols drawn
                        at each point. 

   Notes:       1) X and Y are transposed in the M-file */

   char       r[81],s[81],
              *colors = "bgrcmy";
   int        i,j;
   matrix     X0,Y0;
   
/* Check arguments */
   
   if (m < 1)
      nerror (100,"graphmm",3,(float) m,"");
   if (n < 2)
      nerror (100,"graphmm",4,(float) n,"");
   add_m (fname,r);
      
/* Create M-file */

   putstr0 ("clear X Y",r,1);
   X0= mat (n,m,"");
   Y0= mat (n,m,"");
   transpose (X,m,n,X0);
   transpose (Y,m,n,Y0);
   putmat (X0,n,m,"X",r,0);
   putmat (Y0,n,m,"Y",r,0);

/* Check for plot type */   

   if (dm != 0)
   {
      putstr0 ("hold on",r,0);
      for (i = 1; i < m; i++)
      {
         j = (i-1) % 6;
         sprintf (s,"plot(X(:,%i),Y(:,%i),'%c')",i,i,colors[j]);
         putstr0 (s,r,0);
      }
      j = (m-1) % 6;
      sprintf (s,"plot(X(:,%i),Y(:,%i),'%cs')",m,m,colors[j]);
      putstr0 (s,r,0);
      putstr0 ("hold off",r,0);
   }     
   else
      putstr0 ("plot(X,Y)",r,0);
   putstr1 (title,xaxis,yaxis,r);
   putstr0 ("box on",r,0);
   printf ("\nMatlab plot file %s created.\n",r); 
   freemat (X0,n);
   freemat (Y0,n);
}

/* -------------------------------------------------------------------- */
void 	plotmat (matrix Z,int m,int n,char *title,char *xaxis,
                 char *yaxis,char *zaxis,char *fname) {
/* -------------------------------------------------------------------- */

/* Description: Create a Matlab M-file which plots the m by n matrix Z
                as a surface using the row subscript as one independent
                variable and the column subscript as the other
                independent variable, and lable the plot.

   On entry:    Z     = m by n matrix to be plotted.
                m     = number of rows of Z
                n     = number of columns of Z
                title = graph title
                xaxis = x axis label
                yaxis = y axis label
                zaxis = z axis label
                fname = Matlab M-file name 


   Note:        Z is transposed in the M-file */
   
   char        r[81];
   matrix      Z0;

/* Check arguements */

   if (m < 2)
      nerror (100,"plotmat",2,(float) m,"");
   if (n < 2)
      nerror (100,"plotmat",3,(float) n,"");
   add_m (fname,r);
       
/* Create M-file */
 
   putstr0 ("clear Z",r,1);
   Z0 = mat(n,m,"");
   transpose (Z,m,n,Z0);
   putmat (Z0,n,m,"Z",r,0);
   putstr0 ("mesh(Z)",r,0);
   putstr2 (title,xaxis,yaxis,zaxis,r);
   printf ("\nMatlab plot file %s created.\n",r); 
   freemat (Z0,n); }

   
/* -------------------------------------------------------------------- */
void 	plotfun (float x0,float x1,float y0,float y1,
                 char *title,char *xaxis,char *yaxis,
                 char *zaxis,float (*f)(float x,float y),
                 char *fname) {
/* -------------------------------------------------------------------- */

/* Description: Create a Matlab M-file which plots the function f(x,y)
                as a surface over the rectanglar region [x0,x1] by
                [y0,y1], and labels the plot.

   On entry:    x0    = lower limit of x
                x1    = upper limit of x
                y0    = lower limit of y
                y1    = upper limit of y
                title = graph title
                xaxis = x axis label
                yaxis = y axis label
                zaxis = z axis label
                f     = name of user-supplied function to be
                        graphed.  When f is called, it must
                        return the value f(x,y).
                fname = Matlab M-file name */

   char         r[81]; 
   int		i,j,n=61;
   vector	x = vec (n,""),
		y = vec (n,"");
   matrix	Z = mat (n,n,"");

/* Initialize */

   add_m (fname,r); 

/* Create M-file */

   for (i = 1; i <= n; i++) {
      x[i] = x0 + (i-1)*(x1-x0)/(n-1);
      y[i] = y0 + (i-1)*(y1-y0)/(n-1); }
   for (i = 1; i <= n; i++) 
      for (j = 1; j <= n; j++)
         Z[j][i] = f(x[i],y[j]);

   putstr0 ("clear x y Z",r,1);
   putvec (x,n,"x",r,0);
   putvec (y,n,"y",r,0);
   putmat (Z,n,n,"Z",r,0);
   putstr0 ("mesh(x,y,Z)",r,0);
   putstr2 (title,xaxis,yaxis,zaxis,r);
   printf ("\nMatlab plot file %s created.\n",r); 
}

/* -------------------------------------------------------------------- */
void 	plotxyz (vector x,vector y,matrix Z,int m,int n,char *title,
                 char *xaxis,char *yaxis,char *zaxis, char *fname) {
/* -------------------------------------------------------------------- */

/* Description: Create a Matlab M-file which plots the m by n matrix Z
                as a surface using the m by 1 vector x as the first
                independent variable and n by 1 vector y as the second
                independent variable, and lable the plot.

   On entry:    x     = m by 1 vector of independent variables
                y     = n by 1 vector of independent variables 
                Z     = m by n matrix to of dependent variables
                m     = number of rows of Z
                n     = number of columns of Z
                title = graph title
                xaxis = x axis label
                yaxis = y axis label
                zaxis = z axis label
                fname = Matlab M-file name

   Note:        Z is transpose in the M-file */

   char       r[81];
   matrix     Z0;

/* Check arguements */

   if (m < 2)
      nerror (100,"plotxyz",4,(float) m,"");
   if (n < 2)
      nerror (100,"plotxyz",5,(float) n,"");
   add_m (fname,r);
      
/* Create M-file */
 
   putstr0 ("clear x y Z",r,1);
   Z0 = mat(n,m,"");
   transpose (Z,m,n,Z0);
   putvec (x,m,"x",r,0);
   putvec (y,n,"y",r,0);
   putmat (Z0,n,m,"Z",r,0);
   putstr0 ("mesh(x,y,Z)",r,0);
   putstr2 (title,xaxis,yaxis,zaxis,r);
   printf ("\nMatlab plot file %s created.\n",r); 
   freemat (Z0,n); }

