/* ------------------------------------------------------------------- */
/* File:     dsp.c

   Contents: Digital signal processing functions

   Uses:     nlib.c, linear.c */
/* ------------------------------------------------------------------- */


/*---------------------------------------------------------------------*/
void dft (cvector z,int n,int d) {
/*---------------------------------------------------------------------*/

/* Description: Compute the one-dimensional fast Fourier Transform (FFT)
                or inverse FFT of a complex signal.

   On entry:    z = n by 1 complex vector containing samples
                n = number of samples (positive power of 2)    
                d = a direction code defined as follows:

                       d   Operation
                      ------------------------------
                      +1    Compute FFT of z
                      -1    Compute inverse FFT of z
                      ------------------------------

   On exit:     z = n by 1 complex vector containing transformed
                    samples */

   int		i=1,k,m,r,i0,i1;
   complex	w,u,t;
   float	theta;	
   
/* Initialize */

   if      (n < 2)
      nerror (100,"dft",2,(float) n,"");
   else if ((d != -1) && (d != 1))
      nerror (100,"dft",3,(float) d,"");
   while (i < n) 
      i *= 2;
   if (i > n)
      nerror (300,"dft",0,0,"");
   theta = -d*2*PI/n;
   r = n/2;
   
/* Compute FFT by decomposition */

   for (i = 1; i < n; i *= 2) {
      w = cnum ((float) cos(i*theta),(float) sin(i*theta));
      for (k = 0; k < n; k += 2*r) {
         u = cnum (1,0);
         for (m = 0; m < r; m++) {
            i0 = k + m + 1;
            i1 = i0 + r;
            t     = csub (z[i0],z[i1]);
            z[i0] = cadd (z[i0],z[i1]); 
            z[i1] = cmul (t,u);
            u     = cmul (u,w); } }
      r /= 2; }

/* Sort the results using bit reversal */
   
   for (i = 0; i < n; i++) {
      r = i;
      k = 0;
      for (m = 1; m < n; m *= 2) {
         k =  2*k + (r % 2);
         r /= 2; }
      if (k > i) {   
         t      = z[i+1];
         z[i+1] = z[k+1];
         z[k+1] = t; } }

/* Normalize if needed */

   if (d < 0) {
      for (i = 1; i <= n; i++) {
         z[i].x /= n;
         z[i].y /= n; } } }

/*---------------------------------------------------------------------*/
void   dft2      (cmatrix Z,int m,int n,int d) {
/*---------------------------------------------------------------------*/

/* Description: Compute the two-dimensional fast Fourier Transform (FFT)
                or inverse FFT of a complex signal.

   On entry:    Z = m by n complex matrix containing samples
                m = number of rows of Z (postive power of 2) 
                n = number of columns of Z (positive power of 2)    
                d = a direction code defined as follows:

                       d   Operation
                      ------------------------------
                      +1    Compute FFT of Z
                      -1    Compute inverse FFT of Z
                      ------------------------------

   On exit:     Z = m by n complex matrix containing transformed
                    samples */

   cmatrix	Y;
   cvector	y,z;         
   int		k,i;

/* Initialize */

   if      (m < 2)
      nerror (100,"dft2",2,(float) m,"");
   else if (n < 2)
      nerror (100,"dft2",3,(float) n,"");
   else if ((d != 1) && (d != -1))
      nerror (100,"dft2",2,(float) d,"");
   y = cvec (n,""); 
   z = cvec (n,""); 
   Y = cmat (n,m,"");

/* Compute m FFTs of length n */          
   
   for (k = 1; k <= m; k++) {
      for (i = 1; i <= n; i++)
         y[i] = Z[k][i];
      dft (y,n,d);
      for (i = 1; i <= n; i++)
         Y[i][k] = y[i]; }
         
/* Compute n FFTs of length m */          
   
   for (i = 1; i <= n; i++) {
      for (k = 1; k <= m; k++)
         z[k] = Y[i][k];
      dft (z,m,d);
      for (k = 1; k <= m; k++)
         Y[i][k] = z[k]; }

/* Finalize */

   for (k = 1; k <= m; k++)
      for (i = 1; i <= n; i++)
         Z[k][i] = Y[i][k];
   freecvec (y);
   freecvec (z);
   freecmat (Y,n); }

/*---------------------------------------------------------------------*/
void spectra (vector x,int n,float fs,vector a,vector phi,vector f) {
/*---------------------------------------------------------------------*/

/* Description: Compute the amplitude and phase spectra of a real
                discrete-time signal.

   On entry:    x  = n by 1 vector containing the discrete-time signal 
                n  = number of samples (positive power of 2)
                fs = sampling frequency in Hz
                
   On exit:     a   = n by 1 vector containing magnitude spectrum of x
                phi = n by 1 vector containing phase spectrum of x  
                f   = n by 1 vector containing discrete frequencies */  
                
   int		i = 1;
   float	em = epsilon();             
   cvector	z;         
   
/* Initialize */

   if (n < 2)
      nerror (100,"spectra",2,(float) n,"");
   while (i < n) 
      i *= 2;
   if (i > n)
      nerror (300,"spectra",0,0,"");
      
/* Compute complex FFT using Hanning window on x */      	 

   z = cvec (n,"");
   for (i = 1; i <= n; i++)
   {
      z[i] = cnum (x[i],0);
      f[i] = (i-1)*fs/n; 
   }
   dft (z,n,1);  
   
/* Compute magnitude and phase */   

   for (i = 1; i <= n; i++)
   {
      a[i] = cmag (z[i]);
      if (fabs(a[i]) > em)
         phi[i] = (float) atan2(z[i].y,z[i].x);
      else
         phi[i] = 0;
   }
   freecvec (z); }

/*---------------------------------------------------------------------*/
void crosscor (vector x,vector y,int n,int m,vector rho) {
/*---------------------------------------------------------------------*/

/* Description: Compute the normalized cross correlation of two
                discrete-time signals.

   On entry:    x = n by 1 vector containing samples of first signal
                y = m by 1 vector containing samples of second signal
                n = number of samples in x (positive power of 2)
                m = number of samples in y (positive power of 2,
                    m <= n)

   On exit:     rho = n by 1 vector containing cross correlation of x 
                      with y.

   Notes:       The auto correlation of a signal x is just the cross
                correlation of x with x.  */

   int		i;
   float 	a = 0,
		b = 0;
   cvector	X,Y;
   
/* Initialize */

   if      (n < 2)
      nerror (100,"crosscor",3,(float) n,"");
   else if ((m < 2) || (m > n))
      nerror (100,"crosscor",4,(float) m,""); 
   X = cvec (n,"");
   Y = cvec (n,"");

/* Compute FFTs of x(k) and y(k) */    
   
   for (i = 1; i <= n; i++) {
      X[i].x = x[i]; 	
      if (i <= m)
         Y[i].x = y[i]; }
   dft (X,n,1);
   dft (Y,n,1);

/* Compute IFFT of X(m)Y*(m) */

   for (i = 1; i <= n; i++) {
      Y[i].y = -Y[i].y;
      X[i]   = cmul (X[i],Y[i]); }
   dft (X,n,-1);

/* Normalize */
   
   for (i = 1;i <= n; i++) {
      a += x[i]*x[i];
      if (i <= m)
         b += y[i]*y[i]; }
   a = (float) (sqrt(a)*sqrt(b));
   for (i = 1;i <= n; i++)
      rho[i] = X[i].x/a;

/* Finalize */

   freecvec (X);
   freecvec (Y); }

/*---------------------------------------------------------------------*/
void   convolve  (vector x,vector y,int n,vector z) {
/*---------------------------------------------------------------------*/

/* Description: Compute the convolution of two discrete-time signals.

   On entry:    x = n by 1 vector containing samples of first signal
                y = n by 1 vector containing samples of second signal
                n = number of samples in x and y (positive power of 2)

   On exit:     z = n by 1 vector containing convolution of x 
                    with y. */

   cvector	X,Y;
   int		i;
   
/* Initialize */

   if (n < 2)
      nerror (100,"convolve",3,(float) n,"");
   X = cvec (n,"");
   Y = cvec (n,"");

/* Compute FFTs of x(k) and y(k) */    
   
   for (i = 1; i <= n; i++) {
      X[i].x = x[i]; 	
      Y[i].x = y[i]; }
   dft (X,n,1);
   dft (Y,n,1);

/* Compute IFFT of X(m)Y(m) */

   for (i = 1; i <= n; i++) 
      X[i] = cmul (X[i],Y[i]); 
   dft (X,n,-1);

/* Finalize */

   for (i = 1;i <= n; i++)
      z[i] = X[i].x;
   freecvec (X);
   freecvec (Y); }

/*---------------------------------------------------------------------*/
void   fir    (float (*g)(float f),int p,vector b) {
/*---------------------------------------------------------------------*/

/* Description: Design a linear-phase FIR digital filter.  Use
                the Fourier series method with a Hamming window
                to reduce the side lobes.

   On entry:    g = name of function specifies desired magntide
                    response. The prototype of g is:

                    float g (float f);

                    When g is called with input freqency f, it must
                    return the desired filter gain g(f).  The frequency
                    f is expressed as a fraction of the Nyquist
                    frequency which means 0 <= f <= 1. 
                 p  = filter order (p >= 0)
 
   On exit:      b = (2p + 1) by 1 filter coefficient vector.  The
                     FIR filter output is:

                     y(k) = b[1]*u[k] + b[2]*u[k-1] +...+
                            b[2p+1]*u[k-2p] */
 
   int		n = 100,
                k,i;
   float	h,d;
   vector       c;

/* Initialize */

   if (p < 0)
      nerror (100,"fir",3,(float) p,"");
   c = vec (p,"");
   h = 1.0f/n;
   d = 1.0f/(n-1);
      
/* Compute filter coefficients using Simpson integration */

   for (i = 1; i <= n; i++)
      b[p+1] += g((i-1)*d);
   b[p+1] /= n;
   for (k = 1; k <= p; k++)
   {
      c[k] = g(0) + (float) cos(PI*k)*g(1);
      for (i = 1; i < n; i++)
         if (i % 2)
            c[k] = (float) (c[k] + 4*cos(PI*k*(i*h))*g(i*h));
         else
            c[k] = (float) (c[k] + 2*cos(PI*k*(i*h))*g(i*h)); 
      c[k] = h*c[k]/3;
   }

/* Add Hamming window */

   for (k = 1; k <= p; k++)
   {
      b[p+1+k] = (float) (0.54 + 0.46*cos(k*PI/p))*c[k]; 
      b[p+1-k] = b[p+1+k];
   }
   freevec (c);
}

/* ----------------------------------------------------------------- */
void freqrsp (vector u,vector y,int n,float fs,vector a,
               vector phi,vector f) {
/* ----------------------------------------------------------------- */

/* Description: Compute the frequency response of a discrete-time
                system from its input and output.

   On entry:    u  = n by 1 vector containing the input signal
                y  = n by 1 vector containing the output signal
                n  = number of samples (positive power of 2)
                fs = sampling frequency in Hz
                
   On exit:     a   = n by 1 vector containing magnitude response
                phi = n by 1 vector containing phase response
                      in degrees.
                f   = n by 1 vector containing discrete frequencies */

   int		i;
   float 	em = epsilon(),
    		q;
   vector	c,d;         

/* Initialize */

   if (n < 2)
      nerror (100,"freqrsp",3,(float) n,"");
   q = PI/180;
   c = vec (n,"");
   d = vec (n,"");

/* Compute magnitude and phase */

   spectra (u,n,fs,c,d,f);
   spectra (y,n,fs,a,phi,f); 
   for (i = 1; i <= n; i++) {
      a[i]   /= MAX(c[i],10*em);
      phi[i] = (phi[i] - d[i])/q;
      while (phi[i] < -180.0) phi[i] += 360.0;
      while (phi[i] >  180.0) phi[i] -= 360.0; }
   freevec (c);
   freevec (d); }

/* ------------------------------------------------------------------- */
float arma (float u,vector theta,vector x,int n,int m) {
/* ------------------------------------------------------------------- */

/* Description: Compute the output of an auto-regressive moving-
                average (ARMA) model having the following transfer
                function:

                Y(z)      theat[n+1] + ... + theta[n+m+1]*pow(z,-m)
                ---- = -----------------------------------------------
                U(z)   1 + theta[1]*pow(z,-1) +...+ theta[n]*pow(z,-n)   

   On entry:    u     = system input
                theta = (n+m+1) by 1 vector of system parameters:
                        [a1,...,an,b0,...,bm]
                x     = (n+m+1) by 1 vector containing system state:
                        [-y[k-1],...,-y[k-n],u[k],u[k-1],...,u[k-m]]
                n     = number of past outputs used (n >= 0)
                m     = number of past inputs used (m >= 0)

   On exit:     x     = (n+m+1) \times 1 vector containing updated
                        system state with x[n+1] = 0. 

   Returned:    y = the system output.  */
                       
   int		i,r;
   float	y = 0;	

   if      (n < 0)
      nerror (100,"arma",4,(float) n,"");
   else if (m < 0)
      nerror (100,"arma",5,(float) m,"");
   r = n + m + 1;
   
/* Compute y */

   x[n+1] = u;
   for (i = 1; i <= r; i++) 
      y += theta[i]*x[i];

/* Update x */

   for (i = r; i > 1; i--)
      x[i] = x[i-1];
   if (n > 0)
      x[1] = -y;
   x[n+1] = 0;  
   return y; } 

/* ----------------------------------------------------------------- */
float getarma (vector u,vector y,int p,int n,int m,vector theta) {
/* ----------------------------------------------------------------- */

/* Description: Identify the parameters an auto-regressive moving-
                average (ARMA) model having the following transfer
                function:

                Y(z)      theat[n+1] + ... + theta[n+m+1]*pow(z,-m)
                ---- = -----------------------------------------------
                U(z)   1 + theta[1]*pow(z,-1) +...+ theta[n]*pow(z,-n)   

   On entry:    u = p by 1 vector containing input samples
                y = p by 1 vector containing output samples
                p = number of samples (p >= n+m+1)
                n = number of past outputs used (n >= 0)
                m = number of past inputs used (m >= 0)

   On exit:     theta = (n+m+1) by 1 vector containing ARMA model
                        parameters.
            
   Returned:    r = Euclidean norm of residual error vector

   Notes:       1. The input u must be rich in frequency content
                2. Use the function arma to compute the output of
                   ARMA model. */

   matrix	X;         
   float 	e=0,r;
   int	        q,i,j;

/* Initialize */

   if      (n < 0)
      nerror (100,"getarma",4,(float) n,"");
   else if (m < 0)
      nerror (100,"getarma",5,(float) m,"");
   q = n + m + 1;
   if (p < q)
      nerror (302,"getarma",1,0,"");
   X = mat (p,q,"");
  
/* Compute state matrix X */

   for (i = 1; i <= p; i++) {
      for (j = 1; j <= n; j++)
         if (j < i)
            X[i][j] = -y[i-j];
      for (j = 0; j <= m; j++)
         if (j < i)
            X[i][n+1+j] = u[i-j]; }
      
/* Compute the least-squares estimate, theta */ 

   i = gensys (X,y,p,q,theta);
   if (!i) 
      nerror (303,"getarma",1,0,"");
   for (i = 1; i <= p; i++) {
      r = y[i];	
      for (j = 1; j <= q; j++)
         r -= X[i][j]*theta[j];
      e += r*r; }
   
/* Finalize */   

   freemat (X,p);
   return (float) sqrt(e); }

/* ------------------------------------------------------------------ */
float lms (float u,float y,vector x,vector theta,float mu,int m) {
/* ------------------------------------------------------------------ */

/* Description: Use the current input samples u and output sample y to
                update the (m+1) by 1 parameter vector theta of a MA
                model using the least-mean square (LMS) method.  

   On entry:    u     = current value of system input
                y     = current value of system output
                x     = (m+1) by 1 vector containing current state of
                        MA model: [u(k),u(k-1),...,u(k-m)].
                theta = (m+1) by 1 vector containing estimated values
                        of MA model parameters.

                mu    = step size used for stepest descent search
                        (mu > 0)
                m     = order of MA model (m >= 0)

   On exit:     x     = (m+1) by 1 vector containing updated state of
                        MA model with x[1] = 0.
		theta = (m+1) by 1 vector containing updated estimate of
                        of MA model parameters.

   Returned:    e = error between actual output y and output predicted
                    by MA moded, e = y - x'*theta  
   
   Notes:       For the LMS method to converge, the step size mu must
                not be too large.  If P(u) denotes the average power of
                the input signal u(k), then the step size should be in
                the range:

                   0 < mu < 1/[(m+1)*P(u)] */
                  
   int		i;
   float	e;
   vector	x1;

/* Initialize */

   if      (mu <= 0)
      nerror (100,"lms",5,mu,"");
   else if (m < 0)
      nerror (100,"lms",6,(float) m,"");
   x1 = vec (m+1,"");

/* Update theta */

   copyvec (x,m+1,x1);			/* save original x */
   x1[1] = u;				/* update first element */
   e = y - arma(u,theta,x,0,m);
   for (i = 1; i <= m+1; i++) 
      theta[i] += 2*mu*e*x1[i];
   freevec (x1);
   return e; }

