/* ------------------------------------------------------------------- */
/* File:     eigen.c 

   Contents: Eigenvalue and eigenvector functions

   Uses:     nlib.c, linear.c 
/* ------------------------------------------------------------------- */

/* ------------------------------------------------------------------- */
/* Local functions */
/* ------------------------------------------------------------------- */
                                   
static void    identity  (matrix A,int n);
static void    getmax    (matrix A,int n,int *p,int *q);
static void    eig2      (matrix A,int k,cvector q);

/* ------------------------------------------------------------------- */
void leverrier (matrix A,int n,vector a) {
/* ------------------------------------------------------------------- */

/* Description: Compute the coefficients of the characteristic
                polynomial of matrix A using Leverrier's method.

   On entry:    A = n by n matrix
                n = dimension of A (n >= 1)

   On exit:     a = (n+1) by 1 vector containing the coefficients
                of the characteristic polynomial of A:

                 det(sI - A) = a[1] + a[2]*s + ... + a[n]*pow(s,n-1) +
                               pow(s,n)

   Notes:        The dimension of a is one larger than the dimension of
                 size of A with a[n+1] = 1; */
            
   matrix	B; 
   int		i,k;

/* Initialize */

   if (n < 1)
      nerror (100,"leverrier",2,(float) n,"");
   B = mat (n,n,"");
   copymat (A,n,n,B);
   zerovec (a,n+1);
   a[n+1] = 1;
   for (i = 1; i <= n; i++)
      a[n] -= B[i][i];
      
/* Compute remaining coefficients */

   for (k = n-1; k >= 1; k--) {
      for (i = 1; i <= n; i++)
         B[i][i] += a[k+1];
      matmat (A,B,n,n,n,B);
      for (i = 1; i <= n; i++)
         a[k] -= B[i][i];
      a[k] /= (n-k+1); }

/* Finalize */

   freemat (B,n); }
   
/* ------------------------------------------------------------------- */
int powereig (matrix A,int n,float eps,int m,int d,float *c,vector x) {
/* ------------------------------------------------------------------- */

/* Description: Apply the power method to find the dominant eigenvalue
                of A and its eigenvector, or apply the inverse power
                method to find the least dominant eigenvalue of A and
                its eigenvector.

   On entry:    A   = n by n matrix 
                n   = dimension of A (n >= 1)
                eps = error tolerance used to terminate the
                      search (eps >= 0)
                m   = maximum number of iterations (m >= 1)
                d   = an operation code defined as follows:

                    d        Operation
                    -------------------------
                    1    power method
                   -1    inverse power method   
                    -------------------------

   On exit:    *c   = dominant eigenvalue of A if p = 1) or
                      least dominant eigenvalue of A if p = -1.
                x   = n by 1 eigenvector associated with the
                      eigenvalue *c

   Returns:     The number of iterations performed.  If it is
                positiave and less than the user-specified maximum
                m, then the following convergence criterion was
                satisfied where r = (*cI-A)x:

                   ||r|| < eps

    Notes:      If p = -1, then A must be nonsingular.  When A is
                singular, the least dominant eigenvalue is *c = 0. */

   int		k = 0,
   		i;
   float 	em = epsilon(),
		s,t;
   vector	y,r,p;        
   matrix	Q;
   
/* Initialize */

   if      (n < 1)
      nerror (100,"powereig",2,(float) n,"");
   else if (eps < 0)
      nerror (100,"powereig",3,eps,"");
   else if (m < 1)
      nerror (100,"powereig",4,(float) m,"");
   r = vec (n,"");
   y = vec (n,"");
   if (vecnorm(x,n) > em)
      copyvec (x,n,y);
   else
      for (i = 1; i <= n; i++)
         y[i] = randu (-1,1);   
   if (d >= 1) {

/* Power method */

      matvec (A,y,n,n,x);
      do {
         s = vecnorm (x,n);
         for (i = 1; i <= n; i++) 
            y[i] = x[i]/s;
         matvec (A,y,n,n,x);
         *c = inner(y,x,n)/inner(y,y,n);
         for (i = 1; i <= n; i++)
            r[i] = (*c)*y[i] - x[i];
         t = vecnorm (r,n);
         k++;
         #if defined(DISPLAY_EIGEN)
            printf ("\n%i & %.7f",k,*c);
            for (i = 1; i <= n; i++)
               printf (" & %.7f",y[i]);
            printf (" & %.7f \\\\",t);
         #endif   
         }
      while ((t > eps) && (k < m)); }
      
   else {

/* Inverse power method */

      if (fabs(det(A,n)) < em) {		/* singular A */
         *c = 0;
         k = 0;
         printf ("\nThe matrix in powereig is singular.\n");
         wait(); }
      else {  					/* nonsingular A */
         Q = mat (n,n,"");
         p = vec (n,""); 
         lufac (A,n,Q,p);
         lusub (Q,p,y,n,x);
         do {
            s = vecnorm (x,n);
            for (i = 1; i <= n; i++) 
               y[i] = x[i]/s;
            lusub (Q,p,y,n,x);  
            *c = inner(y,x,n)/inner(y,y,n);
            for (i = 1; i <= n; i++)
               r[i] = (*c)*y[i] - x[i];
            t = vecnorm(r,n);
            k++; 
            #if defined(DISPLAY_EIGEN)
               printf ("\n%i & %.7f",k,*c);
               for (i = 1; i <= n; i++)
                  printf (" & %.7f",y[i]);
               printf (" & %.7f \\\\",t);
            #endif
         }  
         while ((t > eps) && (k < m)); 
         *c = 1/(*c);
         freemat (Q,n);
         freevec (p); } }
   
/* Finalize */

   #if defined(DISPLAY_EIGEN)
      wait();
   #endif
   copyvec (y,n,x);
   freevec (r);
   freevec (y);
   return k; }

/* ------------------------------------------------------------------- */
void house (matrix A,int n,matrix B,matrix Q) {
/* ------------------------------------------------------------------- */

/* Description: Convert the n by n matrix A to upper-Hessenberg form
                using the Householder transformations.

   On entry:    A = n by n matrix
                n = dimension of A (n >= 1)

   On exit:	B = n by n upper-Hessenberg version of A.  B is zero
                    below the subdiagonal.  If A is symmetric, then
                    B is tridiagonal.
                Q = n by n orthogonal transformation matrix used to
                    find B using B = Q'AQ.

   Notes:       house is used to as a preprocessing step by
                Jacobi and QR to speed convergence. */
                   
   int		i,j,k;
   float	beta = 0,
   		em = epsilon(),
		alpha;
   vector	u;         
   matrix	P;
            
/* Initialize, check elements below subdiagonal */

   if (n < 1)
      nerror (100,"house",2,(float) n,""); 
   copymat (A,n,n,B);
   identity (Q,n);
   if (n < 3) return;
   for (k = 1; k <= n-2; k++)
      for (j = k+2; j <= n; j++)
         beta += (float) fabs(B[k+2][k]);
   if (beta < em)			
      return;   
   u = vec (n,"");
   P = mat (n,n,"");
   
/* Perform Householder transformations */   

   for (k = 1; k <= n-2; k++) {

/* Compute alpha */ 

      alpha = 0;
      for (j = k+1; j <= n; j++)
         alpha += B[j][k]*B[j][k];
      alpha = SGN(B[k+1][k])*((float) sqrt(alpha));
      
/* Compute u */      
      
      zerovec (u,n);
      for (j = k+1; j <= n; j++)
         u[j] = B[j][k];
      u[k+1] += alpha;

/* Compute P */
      
      identity (P,n);
      beta = (float) 0.5*inner(u,u,n);   
      for (i = 1; i <= n; i++)
         for (j = 1; j <= n; j++)
            P[i][j] -= u[i]*u[j]/beta;
            
/* Update Q and B */
            
      matmat (Q,P,n,n,n,Q);
      matmat (B,P,n,n,n,B);
      matmat (P,B,n,n,n,B); }

/* Finalize */

   freemat (P,n);
   freevec (u); }

/* ------------------------------------------------------------------- */
int jacobi (matrix A,int n,int m,float eps,vector c,matrix P) {
/* ------------------------------------------------------------------- */

/* Description: Find the eigenvalues and eigenvectors of a symmetric
                n by n matrix A using Jacobi's iterative method.

   On entry:    A   = n by n matrix
    		n   = dimension of A (n >= 1)
    		m   = maximum number of iteratins (m >= 1)
    		eps = error tolerance used to terminate search (eps >= 0)

   On exit: 	c   = n by 1 vectors containing eigenvalues ordered
                      by decreasing magnitude
                P   = n by n matrix containing eigenvector.  The kth
                      eigenvector is in the kth column of P

   Returned:    The number of iterations performed.  If it is less than
                the user-specified maximum, m, then A was successfully
                converted to a diagonal matrix with elements off the
                diagonal having magnitudes less than eps. */

   int		r = 0,
		i,j,k,p,q;
   float	em = epsilon(),
		phi,cphi,sphi,a,b;
   matrix	B,Q;            
   
/* Initialize */

   if      (n < 1)
      nerror (100,"jacobi",3,(float) n,"");
   else if (m < 1)
      nerror (100,"jacobi",4,(float) m,"");
   for (i = 1; i <= n; i++)
      for (j = i+1; j <= n; j++)
         if (fabs(A[i][j] - A[j][i]) > em/100)
            nerror (306,"jacobi",0,0,"");
   if (n == 1) {
      c[1] = A[1][1];
      P[1][1] = 1;
      return 0; }
   identity (P,n);
   B = mat (n,n,"");
   Q = mat (n,n,"");
   house (A,n,B,Q);
   
/* Perform iterations to diagonalize B */        

   getmax (B,n,&p,&q);
   while ((fabs(B[p][q]) >= eps) && (r < m)) {

/* Compute phi */

      phi = 0.5f*((float) atan2(2*B[p][q],B[p][p]-B[q][q]));
      cphi = (float) cos (phi);
      sphi = (float) sin (phi);
      
/* Compute P and B = BP */      
      
      for (i = 1; i <= n; i++) {
      	 a = P[i][p];
      	 b = P[i][q];
      	 P[i][p] =  cphi*a + sphi*b;
      	 P[i][q] = -sphi*a + cphi*b;
      	 a = B[i][p];
      	 b = B[i][q];
      	 B[i][p] =  cphi*a + sphi*b;
      	 B[i][q] = -sphi*a + cphi*b; }

/* Compute B = P'BP */
      	 
      for (j = 1; j <= n; j++) {
      	 a = B[p][j];
      	 b = B[q][j];
         B[p][j] =  cphi*a + sphi*b;
         B[q][j] = -sphi*a + cphi*b; }
	 
/* Find largest element */
      
      getmax (B,n,&p,&q);
      r++; }

/* Sort eigenvalues and eigenvectors */

   for (i = 1; i <= n; i++)
      c[i] = B[i][i];
   matmat (Q,P,n,n,n,P);
   for (i = 1; i <= n; i++)
      for (j = i; j <= n; j++) 
         if (fabs(c[j]) > fabs(c[i])) {
            a = c[i];
            c[i] = c[j];
            c[j] = a;
            for (k = 1; k <= n; k++) {
               a = P[k][i];
               P[k][i] = P[k][j];
               P[k][j] = a; } } 	 

/* Finalize */

   freemat (Q,n);
   freemat (B,n);
   return r; }

/* ------------------------------------------------------------------- */
int	    qr (matrix A,int n,int m,float eps,cvector c) {
/* ------------------------------------------------------------------- */

/* Description: Find the eigenvalues of the n by n matrix A using
                the QR iterative method.

    On entry:   A   = n by n matrix,
                n   = dimension of A (n >= 1)
                m   = maximum number of iterations (m >= 1)
                eps = error tolerance used to terminate search (eps >= 0)

    On exit:    c   = n by 1 vectors containing eigenvalues ordered
                      by descreasing magnitudes
                      
    Returned:   The number of iterations performed.  If it is less
                that the user-specified maximum, m, then A was
                successfully converted to an upper block triangular
                matrix the entries below the subdiagonal having
                magnitudes less than eps. */

   int		p = 0,
	    	i,j,k,h,s;
   float	dr = 0.01f,
		a,b,cphi,sphi,d,r;
   vector	u;         
   cvector	q;
   matrix	Q,B;
   
/* Initialize */

   if      (n < 1)
      nerror (100,"qr",2,(float) n,"");
   else if (m < 1)
      nerror (100,"qr",3,(float) m,"");
   else if (eps <0)
      nerror (100,"qr",4,eps,"");
    switch (n) {
      case 1:
         c[1].x = A[1][1];
         c[1].y = 0;
         return 0;
      case 2:
         eig2 (A,1,c);
         return 0; }
   Q = mat (n,n,"");
   B = mat (n,n,"");
   u = vec (n,"");
   q = cvec (2,"");
   house (A,n,B,Q);
   s = n;
   #if defined(DISPLAY_EIGEN)
      showmat ("house",B,n,n,0);
   #endif
   
/* Transform B to upper block triangular form */        

   while ((s > 1) && (p <= m)) {

/* Shift eigenvalues: B = B - rI */

      eig2 (B,s-1,q);
      k = (fabs(q[1].x-B[s][s]) < fabs(q[2].x-B[s][s])) ? 1 : 2; 
      r = q[k].x + randu(-dr,dr);
      for (j = 1; j <= s; j++)
         B[j][j] -= r;
   
/* Compute Q' and B = Q'B */

      identity (Q,n);
      for (k = 1; k < s; k++) {
         d = (float) sqrt (B[k][k]*B[k][k] + B[k+1][k]*B[k+1][k]);
         cphi = B[k][k]/d;
         sphi = B[k+1][k]/d;    	
         for (j = 1; j <= s; j++) {
            a = Q[k][j];
            b = Q[k+1][j];
            Q[k][j] =    cphi*a + sphi*b;
            Q[k+1][j] = -sphi*a + cphi*b;
            a = B[k][j];
            b = B[k+1][j];
            B[k][j] =    cphi*a + sphi*b;
            B[k+1][j] = -sphi*a + cphi*b; } }
         
/* Update B = BQ */

      for (i = 1; i <= s; i++) {
         for (j = 1; j <= s; j++)
            u[j] = B[i][j];
         for (j = 1; j <= s; j++) {
            B[i][j] = 0;
            h = MIN(j+2,s);	
            for (k = 1; k <= h; k++)
               B[i][j] += u[k]*Q[j][k]; } }   
         
/* Restore shift: B = B + rI */

      for (j = 1; j <= s; j++)
         B[j][j] += r;
      p++;
      
      #if defined(DISPLAY_EIGEN)
         showmat ("B",B,s,s,0);
      #endif
      
/* Check for convergence */

      if (fabs(B[s][s-1]) < eps) {
         c[n-s+1] = cnum(B[s][s],0);
         s -= 1; }

      else if (fabs(B[s-1][s-2]) < eps) {
         eig2 (B,s-1,q);
         c[n-s+1] = q[1];
         c[n-s+2] = q[2]; 
         s -= 2; }

      switch (s) {

         case 1:
            c[n] = cnum(B[1][1],0);
            s -= 1;
            break;

         case 2:
            
            eig2 (B,1,q);
            c[n-1] = q[1];
            c[n] = q[2];
            s -= 2;
            break; } }

/* Finalize */
      
   csort (c,n);
   freevec (u);
   freemat (B,n);
   freemat (Q,n);
   freecvec (q);
   return MAX(p-1,0); }

/* ------------------------------------------------------------------- */
int danilevsky (matrix A,int n,int m,float eps,cvector c,cmatrix X) {
/* ------------------------------------------------------------------- */

/* Description: Find the eigenvalues and eigenvectors of an n by n
                matrix A using Danilevsky's method.

   On entry:    A   = n by n matrix
                n   = dimension of A (n >= 1)
                m   = maximum number of iterations (m >= 1)
                eps = error tolerance used to terminate search (eps >= 0)

   On exit:     c   = n by 1 vector of eigenvalues ordered according
                      to decreasing magnitude
                X   = n by n matrix of eigenvectors.  The kth column
                      of X contains the eigenvector corresponding
                      to eigenvlue c[k].

   Returned:    The number of iterations performed.  If it is positive
                and less than the user-specified maximum, m, then A
                was successfully converted to upper block diagonal form
                with the "zero" entries having magnitudes less than eps.
                A zero is returned if A was found to be a degenerate
                matrix. */

   int		i,j,k,p,r;
   float	em = epsilon(),
		alpha,q;
   vector	a,b;
   cvector	y,z;            
   matrix	Q;

/* Initialize */

   if      (n < 1)
      nerror (100,"danilevsky",2,(float) n,"");
   else if (m < 1)
      nerror (100,"danilevsky",3,(float) m,"");
   else if (eps < 0)
      nerror (100,"danilevsky",4,eps,"");
   if (n == 1) {
      c[1] = cnum (A[1][1],0);
      X[1][1] = cnum (1,0);
      return 1; }
   a = vec (n+1,"");
   b = vec (n,"");
   y = cvec (n,"");
   z = cvec (n,"");
   Q = mat (2*n,n,"");
   for (k = 1; k <= n; k++)
      for (j = 1; j <= n; j++) {
      	 Q[k][j]   = A[k][j];
      	 if (j == k)
            Q[n+k][j] = 1; }
            
/* Reduce A to companion form with similarity transformations */

   k = n;
   while (k >= 2) { 
   
/* Select pivot and test for degenerate matrix */

      p = 1;
      alpha = (float) fabs(Q[k][p]);
      for (j = 1; j < k; j++)
         if (fabs(Q[k][j]) > alpha) {
            p =	j;
            alpha = (float) fabs(Q[k][p]); }
      if (alpha < em) {			/* degenerate matrix */
         freecvec (y);
         freecvec (z);
         freevec (b);
         freevec (a);
         freemat (Q,2*n);
         return 0; }
      else {
         if (p < k-1) {

/* Interchange columns p and k-1 */

            for (j = 1; j <= 2*n; j++) {
               q = Q[j][p];
               Q[j][p] = Q[j][k-1];
               Q[j][k-1] = q; }
         
/* Interchange rows p and k-1 */

            for (j = 1; j <= n; j++) {
               q = Q[p][j];
               Q[p][j] = Q[k-1][j];
               Q[k-1][j] = q; } }

/* Save row k */
      
         for (j = 1; j <= n; j++)
       	    a[j] = Q[k][j];			

/* Divide column k-1 by alpha */

         alpha = Q[k][k-1];
         for (j = 1; j <= 2*n; j++)
            Q[j][k-1] /= alpha;

/* Eliminate entries in row k */

         for (j = 1; j <= n; j++) {
            if (j != k-1) {
               q = Q[k][j];	
               for (i = 1; i <= 2*n; i++)
                  Q[i][j] -= q*Q[i][k-1]; } } }

/* Complete similarity transformation by updating row k-1 */

      for (j = 1; j <= n; j++) {
         b[j] = 0;
      	 for (i = 1; i <= n; i++)
            b[j] += a[i]*Q[i][j]; }
      for (j = 1; j <= n; j++)      
         Q[k-1][j] = b[j];
      #if defined(DISPLAY_EIGEN)
         showmat ("Q",Q,2*n,n,0);
      #endif   
      k--; } 
         
/* Find eigenvalues */
      	    
   r =  qr (A,n,m,eps,c);
   #if defined(DISPLAY_EIGEN)
      showcvec ("c",c,n,0);
   #endif   

#if 0
   for (k = 1; k <= 2*n; k++) {
      if (k == n+1)
         printf ("\n");
      for (j = 1; j < n; j++)
         printf ("%.4f & ",Q[k][j]);
      printf ("%.4f \\\\ \n",Q[k][n]); }    
#endif
      
/* Construct eigenvectors */
	
   for (k = 1; k <= n; k++) {
      y[n] = cnum (1,0);
      for (j = n-1; j >= 1; j--)
         y[j] = cmul(c[k],y[j+1]);
      for (j = 1; j <= n; j++) {
         X[j][k] = cnum (0,0);	
   	    for (i = 1; i <= n; i++) {
   	       X[j][k].x += Q[n+j][i]*y[i].x;
   	       X[j][k].y += Q[n+j][i]*y[i].y; } } } 
      
/* Finalize */

   freecvec (y);
   freecvec (z);
   freevec (b);
   freevec (a);
   freemat (Q,2*n);
   return r; }

/* ------------------------------------------------------------------- */
int roots (vector a,float eps,int n,int m,cvector x) {
/* ------------------------------------------------------------------- */

/* Purpose: Find the n roots of an nth degree polynomial f(x) using
            the eigenvalue method with QR iteration.

            f(x) = a[n+1]*pow(x,n) + ... + a[2]*x + a[1]  

            On entry, the n+1 by 1 vector a contains the coefficients
            of the polynomial f, eps > 0 is an error tolerance used
            by function qr, and m is the maximum number of QR iterations
            to perform.  On exit, the n by 1 complex vector x contains
            the roots of f.  The value returned by roots is the number of
            QR iterations performed.  If it is less than m, then
            convergence was achieved. */

   matrix	A;         
   float	em;
   int 		i,k;
   #if defined(DISPLAY_EIGEN)
      complex z;
      float y;
   #endif

/* Initialize */

   em = epsilon();
   if      (n < 1)
      nerror (100,"roots",3,(float) n,"");
   else if (fabs(a[n+1]) < em)
      nerror (304,"roots",n,0,"");
      
/* Construct companion matrix, find its eigenvalues */      

   A = mat (n,n,"");
   for (i = 1; i <= n; i++) {
      A[1][i] = -a[n-i+1]/a[n+1];
      if (i < n)
         A[i+1][i] = 1; } 
   k = qr (A,n,m,eps,x);

   #if defined(DISPLAY_EIGEN)
      for (i = 1;i <= n; i++) {
         z = cpoly (x[i],a,n);
         y = sqrt (z.x*z.x + z.y*z.y);	
         printf ("\n%2i & %10.7f %+10.7fj & %10.7f \\\\",i,
                  x[i].x,x[i].y,y); }
   #endif
   
/* Finalize */

   freemat (A,n);
   return k; }
   
/* ------------------------------------------------------------------ */
static void  identity (matrix A,int n) {
/* ------------------------------------------------------------------ */

/* Description: Set A equal to the n by n identity matrix I.

   On entry:    A = n by n matrix
		n = dimension of A

   On exit:     A = I */

   int 		i;
   zeromat (A,n,n);
   for (i = 1; i <= n; i++)
      A[i][i] = 1; }

/* ------------------------------------------------------------------- */
static void getmax (matrix A,int n,int *p,int *q) {
/* ------------------------------------------------------------------- */

/* Description: Find the row p and column q of the element of the
                n by n matrix A above the diagonal whose magnitude
                is largest.

   On entry:    A = n by n matix
		n = dimension of A

   On exit:     *p = row of maximum element above diagonal
                *q = column of maximum element above diagonal */
            
   int		i,j;
   float	a;
   *p = 1;
   *q = 2;
   a = (float) fabs (A[*p][*q]);
   for (i = 1; i < n; i++)
      for (j = i+1; j <= n; j++) 
      	 if (fabs(A[i][j]) > a) {
            *p = i;
            *q = j;
            a = (float) fabs (A[*p][*q]); } }
   
/* ------------------------------------------------------------------- */
static void eig2 (matrix A,int k,cvector q) {
/* ------------------------------------------------------------------- */

/* Description: Find eigenvalues of 2 by 2 matrix with upper-left
                corner at A[k][k].  Return the results in the
                2 by 1 complex vector q.

   On entry:    A = n by n matrix with n >= k+1
   		k = subscript of upper-left corner element

   On exit:     q = 2 by 1 vector containing eigenvalues */
 
   float 	t,d,r,
		em = epsilon();
   t = A[k][k] + A[k+1][k+1];
   d = A[k][k]*A[k+1][k+1] - A[k][k+1]*A[k+1][k];
   r = t*t - 4*d;
   if (r < 0) {

/* Complex conjugate eigenvalues */

      q[1] = cnum (t/2,(float) sqrt(-r)/2);
      q[2] = cnum (q[1].x,-q[1].y); }

   else {
      
/* Real eigenvalues */

      r = (float) sqrt(r);
      q[1] = cnum ((t+SGN(t)*r)/2,0);
      if (fabs(q[1].x) > em)
         q[2] = cnum (d/q[1].x,0);
      else
         q[2] = cnum ((t - SGN(t)*r)/2,0); } }


