/* -------------------------------------------------------------------- */
/* File:     integ.c

   Contents: Numerical differentiation and integration functions

   Uses:     nlib.c */
/* -------------------------------------------------------------------- */

/* -------------------------------------------------------------------- */
float	deriv (float a,float h,int n,int m,int e,float (*f)(float x)) {
/* -------------------------------------------------------------------- */

/* Description: Numerically estimate the first derivative of f(x) at
                x = a.

   On entry:    a = evaluation point
                h = step size (h > 0)
                n = number of points 
                m = type of approximation:

                        m      Approximation                n
                       ----------------------------------------
                       -1   n-point backward difference   2,3,4
                        0   n-point central difference    3,5
                        1   n-point forward difference    2,3,4
                       ----------------------------------------

               e  = extrapolation parameter.  If e is nonzero,
                    extrapoaltion from step sizes 2h and h is used.
               f  = name of user-supplied function to be differentiated
                    The prototype of f is

                       float f(float x);

                    When f is called it must return the value f(x). 

   Returned:   y = f'(a) */

   float	x,y;             

/* Initialization */

   if      (h <= 0)
      nerror (100,"deriv",2,h,"");

/* Extrapolation */

   if (e) {
      x = (float) pow (2,n-1);	
      y = (x*deriv(a,h,n,m,0,f) - deriv(a,2*h,n,m,0,f))/(x-1);
      return y; }
   else if (m < 0) {

/* Backward difference */

      if ((n < 2) || (n > 4))
         nerror (100,"deriv",3,(float) n,"");
      switch (n) {
         case 2:
            y = f(a) - f(a-h);
            break;
         case 3:
            y = (3*f(a) - 4*f(a-h) + f(a-2*h))/2;
            break;
         case 4:
            y = (11*f(a) - 18*f(a-h) + 9*f(a-2*h) -
                 2*f(a-3*h))/6;
            break; } }

   else if (m == 0) {
    
/* Central difference */

      if ((n != 3) && (n != 5))
         nerror (100,"deriv",3,(float) n,"");
      switch (n) {
         case 3:
            y = (f(a+h) - f(a-h))/2;
            break;
         case 5:
            y = (f(a-2*h) - 8*f(a-h) + 8*f(a+h) -
                 f(a+2*h))/12;
            break; } }

   else {
       
/* Forward difference */

      if ((n < 2) || (n > 4))
         nerror (100,"deriv",3,(float) n,"");
      switch (n) {
         case 2:
            y = -f(a) + f(a+h);
            break;
         case 3:
            y = (-3*f(a) + 4*f(a+h) - f(a+2*h))/2;
            break;
         case 4:
            y = (-11*f(a) + 18*f(a+h) - 9*f(a+2*h) +
                 2*f(a+3*h))/6;
            break; } }
 
  return y/h; }

/* -------------------------------------------------------------------- */
float	trapint (float a,float b,int n,float (*f)(float x)) {
/* -------------------------------------------------------------------- */

/* Description: Use the trapezoid rule to numerically integrate the
                function f(x) over the interval [a,b] using n panels.

   On entry:    a = lower limit of integration
                b = upper limit of integration
                n = number of panels (n >= 1)
                f = name of user-supplied function to be integrated.   
                    The prototype of f is:

                       float f (float x);

                    When f is called it must return the value f(x).

   Returned:    y = estimate of integral

   Note:        The truncation error is proportional to pow(n,-2). */

   int		i;
   float	y,h;
   if (n < 1)
      nerror (100,"trapint",4,(float) n,"");   
   h = (b - a)/n;
   y = ((*f)(a) + (*f)(b))/2;
   for (i = 1; i < n; i++)
      y += (*f)(a + i*h);
   return h*y; }

/* -------------------------------------------------------------------- */
float	simpson (float a,float b,int n,float (*f)(float x)) {
/* -------------------------------------------------------------------- */

/* Description: Use Simpson's rule to numerically integrate the
                function f(x) over the interval [a,b] using n panels.

   On entry:    a = lower limit of integration
                b = upper limit of integration
                n = number of panels (n >= 2 and even)
                f = name of user-supplied function to be integrated.   
                    The prototype of f is:

                       float f (float x);

                    When f is called it must return the value f(x).

   Returned:    y = estimate of integral

   Note:        The truncation error is proportional to pow(n,-4). */

   int		i;
   float	y,h;
   if ((n < 2) || (n % 2))
      nerror (100,"simpson",4,(float) n,"");   
   h = (b - a)/n;
   y = (*f)(a) + (*f)(b);
   for (i = 1; i < n; i++) 
      if (i % 2)
         y += 4*(*f)(a + i*h);
      else
         y += 2*(*f)(a + i*h);   
   return (h*y/3); }

/* -------------------------------------------------------------------- */
float	midpoint (float a,float b,int n,float (*f)(float x)) {
/* -------------------------------------------------------------------- */

/* Description: Use the midpont rule to numerically integrate the
                function f(x) over the interval [a,b] using n panels.

   On entry:    a = lower limit of integration
                b = upper limit of integration
                n = number of panels (n >= 1)
                f = name of user-supplied function to be integrated.   
                    The prototype of f is:

                       float f (float x);

                    When f is called it must return the value f(x).

   Returned:    y = estimate of integral

   Note:        The truncation error is proportional to pow(n,-2). 
                The midpont rule is an open integration formula that
                does not requres the value of f(x) at the end
                points, a and b. */

   int		i;
   float	y = 0,
		h;
   if (n < 1)
      nerror (100,"midpoint",4,(float) n,"");   
   h = (b - a)/n;
   for (i = 1; i <= n; i++)
      y += (*f)(a + (2*i-1)*h/2);
   return h*y; }
  
/* -------------------------------------------------------------------- */
float	romberg (float a,float b,float eps,int m,
                 int *r,float (*f)(float x)) {
/* -------------------------------------------------------------------- */

/* Description: Use the Romberg integration mathod tonumerically
                integrate the function f(x) over the interval [a,b].

   On entry:    a   = lower limit of integration
                b   = upper limit of integration
                eps = error tolerance for truncation error (eps >= 0)
                m   = maximum extrapolation level (2 to 10) 
                f   = name of user-supplied function to be integrated.   
                      The prototype of f is:

                         float f (float x);

                      When f is called it must return the value f(x).

   On exit:     *r = extrapolation level.  If *r < m then the following
                     termination criterion was satisfied where E is the
                     estimated truncation error:

                        |E| < eps

   Returned:    y = estimate of integral. */

   int 		k = 1,
		n = 1,
		j;
   float	h,E,x;
   matrix 	R;  

/* Initialize */ 

   if      (eps < 0)
      nerror (100,"romberg",3,eps,"");
   else if ((m < 2) || (m > 10))
      nerror (100,"romberg",4,(float) m,"");
   R = mat(m,m,"");
   h = b - a;
   R[1][1] = h*((*f)(a) + (*f)(b))/2;
   #if defined(DISPLAY_INTEG)
            printf ("\n %i & %12.7f & & & & & %12.7f \\\\",1,R[1][1],
                     R[1][1]-1);
   #endif
      
/* Compute estimates */
   
   if (m > 1) {
      do {
         k++;
         n *= 2;
         h /= 2;
         R[k][1] = ((*f)(a) + (*f)(b))/2;
         for (j = 1; j < n; j++) 
            R[k][1] +=  (*f)(a + j*h); 	
         R[k][1] *= h;  	
         for (j = 2; j <= k; j++) {
            x = (float) pow(4,j-1);
            R[k][j] = (x*R[k][j-1] - R[k-1][j-1])/(x - 1); }
         E = (float) ((R[k][k-1] - R[k-1][k-1])/(pow(4,k-1) - 1));
         #if defined(DISPLAY_INTEG)
            printf ("\n %i & ",k);
            for (j = 1; j <= k; j++)
               printf ("%12.7f & ",R[k][j]);
            for (j = k+1; j <= m; j++)
               printf ("& ");   
            printf ("%12.7f \\\\",R[k][k]-1); 
         #endif
         }
      while ((fabs(E) >= eps) && (k < m)); }

/* Finalize */

   *r = k;
   x = R[k][k];
   freemat (R,m);
   return x; }
      
/* -------------------------------------------------------------------- */
float gausquad (float a,float b,int n,float (*f)(float x)) {
/* -------------------------------------------------------------------- */

/* Description: Use the n-point Gauss-Legendre formula to numerically
                integrate the function f(x) over the interval [a,b].

   On entry:    a = lower limit of integration
                b = upper limit of integration
                n = number of points (1 <= n <= 10)
                f = name of user-supplied function to be integrated.   
                    The prototype of f is:

                       float f (float x);

                    When f is called it must return the value f(x).

   Returned:    y = estimate of integral. */

   int		i;
   float	y = 0,
		alpha,beta;
   vector	x,c;

/* Initialize */
   
   if ((n < 1) || (n > 10))
      nerror (100,"gausquad",4,(float) n,"");  	
   x = vec (n,"");
   c = vec (n,"");

/* Compute parameters */

   switch (n) {

      case 1: x[1] = 0.0f;
              c[1] = 2.0f;
              break;

      case 2: x[1] = 0.5773503f; x[2] = -x[1];
              c[1] = c[2] = 1;
              break;

      case 3: x[1] = 0.0f;
              x[2] = 0.7745967f; x[3] = -x[2];
              c[1] = 0.8888889f;         
              c[2] = c[3] = 0.5555556f;   
              break;

      case 4: x[1] = 0.3399810f; x[2] = -x[1];
              x[3] = 0.8611363f; x[4] = -x[3];
              c[1] = c[2] = 0.6521452f;         
              c[3] = c[4] = 0.3478548f;   
              break;

      case 5: x[1] = 0.0f;
              x[2] = 0.5384693f; x[3] = -x[2];
              x[4] = 0.9061798f; x[5] = -x[4];
              c[1] = 0.5688880f;
              c[2] = c[3] = 0.4786287f;   
              c[4] = c[5] = 0.2369269f;   
              break;

      case 6: x[1] = 0.2386192f; x[2] = -x[1];
              x[3] = 0.6612094f; x[4] = -x[3];
              x[5] = 0.9324695f; x[6] = -x[5];
              c[1] = c[2] = 0.4679139f;
              c[3] = c[4] = 0.3607616f;   
              c[5] = c[6] = 0.1713245f;   
              break;

      case 7: x[1] = 0.0f;
              x[2] = 0.4058452f; x[3] = -x[2];
              x[4] = 0.7415312f; x[5] = -x[4];
              x[6] = 0.9491079f; x[7] = -x[6];
              c[1] = 0.4179592f;
              c[2] = c[3] = 0.3818301f;   
              c[4] = c[5] = 0.2797054f;   
              c[6] = c[7] = 0.1294850f;   
              break;

      case 8: x[1] = 0.1834346f; x[2] = -x[1];
              x[3] = 0.5255324f; x[4] = -x[3];
              x[5] = 0.7966665f; x[6] = -x[5];
              x[7] = 0.9620899f; x[8] = -x[7];
              c[1] = c[2] = 0.3626838f;
              c[3] = c[4] = 0.3137066f;   
              c[5] = c[6] = 0.2223810f;   
              c[7] = c[8] = 0.1012285f;   
              break;

      case 9: x[1] = 0.0f;
              x[2] = 0.3242534f; x[3] = -x[2];
              x[4] = 0.6133714f; x[5] = -x[4];
              x[6] = 0.8360311f; x[7] = -x[6];
              x[8] = 0.9681602f; x[9] = -x[8];
              c[1] = 0.3302394f;
              c[2] = c[3] = 0.3123471f;   
              c[4] = c[5] = 0.2606107f;   
              c[6] = c[7] = 0.1806482f;   
              c[8] = c[9] = 0.0812744f;   
              break;

      case 10: x[1] = 0.1488743f; x[2] = -x[1];
               x[3] = 0.4333954f; x[4] = -x[3];
               x[5] = 0.6794096f; x[6] = -x[5];
               x[7] = 0.8650634f; x[8] = -x[7];
               x[9] = 0.9739065f; x[10] = -x[9];
               c[1] = c[2] = 0.2955242f;
               c[3] = c[4] = 0.2692602f;   
               c[5] = c[6] = 0.2190864f;   
               c[7] = c[8] = 0.1494513f;   
               c[9] = c[10] = 0.0666713f;   
               break; }
              
/* Estimate integral */

   alpha = (b - a)/2;
   beta  = (b + a)/2;
   for (i = 1; i <= n; i++)               
      y += c[i]*(*f)(alpha*x[i]+beta);
   y *= alpha;

/* Finalize */

   freevec (c);
   freevec (x);
   return y; }

/* -------------------------------------------------------------------- */
float monte (vector a,vector b,int m,int n,float (*f)(vector x,int m)) {
/* -------------------------------------------------------------------- */

/* Description: Use the Monte-Carlo method to integrate an m-dimensional
                function f(x).

   On entry:    a = m by 1 vector of lower integration limits
   		b = m by 1 vector of upper integration limits
       		    The domain of integration D is the following
                    rectangular region:
                
                       D = {x: a[k] <= x[k] <= b[k],  1 <= k <= m}

   		m = number of variables (m >= 1)
                n = number of random points uses to estimate
                    integrand (n >= 1)
                f = name of user-supplied function to be integrated.   
                    The prototype of f is:

                       float f(vector x,int m);

                    When f is called with the m by 1 vector x, it must
                    return the value f(x).

   Returned:    y = estimate of integral. 

   Notes:       The error in the estimate of the integral (standard
                deviation) is roughly proportional to 1/sqrt(n).  */

   int		i,j;
   float	mu = 1,
		y = 0;
   vector	x;
   
/* Initialize */

   if (m < 1)
      nerror (100,"Monte-Carlo",4,(float) m,"");
   if (n < 1)
      nerror (100,"Monte-Carlo",5,(float) n,"");
   x = vec (m,"");
      
/* Compute volume of D and estimate average value of f(x) */
  
   for (j = 1; j <= m; j++)
      mu *= b[j] - a[j]; 
   for (i = 1; i <= n; i++) {
      for (j = 1; j <= m; j++) 
         x[j] = randu (a[j],b[j]);
      y += (*f)(x,m); }
   y *= mu/n;

/* Finalize */

   freevec (x);
   return y; }
      
      
/* -------------------------------------------------------------------- */
float	deriv2 (float a,float h,int p,int m,int e,float (*f)(float x)) {
/* -------------------------------------------------------------------- */

/* Description: Numerically estimate the second derivative of f(x) at
                x = a.

   On entry:    a = point of evaluation
                h  = step size (h > 0)
                p  = number of points to use (2 <= p <= 5)
                m  = type of approximation:

                         m      Approximation                 p
                        ------------------------------------------
                        -1   p-point backward difference   3,4,5
                         0   p-point central difference    3,5
                         1   p-point forward difference    3,4,5
                        ------------------------------------------

                e  = extrapolation parameter.  If e is nonzero,
                     extrapoaltion from step sizes 2h and h is used.
                f  = name of user-supplied function to be differentiated
                     The prototype of f is

                        float f(float x);

                     When f is called it must return the value f(x). 

   Returned:    y = f''(a) */

   float	y,x;             

/* Extrapolation */

   p = MAX (2,MIN(5,p));
   if (e) {
      x = (float) pow (2,p-1);	
      y = (x*deriv2(a,h,p,m,0,f) - deriv2(a,2*h,p,m,0,f))/(x-1);
      return y; }

   if (m < 0) {

/* Backward difference */

      p = MAX (3,MIN(4,p));
      switch (p) {
         case 3:
            y = (f(a) - 2*f(a-h) + f(a-2*h));
            break;
         case 4:
            y = (2*f(a) - 5*f(a-h) + 4*f(a-2*h) - f(a-3*h));
            break; } }

   else if (m == 0) {
    
/* Central difference */

      if (p <= 3) p = 3;
         else     p = 5;
      switch (p) {
         case 3:
            y = f(a-h) - 2*f(a) + f(a+h);
            break;
         case 5:
            y = (-f(a-2*h) + 16*f(a-h) - 30*f(a) + 16*f(a+h)
                 -f(a+2*h))/12;
            break; } }

   else {
       
/* Forward difference */

      p = MAX (3,MIN(4,p));
      switch (p) {
         case 3:
            y = f(a) - 2*f(a+h) + f(a+2*h);
            break;
         case 4:
            y = 2*f(a) - 5*f(a+h) +4*f(a+2*h) - f(a+3*h);
            break; } }

       return y/(h*h); }
   
