/* ------------------------------------------------------------------- */
/* File:     linear.c

   Contents: Linear algebraic system functions

   Uses:     nlib.c */
/* ------------------------------------------------------------------- */

/* ------------------------------------------------------------------- */
/* Local Functions */
/* ------------------------------------------------------------------- */

static float zero     (void);

/* ------------------------------------------------------------------- */
static float zero     (void) {
/* ------------------------------------------------------------------- */

/* Description: Compute a small positive quantity to be defined
                as zero for pivoting operations, singular matrix
                detection, etc. 

   Returned:    A small positive number. */
              
   return epsilon()/100; } 
              
/* ------------------------------------------------------------------- */
int scalemat (matrix A,int m,int n) {
/* ------------------------------------------------------------------- */

/* Description: Scale the m by n matrix A using equilibration.
                That is, normalize each row by dividing by the
                component whose magnitude is largest.  Scan
                only the first m columns for the maximum. 

   On entry:    A = m by n matrix
                m = number of rows of A
                n = number of columns of A.

   On exit:     A = scaled m by n matrix

   Returned:    Zero if A contains a zero row. */

   int 		i,j;
   float	a,
		eps = zero();

   for (i = 1; i <= m; i++) {
      a = 0;
      for (j = 1; j <= m; j++)
         a = MAX(a,(float) fabs(A[i][j]));
      if (a < eps)
         return 0;
      else
         for (j = 1; j <= n; j++)
            A[i][j] /= a; }
   return 1; }

/* ------------------------------------------------------------------- */
float matnorm (matrix A,int n) {
/* ------------------------------------------------------------------- */

/* Descripton: Compute the row-sum norm of the matrix A.

   On entry:   A = n by n matrix
               n = dimension of A (n >= 1)

   Returned:   ||A|| */

   float	sum,norm = 0;
   int		i,j;
   if (n < 1)
      nerror (100,"matnorm",2,(float) n,"");
   for (i = 1; i <= n; i++) {
      sum = 0;
      for (j = 1; j <= n; j++) 
         sum += (float) fabs(A[i][j]);
      if (sum > norm)
         norm = sum; }
   return norm; }

/* ------------------------------------------------------------------- */
float lufac (matrix A,int n,matrix Q,vector p) {
/* ------------------------------------------------------------------- */

/* Descripton: Factor the n by n matrix A into a lower-triangular factor
               L and and upper-triangular factor U.

   On entry:   A = n by n matrix
               n = dimension of A (n >= 1)

   On exit:    Q = n by n storage matrix containing factors L and
                   U.  The diagonal elements of U are all one.
               p = n by 1 row permutation vector.  Initially,
                   p[k] = k.  All row changes during pivoting
                   are also applied to p.

   Returned:   The determinant of A.  If it is nonzero, then
               Q and p have been successfully computed.

   Notes:      The function lufac can be used with the function
               lusub to efficiently solve the system Ax = b
               multiple times using different right-hand side
               vectors b. */

   matrix	D;         
   int		i,j,k,q;
   float 	d,
   		eps = zero(),
   	 	delta = 1;
   
/* Form augumented matrix D = [A,p] */

   if (n < 1)
      nerror (100,"lufac",2,(float) n,"");
   D = mat (n,n+1,"");
   for (i = 1; i <= n; i++) {
      for (j = 1; j <= n; j++) 
         D[i][j] = A[i][j];
      D[i][n+1] = (float) i; } 

/* Factor A into L and U */

   for (j = 1; j <= n; j++) {

/* Compute column j of D */

      for (k = j; k <= n; k++) 
         for (i = 1; i <= j-1; i++)
            D[k][j] -= D[k][i]*D[i][j];
            
/* Select pivot */

      q = j;
      for (i = j+1; i <= n; i++) 
         if (fabs(D[i][j]) > fabs(D[q][j]))
            q = i;

/* Check for singular A */

      if (fabs(D[q][j]) < eps) {
         freemat (D,n);
         zeromat (Q,n,n);
         return 0; }

/* Swap rows q and j of D, change sign of delta */

     if (q > j) {
        for (i = 1; i <= n+1; i++) {
           d = D[q][i];
           D[q][i] = D[j][i];
           D[j][i] = d; }
        delta = -delta; }
        
/* Compute row j of D, update delta */

      for (k = j+1; k <= n; k++) {
         for (i = 1; i <= j-1; i++)
            D[j][k] -= D[j][i]*D[i][k];
         D[j][k] /= D[j][j]; }
      delta *= D[j][j]; }

/* Finalize */

   for (k = 1; k <= n; k++) {
      for (j = 1; j <= n; j++)
         Q[k][j] = D[k][j];
      p[k] = D[k][n+1]; }
   freemat (D,n);
   return delta; }

/* ------------------------------------------------------------------- */
int lusub (matrix Q,vector p,vector b,int n,vector x) {
/* ------------------------------------------------------------------- */

/* Descripton: Solve a factored linear algebraic system.

   On entry:   Q = n by n storage matrix which contains the lower-
                   triangula factor L and the upper-trinagular
                   factor U. Q is computed by function lufac.
               p = n by 1 row purmutationa vector. p is computed
                   by function lufac. 
               n = number of unknowns (n >= 1)

   On exit:    x = n by 1 solution vector

   Returned:   Zero if a singular matrix was detected.

   Notes:      The function lusub is used with the function
               lufac to efficiently solve the system Ax = b
               multiple times using different right-hand side
               vectors b. */

   vector	d,y;
   int		i,k;
   float	delta;
   
/* Check for a singular LU */
   
   if (n < 1)
      nerror (100,"lusub",4,(float) n,"");
   delta = Q[1][1];
   for (i = 2; i <= n; i++)
      delta *= Q[i][i];
   if (delta == 0)
      return 0;

/* Sort the right-hand side vector */
 
   d = vec (n,"");
   for (i = 1; i <= n; i++) {
      k = (int) p[i];	
      d[i] = b[k]; }
      
/* Perform forward substitution to find y */
 
   y = vec (n,"");
   for (k = 1; k <= n; k++) {
      y[k] = d[k];
      for (i = 1; i <= k-1; i++)
         y[k] -= Q[k][i]*y[i];
      y[k] /= Q[k][k]; }
    
/* Perform back substitution to find x */
 
   for (k = n; k >= 1; k--) {
      x[k] = y[k];
      for (i = k+1; i <= n; i++)
         x[k] -= Q[k][i]*x[i]; }

/* Finalize */

   freevec (y);
   freevec (d);
   return 1; }

/* ------------------------------------------------------------------- */
int ludec (matrix A,vector b,int n,vector x) {
/* ------------------------------------------------------------------- */

/* Descripton: Solve the linear algebraic system Ax = b using the
               LU decomposition method.

   On entry:   A = n by n coefficeint matrix
               b = n by 1 right-hand side vector
               n = number of unknowns (n >= 1)

   On exit:    x = n by 1 solution vector

   Returned:   r = nonzero if x has been successfully computed.
               Otherwise, A is singular. */

   int		r;         
   matrix	Q,B;
   vector	p,c;
   float	eps = zero();
		
/* Initialize */

   if     (n < 1)
      nerror (100,"ludec",3,(float) n,"");
   else if (n == 1)
      if (fabs(A[1][1]) < eps)
         return 0;
      else {
         x[1] = b[1]/A[1][1];
         return 1; } 	
   Q = mat (n,n,"");
   p = vec (n,"");

/* Prescale using equlibration */

   B = mat (n,n+1,"");
   c = vec (n,"");
   copymat (A,n,n,B);
   putcol (B,n,n+1,n+1,b);
   scalemat (B,n,n+1);
   getcol (B,n,n+1,n+1,c);
   
/* Solve system by factoring A */

   if (lufac (B,n,Q,p))
      r = lusub (Q,p,c,n,x);
   else
      r = 0;
      
/* Finalize */

   freemat (Q,n);   
   freevec (p);
   freemat (B,n);
   freevec (c);
   return r; }
   
/* ------------------------------------------------------------------- */
float det (matrix A,int n) {
/* ------------------------------------------------------------------- */

/* Descripton: Compute the determinant of A using LU factorization.

   On entry:   A = n by n matrix
               n = dimension of A (n >= 1)

   Returned:   The determinant of A.  */

   matrix 	Q;
   vector	p;
   float 	delta;
   if      (n < 1)
      nerror (100,"det",2,(float) n,"");
   else if (n == 1)
      return A[1][1];   
   Q = mat (n,n,"");
   p = vec (n,"");
   delta = lufac (A,n,Q,p);
   freevec (p);
   freemat (Q,n);
   return delta; }
   
/* ------------------------------------------------------------------- */
float inv (matrix A,int n,matrix B) {
/* ------------------------------------------------------------------- */

/* Descripton: Compute the inverse of A using LU decomposition.

   On entry:   A = n by n matrix
               n = dimension of A (n >= 1)

   On exit:    B = n by n matrix containing the inverse of A

   Returned:   The determinant of A.  If it is nonzero, then
               the inverse B has been successfully computed.

   Notes:      The function inv can be called with B = A
               in which case A is destroyed. */
            
   matrix	Q;
   vector 	p,b,x;
   float 	eps = zero(),
		delta;
   int		i,j;	
   
/* Initialize */

   if      (n < 1)
      nerror (100,"inv",2,(float) n,"");
   else if (n == 1)
      if (fabs(A[1][1]) < eps)
         return 0;
      else {
         B[1][1] = 1/A[1][1];
         return A[1][1]; }
   Q = mat (n,n,"");
   p = vec (n,"");
   delta = lufac (A,n,Q,p);
   if (delta == 0) {
      freemat (Q,n);
      freevec (p);
      return 0; }
   
/* Solve system n times using LU decomposition */

   b = vec (n,"");
   x = vec (n,"");
   for (i = 1; i <= n; i++) {
      zerovec (b,n);
      b[i] = 1;
      lusub (Q,p,b,n,x);
      for (j = 1; j <= n; j++)
         B[j][i] = x[j]; }

/* Finalize */

   freevec (x);
   freevec (b);
   freevec (p);
   freemat (Q,n);
   return delta; }

/* ------------------------------------------------------------------- */
float condnum (matrix A,int n,int m) {
/* ------------------------------------------------------------------- */

/* Descripton: Estimate the condition number of the matrix A:

               K(A) = ||A||*||inv(A)||

   On entry:   A = n by n matrix
               n = dimension of A (n >= 1)
               m = number of random test vectors (m >= 0)
               
   Returned:   The condition number K(A). 

   Notes:      If m = 0, the exact condition number is computed.
               K(A) ranges from 1 to infinity.  Large condition
               numbers indicated ill-conditioned coefficient
               matrices. */

   int		i; 
   float	K,s,
		r = 0,
		z = zero();
   vector	b,x,p;
   matrix	B;         
   
/* Initialize */
   
   if      (n < 1)
      nerror (100,"condnum",2,(float) n,"");
   else if (m < 0)
      nerror (100,"condnum",3,(float) m,"");
   else if (n == 1)
      if (fabs(A[1][1]) < z)
         return INFINITY;
      else
         return 1;
   B = mat (n,n,"");
   K = matnorm (A,n);
   if (m == 0) {

/* Exact value */

      if (inv (A,n,B))
         K *= matnorm (B,n);
      else
         K = INFINITY; }
   else {

/* Estimate using m test vectors */     

      x = vec (n,"");
      p = vec (n,"");
      lufac (A,n,B,p);        
      for (i = 1; i <= m; i++) {
         b = randvec (n,-1,1);
         lusub (B,p,b,n,x);
         s = vecnorm(x,n)/vecnorm(b,n);
         r = MAX (r,s);
         freevec (b); }
      K *= r; 	
      freevec (x);
      freevec (p); }

/* Finalize */

   freemat (B,n);
   return K; }
          
/* ------------------------------------------------------------------- */
float residual (matrix A,vector b,vector x,int n) {
/* ------------------------------------------------------------------- */

/* Description: Compute the norm of the residual error vector:

                   r = b - Ax

   On entry:    A = n by n coefficient matrix
                b = n by 1 right-hand side vector
                x = n by 1 solution estimate 
                n = number of unknowns (n >= 1)
               
   Returned:    ||r||

   Notes:       The vector x is a solution of Ax = b if and only if
                r = 0.  Therefore, ||r|| is a measure of the accuracy
                of the solution. */
 
   int		i;
   float	y;
   vector	r;
   if      (n < 1)
      nerror (100,"residual",4,(float) n,"");
   r = vec (n,"");
   matvec (A,x,n,n,r);
   for (i = 1; i <= n; i++)
      r[i] = b[i] - r[i];
   y = vecnorm(r,n);
   freevec (r);   
   return y; }
             
/* ------------------------------------------------------------------- */
int sr (vector x,matrix A,vector b,float alpha,float eps,int n,int m) {
/* ------------------------------------------------------------------- */

/* Description: Solve the linear algebraic system Ax = b using the
                successive over-relaxation or SR method.
 
   On entry:    x     = n by 1 vector containing initial guess
                A     = n by n coefficient matrix (nonzero
                        diagonal elements)
                b     = n by 1 right-hand side vector
                alpha = relaxation parameter (alpha > 0)
                eps   = error tolerance used to terminate search
                n     = number of unknowns (n >= 1)
                m     = maximum number of iterations (m >= 1)

   On exit:     x =    n by 1 solution vector

   Returned:    The number of iterations performed.  If it is positive
                and less than the user-specified maximum m, then the
                following convergence criterion was satisfied
                where r = b - Ax is the residual error vector:

                   ||r|| < eps 

                Zero is returned if one of the diagonal element of A
                was found to be zero.  In this case, the equations must
                be reordered.
                   
   Notes:       When alpha = 1, the function sr reduces to the Gauss-
                Seidel method.  If A is a symmetric positive-definite
                matrix, then the function sr should converge for all
                relaxation parameters in the following range:

                   0 < alpha < 2 */

   int		i,j,
		    k = 0;
   float	d,
		    z = zero();

/* Initialize */

   if       (alpha < 0)
      nerror (100,"sr",4,(float) n,"");
   else if  (eps < 0)
      nerror (100,"sr",5,(float) n,"");
   else if  (n < 1)
      nerror (100,"sr",6,(float) n,"");
   else if (m < 1)
      nerror (100,"sr",7,(float) m,"");
   for (i = 1; i <= n; i++) 
      if (fabs(A[i][i]) < z)
          return 0;

/* Iterate */

   do {
      for (i = 1; i <= n; i++) {
         d = A[i][i];
         x[i] = (1 - alpha)*x[i] + alpha*b[i]/d;
         for (j = 1; j <= n; j++)
            if (j != i)
               x[i] -= alpha*A[i][j]*x[j]/d; }
       k++; }        
   while ((k < m) && (residual(A,b,x,n) >= eps));               
   return k; }

/* ------------------------------------------------------------------- */
int gensys (matrix A,vector b,int m,int n,vector x) {
/* ------------------------------------------------------------------- */

/* Purpose: Find the minimum least-squares solution of the rectangular
            system Ax = b.  On entry, A is an m by n matrix, and b is
            an m by 1 vector.  The value returned is zero if A is not
            of full rank.  Otherwise, on exit, the n by 1 vector x is
            the minumum least-squares solution of Ax = b.  When n = m,
            gensys can be called with x = b. */

   matrix 	C,D;
   vector	c,Ai,Aj;
   int		i,j,k;

   if      (m < 1)
      nerror (100,"gensys",3,(float) m,"");
   else if (n < 1)
      nerror (100,"gensys",4,(float) n,"");

   if (m == n)				/* square system */

      return (ludec (A,b,m,x));

   else if (m > n) {			/* over-determined system */

      Ai = vec (m,"");
      Aj = vec (m,"");
      D = mat (n,n,"");
      c = vec (n,"");
      for (i = 1; i <= n; i++) {
         for (k = 1; k <= m; k++)
            Ai[k] = A[k][i];
         for (j = 1; j <= n; j++) {
             for (k = 1; k <= m; k++)
                Aj[k] = A[k][j];
             D[i][j] = inner (Ai,Aj,m); }
         c[i] = inner (Ai,b,m); }
      k = ludec (D,c,n,x);
      freevec (c);
      freemat (D,n);
      freevec (Ai);
      freevec (Aj);
      return k; }

   else {			/* under-determined system */	            

      C = mat (n,m,"");
      if ((k = geninv (A,m,n,C)) != 0);
         matvec (C,b,n,m,x);
      freemat (C,n);
      return k; } }

/* ------------------------------------------------------------------- */
int geninv (matrix A,int m,int n,matrix B) {
/* ------------------------------------------------------------------- */

/* Purpose: Compute the generalized inverse of the m by n matrix A.
            Store the result in the n by m matrix B. Return zero if
            A is not of full rank.  When m = n, geninv can be 
            called with B = A. */

   int		k;
   float        eps = epsilon()/100,d;
   matrix       C,D;

   if (m < 1)
      nerror (100,"geninv",2,(float) m,"");          
   else if (n < 1)
      nerror (100,"geninv",3,(float) n,"");          

   if (m == n)				/* square matrix */
   {
           d = inv (A,m,B);
	   k = (fabs(d) < eps) ? 0 : 1;
	   return k;
   }
   else if (m > n) 			/* over-determined */	            
   {
      C = mat (n,m,"");
      D = mat (n,n,"");
      transpose (A,m,n,C);
      matmat (C,A,n,m,n,D);
	  d = inv (D,n,D);
	  k = (fabs(d) < eps) ? 0 : 1;
      if (k) 
         matmat (D,C,n,n,m,B);
      freemat (C,n);
      freemat (D,n);
      return k; 
  }
  else 	 			/* under-determined */
  {
      C = mat (n,m,"");
      D = mat (m,m,"");
      transpose (A,m,n,C);
      matmat (A,C,m,n,m,D);
	  d = inv (D,m,D);
	  k = (fabs(d) < eps) ? 0 : 1;
	  if (k)
         matmat (C,D,n,m,m,B);
      freemat (C,n);
      freemat (D,m);
      return k; 
} }


/* ------------------------------------------------------------------- */
float tridec (matrix T,vector b,int n,vector x) {
/* ------------------------------------------------------------------- */

/* Descripton: Solve the tridiagonal linear algebraic system Ax = b
               using the LU decomposition method.

   On entry:   T = 3 by n matrix containing the nonzero entries of A:

                   row                  contents
                   ---------------------------------------------------
                    1     A[1][2], A[2][3], ... , A[n-1][n],   ---    
                    1     A[1][1], A[2][2], ... , A[n-1][n-1], A[n][n]    
                    3     A[2][1], A[3][2], ... , A[n][n-1],   ---    
                   ---------------------------------------------------

               b = n by 1 right-hand side vector
               n = number of unknowns (n >= 3)

   On exit:    x = n by 1 solution vector

   Returned:   The determinant of A.  If it is nonzero, then
               x has been successfully computed. */

   int 		k;            
   float	eps = epsilon()/100,
		delta = 1;
   vector	y;
   matrix	Q;

/* Initilialize */

   if (n < 3)
      nerror (100,"tridec",3,(float) n,"");   
   Q = mat (3,n,"");

/* Factor T into L and U */

   copyvec (T[3],n-1,Q[3]);
   Q[2][1] = T[2][1];
   for (k = 1; k < n; k++) {
      delta *= Q[2][k];	
      if (fabs(Q[2][k]) < eps) {
         freemat (Q,3);
         return 0; }
      else {
         Q[1][k] = T[1][k]/Q[2][k];
         Q[2][k+1] = T[2][k+1] - T[3][k]*Q[1][k]; } }
   delta *= Q[2][n];
   
/* Perform forward substitution to find y */
 
   y = vec (n,"");
   y[1] = b[1]/Q[2][1];
   for (k = 2; k <= n; k++) 
      y[k] = (b[k] - Q[3][k-1]*y[k-1])/Q[2][k];
    
/* Perform back substitution to find x */

   x[n] = y[n];
   for (k = n-1; k >= 1; k--) 
      x[k] = y[k] - Q[1][k]*x[k+1];

/* Finalize */

   freevec (y);
   freemat (Q,3);
   return delta; }


