/* -------------------------------------------------------------------- */
/* File:     nlib.c
   Contents: These are the low-level core functions of NLIB */
/* -------------------------------------------------------------------- */

svector svec    (int m,int n,char *s);
svector getsvec (FILE *f,int *m,int *n);

                      /**********************/
                      /*   ERROR HANDLING   */ 
                      /**********************/
              
/* ------------------------------------------------------------------- */
void nerror (int k,char *s,int i,float x,char *q) {
/* ------------------------------------------------------------------- */

/* Description: Display an NLIB error message.

   On entry:    k = NLIB error message number
                s = function where error was detected
                i = (depends on k)
                x = (depends on k)
                q = (depends on k) */

   int     c;
      
   printf ("\n*** NLIB error %i in function %s.\n    ",k,s);         
   switch (k) {

      case 100:
         printf ("Argument %i out of range. The value was %g. ***\n",i,x);
         break;

      case 101:
         printf
         ("Insufficient memory. Attempt to allocate %.0f bytes. ***\n",x);
         break;

      case 102:
         printf ("Could not output to file %s. ***\n",q);
         break;  

      case 103:
         printf ("File not opened for input. ***\n");
         break; 
         
      case 104:
         printf ("Complex division by zero. ***\n");
         break; 

      case 300:
         printf ("Number of samples must be a power of two. ***\n");
         break; 

      case 301:
         printf ("The dimension of argument %i must be odd. ***",i);
         break; 

      case 302:
         printf ("The dimension of argument %i is too small. ***\n",i);
         break; 

      case 303:
         printf ("Argument %i has too little frequency content. ***\n",i);
         break; 

      case 304:
         printf ("The polynomial is not of degree %i. ***\n",i);
         break; 

      case 305:
         printf ("Arguments one and two in do not bracket a root. ***\n");
         break; 

      case 306:
         printf ("Arguments one is not symmetric. ***\n");
         break; 

      case 307:
         printf
         ("Maximum number of function evaluations performed. ***\n");
         break;

      case 308:
         printf ("The function F does not satisfy: F'(0) < 0. ***\n");
         break;
         
      case 309:
         printf
         ("The independent variables must be strictly increasing. ***\n");
         break;
         
      case 310:
         printf ("The initial guesses must be distinct. ***\n");
         break;
         
      case 311:
         printf
         ("Minimum step size of h = %g reached. Try increase eps\n",x);
         printf ("    or decrease length of solution interval. ***\n");
         break;

      case 312:
         printf ("Coefficient matrix is singular. ***\n");
         break;

      default:
         printf ("\nUndocumented NLIB error?\n");
         break;  
   }

/* Exit or continue ? */

   printf ("\nEnter x to exit, or c to continue ... ");
   do {
      c = getchar();
      c = toupper (c);
      if      (c == 'X')
         exit (1); 
      else if (c == 'C')
         return; }
   while (1); }
         

              /************************************/
              /*   VECTOR AND MATRIX ALLOCATION   */ 
              /************************************/

/* ------------------------------------------------------------------- */
vector vec (int n,char *s) {
/* ------------------------------------------------------------------- */

/* Description: Allocate space for an n by 1 real vector x and
                initialize it.

   On entry:    n = number of components of vector x: subscripts
                    range from 1 to n.
                s = initialization string.  The values x[1],
                    x[2], ...  must be separated by commas or
                    white space.  If less than n values are
                    specified, the remaining values are set to
                    zero.

   Returned:    Pointer to the allocated region of memory. 
   
   Notes:       Vectors can also be allocated and initialized using
                the randvec and getvec functions. */

   vector	x;
   int		i;
   float	q;
   char		*c;   

/* Allocate space for x */
   
   if (n < 1)
      nerror (100,"vec",1, (float) n,"");
   q = (float) n*sizeof(float);
   x = (float *) malloc ((size_t) q);
   if (x == NULL)
      nerror (101,"vec",0, q,"");
   x -= 1;

/* Initialize x using string s */   

   for (i = 1; i <= n; i++)
      x[i] = 0;
   for (i = 1,c = s; (i <= n) && *c; i++,c++)
      x[i] = (float) strtod (c,&c); 
   return x; }

/* ------------------------------------------------------------------- */
vector randvec (int n,float a,float b) {
/* ------------------------------------------------------------------- */

/* Description: Allocate space for an n by 1 real vector x and
                initialize it with random numbers uniformly
                distributed over the interval [a,b].

   On entry:    n = number of components of vector x: subscripts
                    range from 1 to n.
                a = lower limit of random values
                b = upper limit of random values 

   Returned:    Pointer to the allocated region of memory. */
   
   vector       x;
   int		i;
   if (n < 1)
      nerror (100,"randvec",1, (float) n,"");
   x = vec (n,"");
   for (i = 1; i <= n; i++)
     x[i] = randu (a,b);
   return x; }
       
/* ------------------------------------------------------------------- */
cvector cvec (int n,char *s) {
/* ------------------------------------------------------------------- */

/* Description: Allocate space for an n by 1 complex vector u and
                initialize it.

   On entry:    n = number of components of vector u: subscripts
                    range from 1 to n.
                s = initialization string.  The values must be
                    in the order u[1].x, u[1].y, u[2].x, ...
                    and separated by commas or white space. If
                    less than m values are specified, the
                    remaining values are set to zero.

   Returned:    Pointer to the allocated region of memory. 
   
   Notes:       Complex vectors can also be allocated and initialized
                using the getcvec function. */

   cvector	x;         
   int		i;
   float	q;
   char		*c;
   
/* Allocate space for x */
   
   if (n < 1)
      nerror (100,"cvec",1, (float) n,"");
   q = (float) n*sizeof(complex);
   x = (complex *) malloc ((size_t) q);
   if (x == NULL)
      nerror (101,"cvec",0,q,"");
   x -= 1;

/* Initialize x using string s */   

   for (i = 1; i <= n; i++) {
      x[i].x = 0;
      x[i].y = 0 ; }
   for (i = 1,c = s; (i <= n) && *c; i++,c++) {
      x[i].x = (float) strtod (c,&c);
      c++;
      x[i].y = (float) strtod (c,&c); }
   return x; }

/* ------------------------------------------------------------------- */
void 	freevec (vector x) {
/* ------------------------------------------------------------------- */

/* Description: Release space allocated to x by vec, randvec or
                getvec.

   On entry:    x = vector whose memory area is to be released. */

   free (x + 1); }
   
/* ------------------------------------------------------------------- */
void 	freecvec (cvector x) {
/* ------------------------------------------------------------------- */

/* Description: Release space allocated to x by cvec or getcvec. 

   On entry:    x = vector whose memory area is to be released. */

   free (x + 1); }
   
/* ------------------------------------------------------------------- */
void 	zerovec (vector x,int n) {
/* ------------------------------------------------------------------- */

/* Description: Zero the contents of the n by 1 vector x. 

   On entry:    x = n by 1 vector to be zeroed.
                n = number of components of x

   On exit:     x = 0  */

   int		i;
   for (i = 1; i <= n; i++)
      x[i] = 0; }

/* ------------------------------------------------------------------- */
void 	copyvec (vector x,int n,vector y) {
/* ------------------------------------------------------------------- */

/* Description: Copy the n by 1 vector x into the vector y.

   On entry:    x = n by 1 vector to be copied.
                n = number of components of x to be copied
                
   On exit:     y = x */

   int		i;
   if (n < 1)
      nerror (100,"copyvec",0, (float) n,"");
   for (i = 1; i <= n; i++)
      y[i] = x[i]; }

/* ------------------------------------------------------------------- */
matrix mat (int m,int n,char *s) {
/* ------------------------------------------------------------------- */

/* Description: Allocate space for an m by n real matrix A and
                initialize it.

   On entry:    m = number of rows of matrix A: subscripts range
                    from 1 to m.
                n = number of columns of matrix A: subscripts range
                    from 1 to n.
                s = initialization string.  The values must be
                    in the row-wise order A[1][1], A[1][2], ...
                    and separated by commas or white space.  If
                    less than m*n values are specified, the
                    remaining values are set to zero.

   Returned:    Pointer to the allocated region of memory. 
   
   Notes:       Matrices can also be allocated and initialized using
                the randmat and getmat functions. */

   matrix       A;         
   int	    	i,j;
   float      	q;
   char		*c;   

/* Allocate space for A */

   if (m < 1)
      nerror (100,"mat",1, (float) m,"");
   if (n < 1)
      nerror (100,"mat",2, (float) n,"");
   q = (float) m*sizeof(float *);   
   A = (float **) malloc ((size_t) q);
   if (A == NULL)
      nerror (101,"mat",0,q,"");
   A -= 1;
   q = (float) n*sizeof(float);
   for (i = 1; i <= m; i++) {
      A[i] = (float *) malloc ((size_t) q);
      if (A[i] == NULL)
         nerror (101,"mat",0,q,""); 
      A[i] -= 1; }

/* Initialize A using string s */ 

   for (i = 1; i <= m; i++)
      for (j = 1; j <= n; j++)
         A[i][j] = 0;
   for (i = 1,c = s; (i <= m) && *c; i++)
      for (j = 1; (j <= n) && *c; j++,c++)
         A[i][j] = (float) strtod (c,&c); 
   return A; }
   
/* ------------------------------------------------------------------- */
matrix 	randmat (int m,int n,float a,float b) {
/* ------------------------------------------------------------------- */

/* Description: Allocate space for an m by n real matrix A and
                initialize it with random numbers uniformly
                distributed over the interval [a,b].

   On entry:    m = number of rows of matrix A: subscripts range
                    from 1 to m
                n = number of columns of matrix A; subscripts range    
                    from 1 to n.
                a = lower limit of random values
                b = upper limit of random values 

   Returned:    Pointer to the allocated region of memory. */

   matrix       A;	
   int		i,j;
   if (m < 1)
      nerror (100,"randmat",1, (float) m,"");
   if (n < 1)
      nerror (100,"randmat",2, (float) n,"");
   A = mat (m,n,"");
   for (i = 1; i <= m; i++)
     for (j = 1; j <= n; j++)
       A[i][j] = randu (a,b);
   return A; }

/* ------------------------------------------------------------------- */
cmatrix cmat (int m,int n,char *s) {
/* ------------------------------------------------------------------- */

/* Description: Allocate space for an m by n complex matrix A and
                initialize it.

   On entry:    m = number of rows of matrix A: subscripts range
                    from 1 to m.
                n = number of columns of matrix A: subscripts range
                    from 1 to n.
                s = initialization string.  The values must be
                    in the row-wise order A[1][1].x, A[1][1].y,
                    A[1][2].x, ... and separated by commas or
                    white space.  If less than m*n values are
                    specified, the remaining values are set to zero.

   Returned:    Pointer to the allocated region of memory. 
   
   Notes:       Complex matrices can also be allocated and initialized
                using the getcmat function. */

   cmatrix	A;         
   int		i,j;
   float      	q;
   char 	*c;
   
/* Allocate space for A */

   if (m < 1)
      nerror (100,"cmat",1, (float) m,"");
   if (n < 1)
      nerror (100,"cmat",2, (float) n,"");
   q = (float) m*sizeof(complex *);   
   A = (complex **) malloc ((size_t) q);
   if (A == NULL)
      nerror (101,"cmat",0,q,"");   
   A -= 1;
   q = (float) n*sizeof(complex);
   for (i = 1; i <= m; i++) {
      A[i] = (complex *) malloc ((size_t) q);
      if (A[i] == NULL)
         nerror (101,"cmat",0,q,""); 
      A[i] -= 1; }

/* Initialize A using string s */ 

   for (i = 1; i <= m; i++)
      for (j = 1; j <= n; j++) {
         A[i][j].x = 0;
         A[i][j].y = 0; }
   for (i = 1,c = s; (i <= m) && *c; i++)
      for (j = 1; (j <= n) && *c; j++,c++) {
         A[i][j].x = (float) strtod (c,&c);
         c++;
         A[i][j].y = (float) strtod (c,&c); } 
   return A; }

/* ------------------------------------------------------------------- */
void 	freemat (matrix A,int m) {
/* ------------------------------------------------------------------- */

/* Description: Release the space allocated to the m by n matrix A by
                mat, randmat, or getmat.

   On entry:    A = m by n matrix whose memory area is to be released.
                m = number of rows of A. */
                
   int		i;
   if (m < 1)
      nerror (100,"freemat",2, (float) m,"");
   for (i = 1; i <= m; i++)
      free (A[i] + 1);
   free (A + 1); }

/* ------------------------------------------------------------------- */
void 	freecmat (cmatrix A,int m) {
/* ------------------------------------------------------------------- */

/* Description: Release the space allocated to the m by n matrix A by
                cmat or getcmat.

   On entry:    A = m by n matrix whose memory area is to be released.
                m = number of rows of A. */

   int		i;
   if (m < 1)
      nerror (100,"freecmat",2, (float) m,"");
   for (i = 1; i <= m; i++)
      free (A[i] + 1);
   free (A + 1); }

/* ------------------------------------------------------------------- */
void 	zeromat (matrix A,int m,int n) {
/* ------------------------------------------------------------------- */

/* Description: Zero the contents of the m by n matrix A. 

   On entry:    A = m by n matrix to be zeroed.
                m = number of rows of A
                n = number or columns of A

   On exit:     A = 0  */

   int		i,j;
   if (m < 1)
      nerror (100,"zeromat",2, (float) m,"");
   if (n < 1)
      nerror (100,"zeromat",3, (float) n,"");
   for (i = 1; i <= m; i++)
      for (j = 1; j <= n; j++)
         A[i][j] = 0; }

/* ------------------------------------------------------------------- */
void 	copymat (matrix A,int m,int n,matrix B) {
/* ------------------------------------------------------------------- */

/* Description: Copy the m by n matrix A into the matrix B.

   On entry:    A = m by n matrix to be copied.
                m = number of rows to be copied
                n = number of columns to be copied

   On exit:     B = A */

   int		i,j;
   if (m < 1)
      nerror (100,"copymat",3, (float) m,"");
   if (n < 1)
      nerror (100,"copymat",4, (float) n,"");
   for (i = 1; i <= m; i++)
     for (j = 1; j <= n; j++)
        B[i][j]  = A[i][j]; }

        
               /*****************************/
               /*   VECTOR AND MATRIX I/O   */ 
               /*****************************/


/* ------------------------------------------------------------------- */
void	putvec	(vector x,int n,char *vname,char *fname, int mode) {
/* ------------------------------------------------------------------- */

/* Description: Write or append n by 1 real vector x to specified
                Matlab M-file.

   On entry:    x     = n by 1 real vector to be written
                n     = number of components of x
                vname = variable name to be used for x
                fname = file name 
                mode  = writing mode (0 = append, 1 = overwrite) */

   char         r[81];
   int		i;
   FILE		*f;

/* Check arguments */  

   add_m (fname,r);
   if (mode)
      f = fopen (r,"w");
   else
      f = fopen (r,"a");
   if (f == NULL) 
      nerror (102,"putvec",0,0,r);
   if (n < 1)
      nerror (100,"putvec",3, (float) n,"");

/* Write vector */
      
   fprintf (f,"%i;\n",n);   
   fprintf (f,"%s = [\n",vname);   
   for (i = 1; i <= n; i++)
      if (i < n)
         fprintf (f,"%g\n",x[i]);
      else   
         fprintf (f,"%g\n];\n",x[i]);

   fclose (f);
}


/* ------------------------------------------------------------------- */
void	putcvec (cvector x,int n,char *vname,char *fname,int mode) {
/* ------------------------------------------------------------------- */

/* Description: Write or append n by 1 complex vector x to specified
                Matlab M-file.

   On entry:    x     = n by 1 complex vector to be written
                n     = number of components of x
                vname = variable name to be used for x
                fname = file name. 
                mode  = writing mode (0 = append, 1 = overwrite) */

   char         r[81];     
   FILE		*f;
   int		i;

/* Check arguments */

   add_m (fname,r);
   if (mode)
      f = fopen (r,"w");
   else
      f = fopen (r,"a");
   if (f == NULL) 
      nerror (102,"putcvec",0,0,r);
   if (n < 1)
      nerror (100,"putcvec",3, (float) n,"");

/* Write vector */
      
   fprintf (f,"%i;\n",n);   
   fprintf (f,"%s = [\n",vname);   
   for (i = 1; i <= n; i++)
      if (i < n)
         fprintf (f,"%g + j*%g\n",x[i].x,x[i].y);
      else   
         fprintf (f,"%g + j*%g\n];\n",x[i].x,x[i].y);

   fclose (f); }


/* ------------------------------------------------------------------- */
void	putmat	(matrix A,int m,int n,char *vname,char *fname,int mode) {
/* ------------------------------------------------------------------- */

/* Description: Write or append m by n real matrix A to specified file.

   On entry:    A     = m by n real matrix to be written
                m     = number of rows of A  
                n     = number of columns of A 
                vname = variable name to be used for A
                fname = file name. 
                mode  = writing mode (0 = append, 1 = overwrite) */

   char         r[81];
   FILE		*f;
   int		i,j;

/* Check arguments */

   add_m (fname,r);
   if (mode)
      f = fopen (r,"w");
   else
      f = fopen (r,"a");
   if (f == NULL) 
      nerror (102,"putmat",0,0,r); 
   if (m < 1)
      nerror (100,"putmat",3, (float) m,""); 
   if (n < 1)
      nerror (100,"putmat",4, (float) n,"");

/* Write matrix */     

   fprintf (f,"%i;\n",m);   
   fprintf (f,"%i;\n",n);   
   fprintf (f,"%s = [\n",vname);   
   for (i = 1; i <= m; i++)
      for (j = 1; j <= n; j++)
      {
         if (((j % 7) == 0) && (j < n))
            fprintf (f,"%g ...\n",A[i][j]);
         else if (j < n) 
            fprintf (f,"%g ",A[i][j]);
         else if (i < m) 
            fprintf (f,"%g\n",A[i][j]);
         else
            fprintf (f,"%g\n];\n",A[i][j]);
      }     
   fclose (f); }


/* ------------------------------------------------------------------- */
void  putcmat (cmatrix A,int m,int n,char *vname,char *fname,int mode) {
/* ------------------------------------------------------------------- */

/* Description: Write or append m by n complex matrix A to specified file.

   On entry:    A     = m by n complex matrix to be written
                m     = number of rows of A  
                n     = number of columns of A 
                vname = variable name to be used for A
                fname = file name. 
                mode  = writing mode (0 = append, 1 = overwrite) */

   char         r[81];
   FILE		*f;
   int		i,j;

/* Check arguments */

   add_m (fname,r);
   if (mode)
      f = fopen (r,"w");
   else
      f = fopen (r,"a");
   if (f == NULL) 
      nerror (102,"putcmat",0,0,r); 
   if (m < 1)
      nerror (100,"putcmat",3, (float) m,""); 
   if (n < 1)
      nerror (100,"putcmat",4, (float) n,"");

/* Write matrix */      
      
   fprintf (f,"%i;\n",m);   
   fprintf (f,"%i;\n",n);   
   fprintf (f,"%s = [\n",vname);   
   for (i = 1; i <= m; i++)
      for (j = 1; j <= n; j++)
      {
         if (((j % 7) == 0) && (j < n))
            fprintf (f,"%g + j*%g ...\n",A[i][j].x,A[i][j].y);
         else if (j < n) 
            fprintf (f,"%g + j*%g ",A[i][j].x,A[i][j].y);
         else if (i < m)
            fprintf (f,"%g + j*%g\n",A[i][j].x,A[i][j].y);
         else         
            fprintf (f,"%g + j*%g\n];\n",A[i][j].x,A[i][j].y);
      }      
   fclose (f); }

/* ------------------------------------------------------------------- */
vector getvec	(FILE *f,int *n) {
/* ------------------------------------------------------------------- */

/* Description: Allocate space for an n by 1 real vector x, and input 
                its components from the stream f.

   On entry:    f = file pointer to a file that has been opened for
                    reading.

   On exit:     *n = number of components of vector x

   Returned:    Pointer to the real vector x. 

   Notes:       Real vectors can also be allocated and initialized using
                the vec and randvec functions.  getvec skips over
                comment lines starting with a semicolon in column one. */

   char		c[81];
   int		i;
   double	y;
   vector	x;
   if (f == NULL) 
      nerror (103,"getvec",0,0,"");
   scanline (f,'%',c);
   fscanf (f,"%i",&i);
   fgets (c,80,f);
   fgets (c,80,f);
   x = vec (i,"");
   *n = i;
   for (i = 1; i <= *n; i++)
   {
      fscanf (f,"%lf",&y);
      x[i] = (float) y;
   }

   fgets (c,80,f);
   fgets (c,80,f);

   return x;
}

/* ------------------------------------------------------------------- */
cvector getcvec (FILE *f,int *n) {
/* ------------------------------------------------------------------- */

/* Description: Allocate space for an n by 1 complex vector x, and input
                its components from the stream f.

   On entry:    f = file pointer to a file that has been opened for
                    reading.

   On exit:     *n = number of components of vector x.

   Returned:    Pointer to the complex vector x. 

   Notes:       Complex vectors can also be allocated and initialized 
                using the cvec function. getcvec skips over comment
                lines starting with a semicolon in column one. */

   char		c[81];
   int		i;
   double	yr,yi;
   cvector      x;
   if (f == NULL) 
      nerror (103,"getcvec",0,0,""); 
   scanline (f,'%',c);
   fscanf (f,"%i",&i);
   fgets (c,80,f);
   fgets (c,80,f);
   x = cvec (i,"");
   *n = i;
   for (i = 1; i <= *n; i++)
   {
      fscanf (f,"%lf %lfj",&yr,&yi);
      x[i].x = (float) yr;
      x[i].y = (float) yi;
   }
   fgets (c,80,f);
   fgets (c,80,f);
   return x;
}

/* ------------------------------------------------------------------- */
matrix getmat (FILE *f,int *m,int *n) {
/* ------------------------------------------------------------------- */

/* Description: Allocate space for an m by n real matrix A, and input 
                its components from the stream f.

   On entry:    f = file pointer to a file that has been opened for
                    reading.

   On exit:     *m = number of rows of matrix A
                *n = number of columns of matrix A

   Returned:    Pointer to the real matrix A. 

   Notes:       Real matrices can also be allocated and initialized using
                the mat and randmat functions. getmat skips over comment
                lines starting with a semicolon in column one. */

   char		c[81];
   int		i,j;
   double	a;
   matrix	A;
   if (f == NULL) 
      nerror (103,"getmat",0,0,""); 
   scanline (f,'%',c);   
   fscanf (f,"%i",&i);
   fgets (c,80,f);
   fscanf (f,"%i",&j);
   fgets (c,80,f);
   fgets (c,80,f);
   A = mat (i,j,"");
   *m = i;
   *n = j;   
   for (i = 1; i <= *m; i++)
      for (j = 1; j <= *n; j++)
      {
         fscanf (f,"%lf",&a);
         A[i][j] = (float) a;
         if (((j % 7) == 0) || (j == *n))
            fgets (c,80,f);
      } 
   fgets (c,80,f);
   return A;
}


/* ------------------------------------------------------------------- */
cmatrix getcmat (FILE *f,int *m,int *n) {
/* ------------------------------------------------------------------- */

/* Description: Allocate space for an m by n complex matrix A, and
                input its components from the stream f.

   On entry:    f = file pointer to a file that has been opened for
                    reading.

   On exit:     *m = number of rows of matrix A
                *n = number of columns of matrix A

   Returned:    Pointer to the complex matrix A. 

   Notes:       Complex matrices can also be allocated and initialized 
                using the cmat function. getcmat skips over comment 
                lines starting with a semicolon in column one. */

   char		c[81];
   int		i,j;
   double	ar,ai;
   cmatrix	A;
   if (f == NULL) 
      nerror (103,"getcmat",0,0,""); 
   scanline (f,'%',c);
   fscanf (f,"%i",&i);
   fgets (c,80,f);
   fscanf (f,"%i",&j);
   fgets (c,80,f);
   fgets (c,80,f);
   A = cmat (i,j,"");
   *m = i;
   *n = j;    
   for (i = 1; i <= *m; i++)
      for (j = 1; j <= *n; j++)
      {
         fscanf (f,"%lf %lfj",&ar,&ai);
         A[i][j].x = (float) ar;
         A[i][j].y = (float) ai;
         if (((j % 7) == 0) || (j == *n))
            fgets (c,80,f);
      }
   fgets (c,80,f);
   return A;
}

/* ------------------------------------------------------------------- */
void putcol (matrix A,int m,int n,int c,vector x) {
/* ------------------------------------------------------------------- */

/* Description: Copy the cth column of the m by n matrix A into the
                m by 1 vector x.

   On entry:    A = m by n real matrix
                m = number of rows of A  
                n = number of columns of A.
                c = subscript of column of A to be copied

   On exit:     x = m by 1 vector containing column c of A. */

   int		k;
   if (m < 1)
      nerror (100,"putcol",2, (float) m,"");
   if (n < 1)
      nerror (100,"putcol",3, (float) n,"");
   if ((c < 1) || (c > n))
      nerror (100,"putcol",4, (float) c,"");
   for (k = 1; k <= m; k++)
      A[k][c] = x[k]; }

/* ------------------------------------------------------------------- */
void getcol (matrix A,int m,int n,int c,vector x) {
/* ------------------------------------------------------------------- */

/* Description: Copy the m by 1 vector x into the cth column of the
                m by n matrix A.

   On entry:    A = m by n real matrix
                m = number of rows of A  
                n = number of columns of A.
                c = subscript of column of A
                x = m by 1 real vector to be copied

   On exit:     A = m by n matrix whose cth column is equals x. */

   int		k;
   if (m < 1)
      nerror (100,"getcol",2, (float) m,"");
   if (n < 1)
      nerror (100,"getcol",3, (float) n,"");
   if ((c < 1) || (c > n))
      nerror (100,"getcol",4, (float) c,"");
   for (k = 1; k <= m; k++)
      x[k] = A[k][c]; }

/* ------------------------------------------------------------------- */
void  add_m  (char *s,char *r)
/* ------------------------------------------------------------------- */
{
/* Description: Add a .m extension to a file name.

   On entry:    s = file name
   On exit:     r = file name with extension .m appended

   Note:        If s has an extension, it will be replaced.  */

   int     i = 0;
   strcpy (r,s);
   while ((r[i] != 0) && (r[i] != '.'))
      i++;
   strcpy (r+i,".m");
}
      
                      /**********************/
                      /*   MATRIX ALGEBRA   */ 
                      /**********************/

   
/* ------------------------------------------------------------------- */
void transpose (matrix A,int m,int n,matrix B) {
/* ------------------------------------------------------------------- */

/* Description: Compute the transpose of the m by n matrix A and store
                the result in B. 
                
   On entry:    A = m by n real matrix to be transposed.
                m = number of rows of A
                n = number of columns of A
 
   On exit:     B = n by m matrix containing the transpose of A.

   Notes:       When n = m, transpose can be called with B = A. */

   matrix	C;          
   int		i,j;
   if (m < 1)
      nerror (100,"transpose",3, (float) m,"");
   if (n < 1)
      nerror (100,"transpose",4, (float) n,"");
   C = mat (n,m,"");
   for (i = 1; i <= m; i++)
      for (j = 1; j <= n; j++) 
         C[j][i] = A[i][j];
   copymat (C,n,m,B);
   freemat (C,n); }

/* ------------------------------------------------------------------- */
float  	inner (vector x,vector y,int n) {
/* ------------------------------------------------------------------- */

/* Description: Compute the inner product of the n by 1 vector x with
                the n by 1 vecto y.  
                
   On entry:    x = n by 1 real vector
                y = n by 1 real vector
                n = number of components of x and y

   Returned:    The inner or dot product x'y. */

   float 	dot=0;
   int		i;
   if (n < 1)
      nerror (100,"inner",3, (float) n,"");
   for (i = 1; i <= n; i++)
      dot += x[i]*y[i];
   return dot; }

/* ------------------------------------------------------------------- */
void  	outer (vector x,vector y,int n,matrix A) {
/* ------------------------------------------------------------------- */

/* Description: Compute the outer product of the n by 1 vector x with
                the n by 1 vector y.  Store the result in the n by n
                matrix A.
                
   On entry:    x = n by 1 real vector
                y = n by 1 real vector
                n = number of components of x and y

   On exit:     A = n by n matrix xy'. */

   int		i,j;
   if (n < 1)
      nerror (100,"outer",4, (float) n,"");
   for (i = 1; i <= n; i++)
      for (j = 1; j <= n; j++)
         A[i][j] = x[i]*y[j]; }

/* ------------------------------------------------------------------- */
void 	vecmat (vector x,matrix A,int m,int n,vector b) {
/* ------------------------------------------------------------------- */

/* Description: Multiply the 1 by m vector x' by the m by n matrix
                A.  Store the result in the 1 by n vector b'.
                
   On entry:    x = m by 1 real vector
                A = m by n real matrix
                m = number of components of x and rows of A
                n = number of columns of A

   On exit:     b' = 1 by n vector x'A.

   Notes:       When n = m, vecmat can be called with b = x. */

   vector	c;
   int		i,j;
   if (m < 1)
      nerror (100,"vecmat",4, (float) m,"");
   if (n < 1)
      nerror (100,"vecmat",5, (float) n,"");
   c = vec (n,"");
   for (i = 1; i <= n; i++)
      for (j = 1; j <= m; j++) 
         c[i] += x[j]*A[j][i];
   copyvec (c,n,b);
   freevec (c); } 
             
/* ------------------------------------------------------------------- */
void 	matvec (matrix A,vector x,int m,int n,vector b) {
/* ------------------------------------------------------------------- */

/* Description: Multiply the m by n matrix A by the n by 1 vector
                x.  Store the result in the m by 1 vector b.
                
   On entry:    A = m by n real matrix
                x = n by 1 real vector
                m = number of rows of A
                n = number of columns of A and components of x

   On exit:     b = n by 1 vector Ax.

   Notes:       When n = m, matvec can be called with b = x. */

   vector 	c;	
   int		i,j;
   if (m < 1)
      nerror (100,"matvec",4, (float) m,"");
   if (n < 1)
      nerror (100,"matvec",5, (float) n,"");
   c = vec (m,"");
   for (i = 1; i <= m; i++)
      for (j = 1; j <= n; j++) 
         c[i] += A[i][j]*x[j];
   copyvec (c,m,b);
   freevec (c); } 
             
/* ------------------------------------------------------------------- */
void matmat (matrix A,matrix B,int m,int n,int p,matrix C) {
/* ------------------------------------------------------------------- */

/* Description: Multiply the m by n matrix A by the n by p matrix
                B.  Store the result in the m by p matrix C.
                
   On entry:    A = m by n real matrix
                B = n by p real matrix
                m = number of rows of A
                n = number of columns of A and rows of B
                p = number of columns B

   On exit:     C = m by p matrix AB

   Notes:       When n = m = p, matmat can be called with C = A.
                or C = B */

   matrix       D;	
   int		i,j,k;
   if (m < 1)
      nerror (100,"matmat",4,(float) m,"");
   if (n < 1)
      nerror (100,"matmat",5,(float) n,"");
   if (p < 1)
      nerror (100,"matmat",6,(float) p,"");
   D = mat (m,p,"");
   for (i = 1; i <= m; i++)
      for (j = 1; j <= p; j++) 
         for (k = 1; k <= n; k++)
            D[i][j] += A[i][k]*B[k][j];
   copymat (D,m,p,C);
   freemat (D,m); } 


/* ------------------------------------------------------------------- */
float minvec (vector x,int n,int *k) {
/* ------------------------------------------------------------------- */

/* Description: Find the minimum component of the n by 1 vector x and
                its subscript.
                
   On entry:    x = n by 1 vector
                n = number of components of x

   On exit:     *k = subscript of minimum component of x

   Returns:     Minimum component of x. */

   int		i;                
   if (n < 1)
      nerror (100,"minvec",2,(float) n,"");
   *k = 1;
   for (i = 1; i <= n; i++)
      if (x[i] < x[*k]) *k = i;
   return x[*k]; }
   
/* ------------------------------------------------------------------- */
float maxvec (vector x,int n,int *k) {
/* ------------------------------------------------------------------- */

/* Description: Find the maximum component of the n by 1 vector x and
                its subscript.
                
   On entry:    x = n by 1 vector
                n = number of components of x

   On exit:     *k = subscript of maximum component of x

   Returns:     Maximum component of x. */

   int		i;                
   if (n < 1)
      nerror (100,"maxvec",2,(float) n,"");
   *k = 1;
   for (i = 1; i <= n; i++)
      if (x[i] > x[*k]) *k = i;
   return x[*k]; }
   
/* ------------------------------------------------------------------- */
float minmat (matrix A,int m,int n,int *k,int *j) {
/* ------------------------------------------------------------------- */

/* Description: Find the minimum component of the m by n matrix A and
                its subscripts.
                
   On entry:    A = m by n matrix
                m = number of rows of A
                n = number of columns of A

   On exit:     *k = row subscript of minimum component of A
                *j = column subscript of minimum component of A    
   
   Returns:     Minimum component of A. */

   int		p,q;                
   if (m < 1)
      nerror (100,"minmat",2,(float) m,"");
   if (n < 1)
      nerror (100,"minmat",3,(float) n,"");
   *k = 1;
   *j = 1;
   for (p = 1; p <= m; p++)
      for (q = 1; q <= n; q++)
      if (A[p][q] < A[*k][*j]) {
         *k = p;
         *j = q; }
   return A[*k][*j]; }
   
   
/* ------------------------------------------------------------------- */
float maxmat (matrix A,int m,int n,int *k,int *j) {
/* ------------------------------------------------------------------- */

/* Description: Find the maximum component of the m by n matrix A and
                its subscripts.
                
   On entry:    A = m by n matrix
                m = number of rows of A
                n = number of columns of A

   On exit:     *k = row subscript of maximum component of A
                *j = column subscript of maximum component of A    
   
   Returns:     Maximum component of A. */

   int		p,q;
   if (m < 1)
      nerror (100,"maxmat",2,(float) m,"");
   if (n < 1)
      nerror (100,"maxmat",3,(float) n,"");
   *k = 1;
   *j = 1;
   for (p = 1; p <= m; p++)
      for (q = 1; q <= n; q++)
      if (A[p][q] > A[*k][*j]) {
         *k = p;
         *j = q; }
   return A[*k][*j]; }
   

                     /**************************/
                     /*   COMPLEX ARITHMETIC   */ 
                     /**************************/

             
/* ------------------------------------------------------------------- */
complex	cnum (float x,float y) {
/* ------------------------------------------------------------------- */

/* Description: Compute the complex number x + jy.
                
   On entry:    x = real part
                y = imaginary part

   Returns:     The complex number x + jy. */

   complex	u;
   u.x = x;
   u.y = y;
   return u; }

/* -------------------------------------------------------------------- */
complex	cadd (complex u,complex v) {
/* -------------------------------------------------------------------- */

/* Description: Compute the complex sum u + v.
                
   On entry:    u = complex number
                v = complex number

   Returns:     The sum u + v. */

   complex	z;
   z.x = u.x + v.x;
   z.y = u.y + v.y;
   return z; }

/* ------------------------------------------------------------------- */
complex	csub (complex u,complex v) {
/* ------------------------------------------------------------------- */

/* Description: Compute the complex difference u - v.
                
   On entry:    u = complex number
                v = complex number

   Returns:     The difference u - v. */

   complex	z;
   z.x = u.x - v.x;
   z.y = u.y - v.y;
   return z; }

/* ------------------------------------------------------------------- */
complex	cmul (complex u,complex v) {
/* ------------------------------------------------------------------- */

/* Description: Compute the complex product u*v. 
                
   On entry:    u = complex number
                v = complex number

   Returns:     The product u*v. */

   complex	z;
   z.x = u.x*v.x - u.y*v.y;
   z.y = u.x*v.y + u.y*v.x;
   return z; }

/* ------------------------------------------------------------------- */
complex	cdiv (complex u,complex v) {
/* ------------------------------------------------------------------- */

/* Description: Compute the complex quotient u/v.
                
   On entry:    u = complex number
                v = complex number

   Returns:     The quotient u/v. 

   Note:        An alternative way to code this is to set
                z.x + jz.y = (u.x + ju.y)/(v.x + jv.y).
                Multiply both sides by v, equate real and
                imaginary parts, and solve the resulting 
                linear system for z.x and z.y using Gaussian
                elimination. */ 
   
   complex	z;
   float	d = v.x*v.x + v.y*v.y,
		eps = epsilon();
   if (d < eps*eps)
      nerror (104,"cdiv",0,0,"");
   z.x = (u.x*v.x + u.y*v.y)/d;
   z.y = (u.y*v.x - u.x*v.y)/d;
   return z; }
   
/* ------------------------------------------------------------------- */
float cmag (complex u) {
/* ------------------------------------------------------------------- */

/* Description: Compute the magnitude of u.
                
   On entry:    u = complex number

   Returns:     The magnitude |u|. */

   return (float) sqrt(u.x*u.x + u.y*u.y); }

/* -------------------------------------------------------------------- */
float cphi (complex u) {
/* -------------------------------------------------------------------- */

/* Description: Compute the phase angle of u.
                
   On entry:    u = complex number

   Returns:     The phase angle arctan(u.y/u.x) */

   float	r = cmag (u),
		eps = epsilon();
   if (r < eps)
      return 0;
   else
      return (float) atan2 (u.y,u.x); }

/* ------------------------------------------------------------------- */
complex	csqrt (complex u) {
/* ------------------------------------------------------------------- */

/* Description: Compute the square root of u.
                
   On entry:    u = complex number

   Returns:     The square root sqrt(u). */

   complex	z;
   float	r   = (float) sqrt(cmag(u)),
		    phi = cphi(u)/2;
   double   x,y; 
            
   x = r*cos(phi);
   z.x = (float) x;
   y = r*sin(phi);
   z.y = (float) y;
   return z; }

/* ------------------------------------------------------------------- */
complex	cexp (complex u) {
/* ------------------------------------------------------------------- */

/* Description: Compute e raised to the power u.
                
   On entry:    u = complex number

   Returns:     The exponential exp(u). */

   complex	z;
   float	r = (float) exp(u.x);
   double   x,y;
   
   x = r*cos(u.y);
   z.x = (float) x;
   y = r*sin(u.y);
   z.y = (float) y;
   return z; }
   
/* ------------------------------------------------------------------- */
complex cpoly (complex u,vector a,int n) {
/* ------------------------------------------------------------------- */

/* Description: Evaluate a polynomial of degree n at u.
                
   On entry:    u = complex number
                a = (n+1) by 1 real vector of polynomial coefficients
                n = degree of polynomial
                
   Returns:     The polynomial f(u) where:

                   f(u) = a[n+1]*pow(u,n) + ... + a[2]*u + a[1] */

   int		k;
   complex	z;
   if (n < 0)
      nerror (100,"cpoly",2,(float) n,"");
   z.x = a[n+1];
   z.y = 0;            
   for (k = n; k >= 1; k--) {
      z = cmul(z,u);
      z.x += a[k]; }
   return z; }    

/* ------------------------------------------------------------------- */
void csort (cvector z,int n) {
/* ------------------------------------------------------------------- */

/* Description: Sort the components of the n by 1 complex vector z into
                decreasing magnitudes.
                
   On entry:    z = n by 1 complex vector
                n = number of components of z
                
   On exit:     z = n by 1 complex vector with components in order of
                decreasing magnitudes. */

   int		i,j;
   complex	u;
   if (n < 1)
      nerror (100,"csort",2,(float) n,"");
   for (i = 1; i <= n; i++)
      for (j = i; j <= n; j++)
         if (cmag(z[j]) > cmag(z[i])) {
            u = z[i];
            z[i] = z[j];
            z[j] = u; } }


                     /*****************************/
                     /*   RANDOM NUMBER FUNCTIONS */
                     /*****************************/
            
static long randi (long s);		/* local function */

/*---------------------------------------------------------------------*/
void randinit (long s) 
/*---------------------------------------------------------------------*/
{
/* Description: Seed the random number generator with the seed s.  Each
                seed produces a different random sequence.
                
   On entry:    s = long integer specifying the seed.  If s < 0,
                then a random seed based on the time of the day is
                generated.

   Notes:       If a random seed (s <= 0) is selected, then a
                different random sequence will be generated by
                functions randu and randg each time a program
                that uses them is executed. The default seed is
                s = 1 */
                
   if (s > 0)
      randi (s);
   else
      randi (-1);
}
      
/*---------------------------------------------------------------------*/
static long randi (long u) 
/*---------------------------------------------------------------------*/
{
/* Description: Generate the next random integer in the sequence.

   On entry:    u = opteating mode:

                    u > 0  =>  initialize random sequence with seed q
                    u = 0  =>  return next random number in sequence
                    u < 0  =>  initialize random sequence base on
                               time of day. */

   static long
                LSEED = 1;                 /* current random integer */
   long
                q = 127773L,
                r = 2836L,
                alpha = 16807L;

   if      (u > 0)
      LSEED = u; 
   else if (u < 0)
      LSEED = time (NULL);
   else
   {             
      LSEED = alpha*(LSEED % q) - r*(LSEED/q);
      if (LSEED < 0)
         LSEED += LRAND_MAX;
   }      
   return LSEED;
}

/*---------------------------------------------------------------------*/
float randu (float a,float b) {
/*---------------------------------------------------------------------*/

/* Description: Generate the next random real number uniformly
                distributed over the interval [a,b].

   On entry:    a = lower limit on value of random number
                b = upper limit on value of random number

   Notes:       The random number generator is seeded with the
                function randinit. */

   float	x,y;
   double   d; 
   static long	t[64];
   static int	rand_init = 0;
   int		i,n=64;
   
/* Initialize */

   y = (float) n;
   if (!rand_init) {
      for (i = 0; i < n; i++)
         t[i] = randi(0);    
      rand_init = 1; }

/*  Generate a random index i and output t[i], then update */

   d = y*randi(0)/(LRAND_MAX + 1.0);
   i = (int) d;
   x = (float) t[i];
   t[i] = randi(0);
   return (a + (b-a)*x/LRAND_MAX); }                

/*---------------------------------------------------------------------*/
float randg (float m,float s) {
/*---------------------------------------------------------------------*/

/* Description: Generate the next random real number with a
                Gaussian distribution with mean m and standard
                deviation s > 0.

   On entry:    m = desired mean value
                s = desired standard deviation (s > 0)

   Notes:       The random number generator is seeded with the
                function randinit. */

   float x1,x2;
   double y;
   x1 = randu (0,1);
   x2 = randu (0,1);
   y = sqrt(-2*log(x1))*cos(2*PI*x2);
   return (float) (m + s*y); }


                     /*****************************/
                     /*   MISCELANIOUS FUNCTIONS  */
                     /*****************************/

/* ------------------------------------------------------------------ */
float args (float x,float a,float b,int k,char *f) 
/* ------------------------------------------------------------------ */
{
/* Description: Check argument k of a function f to see that it is
                in range.

   On entry:    x = value of the argument
                a = lower limit on x
                b = upper limit on x 
                k = argument number 
                f = function name

   On exit:     args = x clipped to interval [a,b] */

   float    y = x;
   if ((x < a) || (x > b))
   {
      y = (x < a) ? a : b;
      printf ("\nArgument %i of function %s was outside the range [%g,%g].",
               k,f,a,b);
      printf ("\nIts value was %g. Enter 'q' to quit...\n",x);
      if(getchar() == 'q')
         exit(1);
   }
   return y;
}      
   
/* ------------------------------------------------------------------- */
float	epsilon (void) {
/* ------------------------------------------------------------------- */

/* Description: Compute the machine epsilon for type float.

   Returns:     The smallest number e such that 1 + e > 1. */

   float	e = 1,
		x;
   do {
      e /= 2; 
      x = 1 + e; }
   while (x > 1);
   return 2*e; }

/*------------------------------------------------------------------*/
void dec2bin  (unsigned x,char *b)
/*------------------------------------------------------------------*/
{
/* Description: Convert from decimal to binary.

   On entry:    x = decimal number to be converted 

   On exit:     b = string of length 17 containing binary digits */
   
   int        i;
   unsigned   y = (unsigned) pow(2,15);

   for (i = 0; i < 16; i++)
   {
       b[i] = (char) (x/y + '0');
       x -= (x/y)*y;
       y /= 2;
   }
   b[16] = 0;
}   

/* ------------------------------------------------------------------- */
int scanline (FILE *f,char c,char *s) {
/* ------------------------------------------------------------------- */

/* Description: Scan the data file pointed to by f to find the first
                line that is not a comment or an empty line.

   On entry:    f = pointer to a file that has been opened for reading
                c = character indicating the start of a comment 
   
   On exit:     s = string containing up to 80 characters of last comment
                    line (if present).
   
   Returns:     r = current line number in file, or -1 if file was just
                comments and empty lines. 

   Notes:       When scanline returns, the file pointer f is
                pointing to the start of the first line that is not
                a comment or an empty line. */

	int		ch,
				cmax = 80,
				r = 0;
	do
   {
      ch = fgetc(f);
      if      (ch == c)
      {
         fgets (s,cmax,f);
         if (s[strlen(s)-1] == '\n')
            s[strlen(s)-1] = 0;
         r++;
      }
      else if (ch == '\n')
         r++;
   }   
   while ((ch == c) || (ch == '\n'));
   if (!feof(f))
   {
      ungetc (ch,f);
      return r;
   }
   else   
      return -1;
}

/* ------------------------------------------------------------------- */
void remarks  (FILE *f,char *c) {
/* ------------------------------------------------------------------- */

/* Description: Write the string c as a comment line or sequence of
                comment lines to the file pointed to by f.
                
   On entry:    f = pointer to a file that has been opened for writing
                c = string containing comments.

   Notes:       A simicolon is inserted at the start of each comment line.
                A mult-line comment is possible by including newline
                characters (\n) in c. */

   char		*b;             
   if (strlen(c) > 0)
   {
      fputc (';',f);
      b = c;
      while (*b)
      {
         fputc (*b,f);
         if (*b == '\n')
            fputc (';',f);
         b++;
      }
      fputc ('\n',f);
   }
}

/* ------------------------------------------------------------------- */
float polynom (float x,vector a,int n) {
/* ------------------------------------------------------------------- */

/* Description: Evaluate an nth degree polynomial with coefficients
                a[1]..a[n+1] at x using Horner's rule:

                   f(x) = a[1] + a[2]*x + ... + a[n+1]*pow(x,n)

   On entry:    x = point of evaluation
                a = (n+1) by 1 vector of coefficients
		n = degree of polynomial (n >= 0)

   Returns:	f(x) */

   int		i;
   float	f;

/* Compute f(x) using nesting to minimize multiplications */

   if (n < 0)
      nerror (100,"polynom",3,(float) n,"");
   f = a[n+1];
   for (i = n; i >= 1; i--)
      f = f*x + a[i]; 
   return f; }

/* ------------------------------------------------------------------- */
float vecnorm (vector x,int n) {
/* ------------------------------------------------------------------- */

/* Descripton: Compute the infinity norm of the vector x:

               ||x|| = max{|x[1]|,|x[2]|,...,|x[n]|}

   On entry:   x = n by 1 vector
               n = number of components of x (n >= 1)

   Returned:   ||x|| 

   Notes:      To compute the Euclidean norm of x, use
               sqrt(inner(x,x,n)). */

   float	norm = 0;
   int		i;
   if (n < 1)
      nerror (100,"vecnorm",2,(float) n,"");
   for (i = 1; i <= n; i++)
      if (fabs(x[i]) > norm)
         norm = (float) fabs(x[i]);
   return norm; }




/* ------------------------------------------------------------------- */
svector svec (int m,int n,char *s) {
/* ------------------------------------------------------------------- */

/* Description: Allocate space for an m by 1 vector of strings of length
                n and initialize it.

   On entry:    m = number of rows of matrix A: subscripts range
                    from 1 to m.
                n = number of columns of matrix A: subscripts range
                    from 0 to n.
                s = initialization string.  The values must be
                    in the row-wise order A[1][1], A[1][2], ...
                    and separated by commas or white space.  If
                    less than m*n values are specified, the
                    remaining values are set to blank.

   Returned:    Pointer to the allocated region of memory. */
   
   svector      A;         
   int	    	i,j;
   float      	q;
   char		*c;   

/* Allocate space for A */

   if (m < 1)
      nerror (100,"svec",1, (float) m,"");
   if (n < 1)
      nerror (100,"svec",2, (float) n,"");
   q = (float) m*sizeof(char *);   
   A = (char **) malloc ((size_t) q);
   if (A == NULL)
      nerror (101,"svec",0,q,"");
   A -= 1;
   q = (float) n*sizeof(char);
   for (i = 1; i <= m; i++)
   {
      A[i] = (char *) malloc ((size_t) q);
      if (A[i] == NULL)
         nerror (101,"svec",0,q,"");
   } 

/* Initialize A using string s */ 

   for (i = 1; i <= m; i++)
   {
      for (j = 0; j < n; j++)
         A[i][j] = ' ';
      A[i][n] = 0;
   }   
   for (i = 1,c = s; (i <= m) && *c; i++)
      for (j = 0; (j < n) && *c; j++,c++)
         A[i][j] = *c; 
   return A; }
   

/* ------------------------------------------------------------------- */
svector getsvec (FILE *f,int *m,int *n) {
/* ------------------------------------------------------------------- */

/* Description: Allocate space for an m by 1 vector A of strings of
                length n and and input its components from the stream f.

   On entry:    f = file pointer to a file that has been opened for
                    reading.

   On exit:     *m = number of rows of matrix A
                *n = number of columns of matrix A

   Returned:    Pointer to the vector of strings A. */
 

   char		s[257];
   int		i,j,nmax = 256;
   svector      A;

   if (f == NULL) 
      nerror (103,"getsvec",0,0,""); 
   scanline (f,'%',s);

/* Find length of strings */

   fgets (s,nmax,f);
   j = strlen(s) + 1;

/* Find number of strings */

   i = 1;
   while (!feof(f))
   {
      fgets (s,nmax,f);
      i++;
   }
   i--;
   rewind (f);
   scanline (f,'%',s);

/* Allocate space and read in data */

   A = svec (i,j+1,"");
   *m = i;
   *n = j;   
   for (i = 1; i <= *m; i++)
      fgets (A[i],*n,f);
   return A;
}



