/* -------------------------------------------------------------------- */
/* File:     ode.c                                                      

   Contents: Ordinary differential equation functions 

   Uses:     nlib.c, linear.c */
/* -------------------------------------------------------------------- */

#if defined(DISPLAY_ODE)
   float stepsize;
#endif

/* ---------------------------------------------------------------------- */
/* Local functions */
/* ---------------------------------------------------------------------- */

static void   Coef (matrix J,int n,float h);
static void   Ft   (float t,vector x,int n,vector dfdt,
                    void (*f)(float t,vector x,int n,vector dx));
static void   Fx   (float t,vector x,int n,matrix dfdx,
                    void (*f)(float t,vector x,int n,vector dx));
static void   Efun (vector x,float alpha,float beta,int m,vector e,
                    float (*g)(float t,float y,float dy));

/* ---------------------------------------------------------------------- */
long rkf0 (vector x,float t0,float t1,int n,float eps,long q,float *e,
           void (*f)(float t,vector x,int n,vector dx)) {
/* ---------------------------------------------------------------------- */

/* Description: Use the fifth order Runge-Kutta Fehlberg method with
                automatic step size control to solve an n-dimensional
                system of ordinary differential equations:

                   dx(t)/dt = f(t,x(t))   

                over the time interval [t0,t1].
                   
   On entry:    x   = n by 1 initial conditon vector, x = x(t0)
                t0  = initial time
                t1  = final time
                n   = number of equations (n >= 1)
                eps = upper bound on local truncation error (eps >= 0)
                q   = maximum number of scalar function evaluations (q >= 1) 
                f   = name of user-supplied function which defines the
                      system of equations to be solved. The prototype of
                      f is:

                         void f (float t,vector x,int n,vector dx);

                      When f is called, it must evaluate the right-hand
                      side of the system of differential equations and
                      return the result in the n by 1 vector dx.

   On exit:     x  = n by 1 vector containing estimate of solution:
                     x = x(t1)
                *e = maximum local truncation error over [t0,t1]     
                    
   Returned:    r = number of scalar function evaluations.  If it is less
                    than the user-specified maximum, q, then the following
                    error criterion was satisfied by adjusting the step
                    size. Here ||E|| denotes the infinity norm of the
                    estimated local truncation error:

                       ||E|| < max(||x||,1)*eps  */
      
   double       d; 
   long			k = 0;			/* function evaluations */	
   static float 	h;	        /* current step size */
   float		gamma = 0.8f,	/* safety factor */
          		t = t0,			/* independent variable */
          		hmin,			/* minimum step size */
          		em = epsilon(),
                E0,E1,r,dt,Dt;
   vector		x1,y;
   
/* Initialize */

   if (n < 1)
      nerror (100,"rkf0",4,(float) n,"");
   else if (eps < 0)
      nerror (100,"rkf0",5,eps,"");
   else if (q < 1)
      nerror (100,"rkf0",6,(float) q,"");
   d = em*MAX(fabs(t0),fabs(t1));	/* smallest step */
   hmin = (float) d;
   x1 = vec (n,"");
   y = vec (n,"");
   *e = 0;

/* Adjust step size */

   if (fabs(h) < hmin)				/* first time */
      h = t1 - t0;
   if (h > 0)
      dt = MIN(MAX(h,hmin),t1-t0);		/* suggested step */
   else
      dt = MAX(MIN(h,-hmin),t1-t0);
   copyvec (x,n,x1); 

/* Solve the system */

   do {
      do {
         E1 = rkfstep (x1,t,t+dt,n,y,f);	/* actual error */
         k += 6*n; 
         E0 = MAX(vecnorm(y,n),1)*eps;		/* desired error */
         if (E1 <= E0) {
            t += dt;
            copyvec (y,n,x1); }
         r = MAX(em*em,E1);
      	 d = gamma*dt*pow(E0/r,0.20);	/* new step */
		 Dt = (float) d;

         #if defined(DISPLAY_ODE)
             stepsize = Dt;
         #endif

         if (Dt > 0)    			/* final step ? */
            dt = MIN(Dt,t1-t);
         else
            dt = MAX(Dt,t1-t); }
      while ((E1 > E0) && (fabs(Dt) >= hmin) && (k < q));
      *e = MAX(*e,E1); 
      if (E1 > E0) {
         t += dt;
         copyvec (y,n,x1); }
      if (fabs(Dt) < hmin)
         nerror (311,"rkf0",0,hmin,"");  }
   while ( ((h > 0) && (t < t1)) || ((h <= 0) && (t > t1)) );
  
/* Finalize */      

   h = dt;
   k = MIN(q,k);
   copyvec (y,n,x);
   freevec (y);
   freevec (x1);
   return k; }
   
/* ---------------------------------------------------------------------- */
long      rkf (vector x0,float t0,float t1,int n,int m,
               float eps,long q,vector t,matrix X,float *e, 
               void (*f)(float t,vector x,int n,vector dx)) {
/* ---------------------------------------------------------------------- */

/* Description: Use the fifth order Runge-Kutta Fehlberg method with
                automatic step size control to solve an n-dimensional
                system of ordinary differential equations:

                   dx(t)/dt = f(t,x(t))   ,   x(t0) = x0

                at m steps equally spaced over the time interval [t0,t1].

   On entry:    x   = n by 1 initial conditon vector, x = x(t0)
                t0  = initial time
                t1  = final time
                n   = number of equations (n >= 1)
                m   = number of solution points (m >= 2)  
                eps = upper bound on local truncation error (eps >= 0)
                q   = maximum number of scalar function evaluations (q >= 1) 
                f   = name of user-supplied function which defines the
                      system of equations to be solved. The prototype of
                      f is:

                         void f (float t,vector x,int n,vector dx);

                      When f is called, it must evaluate the right-hand
                      side of the system of differential equations and
                      return the result in the n by 1 vector dx.

   On exit:     t  = m by 1 vector containing solution times
                X  = n by m matrix containing solution trajectory. 
                     Tke kth column of X is the solution at time
                     t[k].
                *e = maximum local truncation error over [t0,t1]     

   Returned:    r = number of scalar function evaluations.  If it is less
                    than the user-specified maximum, q, then the following
                    error criterion was satisfied by adjusting the step
                    size. Here ||E|| denotes the infinity norm of the
                    estimated local truncation error:

                       ||E|| < max(||x||,1)*eps  */
      
   long		k = 0,			/* function evaluations */
		r; 
   int		i,j;
   float	E,dt;
   vector	x;
   
/* Initialize */

   if (n < 1)
      nerror (100,"rkf",4,(float) n,"");
   else if (m < 2)
      nerror (100,"rkf",5,(float) m,"");
   else if (eps < 0)
      nerror (100,"rkf",6,eps,"");
   else if (q < 1)
      nerror (100,"rkf",7,(float) q,"");
   x = vec (n,"");
   for (i = 1; i <= n;  i++) {
      x[i] = x0[i]; 
      X[i][1] = x0[i]; }
   dt = (t1 - t0)/(m-1);
   for (i = 1; i <= m; i++)
      t[i] = t0 + (i-1)*dt;
   *e = 0;
   
/* Solve the system */

   for (i = 2; i <= m; i++) {
      r = rkf0 (x,t[i-1],t[i],n,eps,q,&E,f);	
      k += r;
      *e = MAX(*e,E);
      if (k >= q)
         nerror (307,"rkf",0,0,"");
      for (j = 1; j <= n; j++)
         X[j][i] = x[j]; }

/* Finalize */

   freevec (x);
   return k; }

/* ---------------------------------------------------------------------- */
long bsext0 (vector x,float t0,float t1,int n,float eps,long q,
             float *e,void (*f)(float t,vector x,int n,vector dx)) {
/* ---------------------------------------------------------------------- */

/* Description: Use the Bulirsch-Stoer variable order extrapotion method
                to solve an n-dimensional system of ordinary differential
                equations:

                   dx(t)/dt = f(t,x(t))   

                over the time interval [t0,t1].
                   
   On entry:    x   = n by 1 initial conditon vector, x = x(t0)
                t0  = initial time
                t1  = final time
                n   = number of equations (n >= 1)
                eps = upper bound on local truncation error (eps >= 0)
                q   = maximum number of scalar function evaluations (q >= 1) 
                f   = name of user-supplied function which defines the
                      system of equations to be solved. The prototype of
                      f is:

                         void f (float t,vector x,int n,vector dx);

                      When f is called, it must evaluate the right-hand
                      side of the system of differential equations and
                      return the result in the n by 1 vector dx.

   On exit:     x  = n by 1 vector containing estimate of solution:
                     x = x(t1)
                *e = maximum local truncation error over [t0,t1]     
                    
   Returned:    r = number of scalar function evaluations.  If it is less
                    than the user-specified maximum, q, then the following
                    error criterion was satisfied by adjusting the
                    extrapolation level. Here ||E|| denotes the infinity
                    norm of the estimated local truncation error:

                       ||E|| < max(||x||,1)*eps  */

   int		k = 0,
            levels = 8,			             /* maximum level */
            r = (int) pow(2,levels) + 1,	 /* maximum steps */
		    i,j,m;
   long		p = 0;		
   float	h,r0,t;
   vector	dx,ev;
   matrix 	Ak,Ak1,Q;

/* Initialize */

   if (n < 1)
      nerror (100,"bsext0",4,(float) n,"");
   if (eps < 0) 
      nerror (100,"bsext0",5,eps,"");
   if (q < 1) 
      nerror (100,"bsext0",6,(float) q,"");
   Ak  = mat (n,levels,"");		/* row k of A */
   Ak1 = mat (n,levels,"");		/* row k-1 of A */
   Q = mat (r,n,"");			/* midpoint estimates */	
   dx = vec (n,"");			/* derivative vector */
   ev = vec (n,"");			/* error vector */
   copyvec (x,n,Q[1]);

   do {

/* Apply modified midpoint method */
      
      m = (int) pow (2,++k);
      h = (t1 - t0)/m;

/* First step */
      
      (*f)(t0,Q[1],n,dx);
      p += n;
      for (i = 1; i <= n; i++)
         Q[2][i] = Q[1][i] + h*dx[i];

/* Midpoint steps */

      for (j = 1; j < m; j++) {
         t = t0 + j*h;      
         (*f)(t,Q[j+1],n,dx);
         p += n;
         for (i = 1; i <= n; i++)
            Q[j+2][i] = Q[j][i] + 2*h*dx[i]; }

/* Last step */

      copymat (Ak,n,k,Ak1);      
      (*f)(t1,Q[m+1],n,dx);
      p += n;
      for (i = 1; i <= n; i++)
        Ak[i][1] = (Q[m+1][i] + Q[m][i] + h*dx[i])/2;                     
            
/* Update row k of matrix of estrapolated estimates */

      for (j = 2; j <= k; j++) {
         r0 = (float) pow (4,j-1);
         for (i = 1; i <= n; i++) { 	
            Ak[i][j] = (r0*Ak[i][j-1] - Ak1[i][j-1])/(r0 - 1);
            ev[i] = (Ak[i][j-1] - Ak1[i][j-1])/(r0 - 1); }
         #if defined(DISPLAY_ODE)
            printf ("\n %i %i",k,j);
            for (i = 1; i <= n; i++)
               printf (" %10.3e",ev[i]);  
         #endif
         } 
         #if defined(DISPLAY_ODE)   
            printf ("\n");
            wait();
         #endif
      *e = vecnorm (ev,n);
      for (i = 1; i <= n; i++)
         x[i] = Ak[i][k];
   }

   while ((k == 1) || (*e > MAX(vecnorm(x,n),1)*eps) && (k < levels)
           && (p < q)); 
   
/* Finalize */

   p = MIN(p,q);
   freevec (ev);
   freevec (dx);
   freemat (Q,r);
   freemat (Ak,n);
   freemat (Ak1,n);
   return p; }

/* ---------------------------------------------------------------------- */
long    bsext (vector x0,float t0,float t1,int n,int m,
               float eps,long q,vector t,matrix X,float *e, 
               void (*f)(float t,vector x,int n,vector dx)) {
/* ---------------------------------------------------------------------- */

/* Description: Use the Bulirsch-Stoer variable order extrapotion method
                to solve an n-dimensional system of ordinary differential
                equations:

                   dx(t)/dt = f(t,x(t))   ,   x(t0) = x0

                at m steps equally spaced over the time interval [t0,t1].

   On entry:    x   = n by 1 initial conditon vector, x = x(t0)
                t0  = initial time
                t1  = final time
                n   = number of equations (n >= 1)
                m   = number of solution points (m >= 2)  
                eps = upper bound on local truncation error (eps >= 0)
                q   = maximum number of scalar function evaluations (q >= 1) 
                f   = name of user-supplied function which defines the
                      system of equations to be solved. The prototype of
                      f is:

                         void f (float t,vector x,int n,vector dx);

                      When f is called, it must evaluate the right-hand
                      side of the system of differential equations and
                      return the result in the n by 1 vector dx.

   On exit:     t  = m by 1 vector containing solution times
                X  = n by m matrix containing solution trajectory. 
                     Tke kth column of X is the solution at time
                     t[k].
                *e = maximum local truncation error over [t0,t1]     

   Returned:    r = number of scalar function evaluations.  If it is less
                    than the user-specified maximum, q, then the following
                    error criterion was satisfied by adjusting the
                    extrapolation level. Here ||E|| denotes the infinity
                    norm of the estimated local truncation error:

                       ||E|| < max(||x||,1)*eps  */
            
   int		i,j;
   long		k = 0,     		/* function evaluations */
   		r;
   float	E,dt;
   vector	x;
   
/* Initialize */

   if (n < 1)
      nerror (100,"bsext",4,(float) n,"");
   else if (m < 2)
      nerror (100,"bsext",5,(float) m,"");
   else if (eps < 0)
      nerror (100,"bsext",6,eps,"");
   else if (q < 1)
      nerror (100,"bsext",7,(float) q,"");
   x = vec (n,"");
   for (i = 1; i <= n;  i++) {
      x[i] = x0[i]; 
      X[i][1] = x0[i]; }
   dt = (t1 - t0)/(m-1);
   for (i = 1; i <= m; i++)
      t[i] = t0 + (i-1)*dt;
   *e = 0;
   
/* Solve the system */

   for (i = 2; i <= m; i++) {
      r = bsext0 (x,t[i-1],t[i],n,eps,q,&E,f);
      *e = MAX(*e,E);
      for (j = 1; j <= n; j++)
         X[j][i] = x[j];
      k += r;
      if (k >= q) {
         nerror (307,"bsext",0,0,"");
         k = q;
         break; } }

/* Finalize */

   freevec (x);
   return k; }
      	
/* ---------------------------------------------------------------------- */
long stiff0 (vector x,float t0,float t1,int n,float eps,long q,float *e,   
             void (*f)(float t,vector x,int n,vector dx),
             void (*fx)(float t,vector x,int n,matrix dfdx),
             void (*ft)(float t,vector x,int n,vector dfdt)) {
/* ---------------------------------------------------------------------- */

/* Description: Use the semi-implicit Bulirsch-Stoer variable order
                extrapolation method to solve a stiff n-dimensional
                system of ordinary differential equations:

                   dx(t)/dt = f(t,x(t))   

                over the time interval [t0,t1].
                   
   On entry:    x   = n by 1 initial conditon vector, x = x(t0)
                t0  = initial time
                t1  = final time
                n   = number of equations (n >= 1)
                eps = upper bound on local truncation error (eps >= 0)
                q   = maximum number of scalar function evaluations (q >= 1) 
                f   = name of user-supplied function which defines the
                      system of equations to be solved.
                fx  = name of an optional user-supplied function that
                      defines the Jacobian matrix J = df(t,x)/dx
                ft  = name of an optional user-supplied function that
                      defines df(t,x)/dt.  The prototypes of user-supplied
                      functions are as follows:

                         void f  (float t,vector x,int n,vector dx);
                         void fx (float t,vector x,int n,matrix dfdx);
                         void ft (float t,vector x,int n,vector dfdt);

                      When f is called, it should evaluate the right-hand
                      side of the system of differential equations and
                      return the result in the n by 1 vector dx. When fx
                      is called, it should evaluate the n by n matrix of
                      partial derivatives of f(t,x) with respect to x. That
                      is, dfdx[k][j] = df[k]/dx[j] for 1 <= k,j <= n. When
                      ft is called, it should evaluate the n by 1 vector of
                      partial derivatives of f(t,x) with respect to t. That
                      is, dfdt[k] = df[k]/dt for 1 <= k <= n. The functions
                      fx and ft are optional in the sense that stiff0 can be
                      called with the arguments fx and/or ft set to the
                      symbolic constant NULL defined in the header file
                      <stdio.h>. When this is done, the partial derivatives
                      are approximated numerically by stiff0 using using
                      central differences. Thus the user does not have to
                      supply the functions in this case. 

   On exit:     x  = n by 1 vector containing estimate of solution:
                     x = x(t1)
                *e = maximum local truncation error over [t0,t1]     
                    
   Returned:    r = number of scalar function evaluations.  If it is less
                    than the user-specified maximum, q, then the following
                    error criterion was satisfied by adjusting the
                    extrapolation level.  Here ||E|| denotes theinfinity
                    norm of the estimated local truncation error:

                       ||E|| < max(||x||,1)*eps  */

   int      s = 0,
		    k = 0,			/* starting level */
   	        levels = 8,		/* maximum level */
            r = (int) pow(2,levels)+1,	/* maximum steps */
		    i,j,m,p;
   float	h,r0,t;
   vector	dx,ev,v,b;
   matrix 	Ak,Ak1,Q,J;
		
/* Initialize */

   if     (n < 1)
      nerror (100,"stiff0",4,(float) n,"");
   else if (eps < 0) 
      nerror (100,"stiff0",5,eps,"");
   else if (q < 1) 
      nerror (100,"stiff0",6,(float) q,"");
   Ak  = mat (n,levels,"");		/* row k of A */
   Ak1 = mat (n,levels,"");		/* row k-1 of A */
   Q = mat (r,n,"");			/* midpoint estimates */	
   dx = vec (n,"");			/* derivative vector */
   ev = vec (n,"");			/* error vector */
   v = vec (n,"");			/* linear system solution */ 
   b = vec (n,"");			/* right-hand side */
   J = mat (n,n,"");			/* Jacobian matrix */ 
   copyvec (x,n,Q[1]);
   do {

/* Apply semi-implicit midpoint method */
      
      m = (int) pow (2,++k);
      h = (t1 - t0)/m;

/* First step */

      (*f)(t0,Q[1],n,dx);
      s += n;
      if (fx == NULL) {
         Fx(t0,Q[1],n,J,f);
         s += 2*n*n; }
      else {
         (*fx)(t0,Q[1],n,J);
         s += n*n; }
      if (ft == NULL) {
         Ft(t0,Q[1],n,b,f);
         s += 2*n; }
      else {
         (*ft)(t0,Q[1],n,b);
         s += n; } 
      for (i = 1; i <= n; i++)
         b[i] = h*(h*b[i] + dx[i]);
      Coef (J,n,h);
      ludec (J,b,n,v); 
      for (i = 1; i <= n; i++)
         Q[2][i] = Q[1][i] + v[i];
         
/* Midpoint steps */

      for (j = 1; j < m; j++) {
         t = t0 + j*h;   
         (*f)(t,Q[j+1],n,b);
         s += n;
         if (fx == NULL) {
            Fx(t,Q[j+1],n,J,f);
            s += 2*n*n; }
         else { 	
            (*fx)(t,Q[j+1],n,J);
            s += n*n; }
         for (i = 1; i <= n; i++) {
            for (p = 1; p <= n; p++)
              b[i] -= J[i][p]*(Q[j+1][p] - Q[j][p]);
            b[i] *= 2*h; }
         Coef (J,n,h);
         ludec (J,b,n,v); 
         for (i = 1; i <= n; i++)
            Q[j+2][i] = Q[j][i] + v[i]; }

/* Last step */

      (*f)(t1,Q[m+1],n,dx);
      if (fx == NULL) {
         Fx(t1,Q[m+1],n,J,f);
         s += 2*n*n; }
      else {	
         (*fx)(t1,Q[m+1],n,J);
         s += n*n; }
      if (ft == NULL) {
         Ft(t0,Q[1],n,b,f);
         s += 2*n; }
      else {
         (*ft)(t0,Q[1],n,b);
         s += n; }
      for (i = 1; i <= n; i++)
         b[i] = h*(h*b[i] + dx[i]);
      Coef (J,n,h);
      ludec (J,b,n,v); 
      copymat (Ak,n,k,Ak1);      
      for (i = 1; i <= n; i++)
         Ak[i][1] = (Q[m][i] + Q[m+1][i] + v[i])/2;
         
/* Update row k of matrix of extrapolated estimates */

      for (j = 2; j <= k; j++) {
         r0 = (float) pow (4,j-1);
         for (i = 1; i <= n; i++) { 	
            Ak[i][j] = (r0*Ak[i][j-1] - Ak1[i][j-1])/(r0 - 1);
            ev[i] = (Ak[i][j-1] - Ak1[i][j-1])/(r0 - 1); }
         #if defined(DISPLAY_ODE)
            if (j == 2)
               printf ("\nInterval: [%.7g, %.7g]",t0,t1);
            printf ("\n %i %i",k,j);
            for (i = 1; i <= n; i++)
               printf (" %10.3e",ev[i]);
            if (j == k) {
               printf ("\n");
               wait(); }    
         #endif
      }
      *e = vecnorm (ev,n);
      for (i = 1; i <= n; i++)
         x[i] = Ak[i][k]; }

   while ((k == 1) || (*e > MAX(vecnorm(x,n),1)*eps) && (k < levels)
          && (s < q)); 
   
/* Finalize */

   s = MIN(s,q);
   freemat (J,n);
   freevec (b);
   freevec (v);
   freevec (ev);
   freevec (dx);
   freemat (Q,r);
   freemat (Ak,n);
   freemat (Ak1,n);
   return s; }

/* ---------------------------------------------------------------------- */
long    stiff   (vector x0,float t0,float t1,int n,int m,
                 float eps,long q,vector t,matrix X,float *e,   
                 void (*f)(float t,vector x,int n,vector dx),
                 void (*fx)(float t,vector x,int n,matrix dfdx),
		 void (*ft)(float t,vector x,int n,vector dfdt)) {
/* ---------------------------------------------------------------------- */

/* Description: Use the semi-implicit Bulirsch-Stoer variable order
                extrapotion method to solve a stiff n-dimensional
                system of ordinary differential equations:

                   dx(t)/dt = f(t,x(t))   ,   x(t0) = x0

                at m step equally spaced over the interval [t0,t1].
                   
   On entry:    x0  = n by 1 initial conditon vector
                t0  = initial time
                t1  = final time
                n   = number of equations (n >= 1)
                m   = number of solution points (m >= 2)
                
                eps = upper bound on local truncation error (eps >= 0)
                q   = maximum number of scalar function evaluations (q >= 1) 
                f   = name of user-supplied function which defines the
                      system of equations to be solved.
                fx  = name of an optional user-supplied function that
                      defines the Jacobian matrix J = df(t,x)/dx
                ft  = name of an optional user-supplied function that
                      defines df(t,x)/dt.  The prototypes of user-supplied
                      functions are as follows:

                         void f  (float t,vector x,int n,vector dx);
                         void fx (float t,vector x,int n,matrix dfdx);
                         void ft (float t,vector x,int n,vector dfdt);

                      When f is called, it should evaluate the right-hand
                      side of the system of differential equations and
                      return the result in the n by 1 vector dx. When fx
                      is called, it should evaluate the n by n matrix of
                      partial derivatives of f(t,x) with respect to x. That
                      is, dfdx[k][j] = df[k]/dx[j] for 1 <= k,j <= n. When
                      ft is called, it should evaluate the n by 1 vector of
                      partial derivatives of f(t,x) with respect to t. That
                      is, dfdt[k] = df[k]/dt for 1 <= k <= n. The functions
                      fx and ft are optional in the sense that stiff can be
                      called with the arguments fx and/or ft set to the
                      symbolic constant NULL defined in the header file
                      <stdio.h>. When this is done, the partial derivatives
                      are approximated numerically by stiff using using
                      central differences. Thus the user does not have to
                      supply the functions in this case. 

   On exit:     t  = m by 1 vector containing solution times
                X  = n by m matrix containing solution trajectory. 
                     Tke kth column of X is the solution at time
                     t[k].
                *e = maximum local truncation error over [t0,t1]     

   Returned:    r = number of scalar function evaluations.  If it is less
                    than the user-specified maximum, q, then the following
                    error criterion was satisfied by adjusting the
                    extrapolation level.  Here ||E|| denotes theinfinity
                    norm of the estimated local truncation error:

                       ||E|| < max(||x||,1)*eps  */

   long		p = 0,  		/* function evaluations */
		r;
   int		i,j;
   float	E,dt;
   vector	x;
   
/* Initialize */

   if (n < 1)
      nerror (100,"stiff",4,(float) n,"");
   else if (m < 2)
      nerror (100,"stiff",5,(float) m,"");
   else if (eps < 0)
      nerror (100,"stiff",6,eps,"");
   else if (q < 1)
      nerror (100,"stiff",7,(float) q,"");
   x = vec (n,"");
   for (i = 1; i <= n;  i++) {
      x[i] = x0[i]; 
      X[i][1] = x0[i]; }
   dt = (t1 - t0)/(m-1);
   for (i = 1; i <= m; i++)
      t[i] = t0 + (i-1)*dt;
   *e = 0;
   
/* Solve the system */

   for (i = 2; i <= m; i++) {
      r = stiff0 (x,t[i-1],t[i],n,eps,q,&E,f,fx,ft);
      *e = MAX(*e,E);
      for (j = 1; j <= n; j++)
         X[j][i] = x[j];
      p += r;
      if (p >= q) {
         nerror (307,"stiff",0,0,"");
         p = q;
         break; } }

/* Finalize */

   freevec (x);
   return p; }

/* ---------------------------------------------------------------------- */
int bvp (float alpha,float beta,float a,float b,float eps,  
         int m,int q,vector x,vector y,float *e,
         float (*g)(float x,float y,float dy)) {
/* ---------------------------------------------------------------------- */

/* Description: Use the finite difference method to solve the following
                boundary value problem where y' = dy/dx:

                   y'' = g(x,y,y')  ,   y(alpha) = a,   y(beta) = b

   On entry:    alpha = initial value of x
                beta  = final value of x
                a     = initial value of y,
                b     = final value of y
                eps   = upper bound on difference between estimates 
                m     = number of interior solution points (m >= 1)
                q     = maximum number of iterations (q >= 1)
                g     = name of a user-supplied function which defines the
                        second-order ordinary differential equation to be
                        solved. The prototype of the function g is

                           float g (float x,float y,float dy);

                        When g is called it should return the value
                        y''= f(x,y,dy) where dy = y'.
                        
   On exit:     x  = (m+2) by 1 vector containing equally-spaced
                      independent variables
                     
                y  = (m+2) by 1 vector containing solution points
                *e = norm of difference between last two solution
                     estimates.

   Returned:    r = number of iterations performed. If it is less than the
                    user-specified maximum q, then the following termination
                    criterion was satisfied where dy is the change in y
                    between successive iterations:

                       ||dy|| < eps */

   int		k = 0,
 		    i,j;   
   float	dy = 0.001f,
		    h;
   vector	y0,z0,z1;
   matrix	J;
   
/* Initialize */   

   if (eps < 0)
      nerror (100,"bvp",5,eps,"");   
   if (m < 1)
      nerror (100,"bvp",6,(float) m,"");
   if (q < 1)
      nerror (100,"bvp",7,(float) q,"");   
   h = (beta - alpha)/(m + 1);
   y0 = vec (m+2,"");
   z0 = vec (m,""); 
   z1 = vec (m,""); 
   J = mat (m,m,"");   
   for (i = 1; i <= (m+2); i++)
   {
      x[i] = alpha + (i-1)*h;
      y[i] = a + (b-a)*(i-1)/(m+1);
   }   
  
/* Solve differential equation */

   do {

/* Evaluate Jacobian matrix using central differences */

      copyvec (y,m+2,y0); 
      for (j = 1; j <= m; j++)
      {
         y0[j+1] = y[j+1] + dy;
         Efun (y0+1,alpha,beta,m,z0,g);
         y0[j+1] = y[j+1] - dy;
         Efun (y0+1,alpha,beta,m,z1,g);
         for (i = 1; i <= m; i++) 
            J[i][j] = (z0[i] - z1[i])/(2*dy); 
      }
            
/* Solve linear algebraic system */

      Efun (y+1,alpha,beta,m,z0,g);
      for (i = 1; i <= m; i++)
         z0[i] *= -1;
      ludec (J,z0,m,z1);
      *e = vecnorm(z1,m);
      for (i = 1; i <= m; i++)
         y[i+1] += z1[i]; 
      k++; }

   while ((*e >= eps) && (k < q));
   
/* Finalize */

   freevec (y0);
   freevec (z0);
   freevec (z1);
   freemat (J,m);
   return k; }

/* ---------------------------------------------------------------------- */
static void Efun (vector z,float alpha,float beta,int m,vector e,
                   float (*g)(float x,float y,float dy)) {
/* ---------------------------------------------------------------------- */

/* Description: This function is used by bvp.  Newton's vector method
                is used to find the m by 1 vector z such that
                Efun = 0.  The value of Efun is returned in
                the m by 1 vector e. */

   float	h = (beta - alpha)/(m + 1),
		x,dz;
   int		i;	
   for (i = 1; i <= m; i++) {
      x = alpha + i*h;
      dz = (z[i+1] - z[i-1])/(2*h);
      e[i] = z[i+1] - 2*z[i] + z[i-1] - h*h*g(x,z[i],dz); }  }

   
/* ---------------------------------------------------------------------- */
static void Coef (matrix J,int n,float h) {
/* ---------------------------------------------------------------------- */

/* Description: Compute n by n coefficient matrix A = I - hJ given
                J. Return the result in J. */  

   int		i,j;
   for (i = 1; i <= n; i++)
      for (j = 1; j <= n; j++) {
         J[i][j] *= -h;
         if (j == i)
             J[i][j] += 1; } }

/* ---------------------------------------------------------------------- */
static void Ft (float t,vector x,int n,vector dfdt,
                 void (*f)(float t,vector x,int n,vector dx)) {
/* ---------------------------------------------------------------------- */

/* Description: Estimate df(t,x)/dt numerically using central differences.
                Return the result in the n by 1 vector dfdt. */

   int		i;
   float	dt;
   vector	y = vec (n,"");
   dt = (float) pow(epsilon(),1.0/3.0);
   (*f) (t+dt,x,n,dfdt);
   (*f) (t-dt,x,n,y);
   for (i = 1; i <= n; i++) 
      dfdt[i] = (dfdt[i] - y[i])/(2*dt);
   freevec (y); }
 	
/* ---------------------------------------------------------------------- */
static void    Fx (float t,vector x,int n,matrix dfdx,
                   void (*f)(float t,vector x,int n,vector dx)) {
/* ---------------------------------------------------------------------- */

/* Description: Estimate the Jacobian matrix J = df(t,x)/dx numerically
                using central differences. Return the result in the
                n by n matrix dfdx. */

   int		i,j;
   float	d;
   vector	x0 = vec (n,""),
                x1 = vec (n,""), 
                x2 = vec (n,""); 
   d = (float) pow(epsilon(),1.0/3.0);             
   copyvec (x,n,x0);
   for (j = 1; j <= n; j++) {
      x0[j] = x[j] + d;  	
      (*f)(t,x0,n,x1);
      x0[j] = x[j] - d;
      (*f)(t,x0,n,x2);
      x0[j] = x[j];
       for (i = 1; i <= n; i++)
         dfdx[i][j] = (x1[i] - x2[i])/(2*d); }
   freevec (x2);
   freevec (x1);
   freevec (x0); }

/* ---------------------------------------------------------------------- */
float rkfstep (vector x0,float t0,float t1,int n,vector x1,
               void (*f)(float t,vector x,int n,vector dx)) {
/* ---------------------------------------------------------------------- */

/* Description: Perform one step of the fifth order Runge-Kutta Fehlberg
                method to solve an n-dimensional system of ordinary
                differential equations:

                dx(t)/dt = f(t,x(t)),        x(t0) = x0 

   On entry:    x0 = n by 1 initial condition vector, x0 = x(t0)
                t0 = initial time
                t1 = final time
                n  = number of equations (n >= 1)
                f  = name of user-supplied function which defines the
                     system of equations to be solved. The prototype of
                     f is:

                        void f (float t,vector x,int n,vector dx);

                     When f is called, it must evaluate the right-hand
                     side of the system of differential equations and
                     return the result in the n by 1 vector dx.

   On exit:     x1 = n by 1 vector containing estimate of solution: 
                     x1 = x(t1)

   Returned:    r = the infinity norm of the local truncation error E
                    at time t1. */
                    
   int		i;
   float	E,
    		h = t1 - t0;
   double   d;
   vector	x,e;
   matrix	K;
   
/* Initialize */
   
    if (n < 1)
      nerror (100,"rkfstep",4,(float) n,"");
   K = mat (6,n,"");
   x = vec (n,"");
   e = vec (n,"");
      
/* Evaluate derivatives at points inside [t,t+h] */

   (*f)(t0,x0,n,K[1]);

   for (i = 1; i <= n; i++)  
   {
      d = x0[i] + h*K[1][i]/4.;
	  x[i] = (float) d;
   }
   (*f)(t0+h/4.0f,x,n,K[2]);

   for (i = 1; i <= n; i++)
   {
      d = x0[i] + h*(3.*K[1][i] + 9.*K[2][i])/32.;
	  x[i] = (float) d;
   }
   (*f)(t0+3.0f*h/8.0f,x,n,K[3]);     

   for (i = 1; i <= n; i++)
   {
      d = x0[i] + h*(1932.*K[1][i] - 7200.*K[2][i] + 
		     7296.*K[3][i])/2197.;
	  x[i] = (float) d;
   }
   (*f)(t0+12.0f*h/13.0f,x,n,K[4]);

   for (i = 1; i <= n; i++)
   {
      d = x0[i] + h*(439.*K[1][i]/216. - 8.*K[2][i] + 3680.*K[3][i]/513.
          - 845.*K[4][i]/4104.);
	  x[i] = (float) d;
   }
   (*f)(t1,x,n,K[5]);

   for (i = 1; i <= n; i++)
   {
      d = x0[i] + h*(-8.*K[1][i]/27. + 2.*K[2][i] - 3544.*K[3][i]/2565.
             + 1859.*K[4][i]/4104. - 11.*K[5][i]/40.);
	  x[i] = (float) d;
   }
   (*f)(t0+h/2.0f,x,n,K[6]);

/* Compute x1 = x(t+h) */

   for (i = 1; i <= n; i++)
   {
      d = x0[i] + h*(16.*K[1][i]/135. + 6656.*K[3][i]/12825.
              + 28561.*K[4][i]/56430. - 9.*K[5][i]/50.
              + 2.*K[6][i]/55.);
	  x1[i] = (float) d;
   }

/* Compute error E */

   for (i = 1; i <= n; i++) 
   {
      d = h*(K[1][i]/360. - 128.*K[3][i]/4275.
             - 2197.*K[4][i]/75240. + K[5][i]/50.
             + 2.*K[6][i]/55.);
	  e[i] = (float) d;
   }
   E = vecnorm (e,n);          

/* Finalize */

   freevec (e);
   freevec (x);
   freemat (K,6); 
   return E; }



