/* -------------------------------------------------------------------- */
/* File:     optim.c                                                      

   Contents: Optimization functions 

   Uses:     nlib.c */
/* -------------------------------------------------------------------- */

vector	a_limit,			/* used by q_limit and anneal */
	b_limit;			/* used by q_limit and anneal */
                
/* -------------------------------------------------------------------- */
/* Local Functions */ 
/* -------------------------------------------------------------------- */

static void  step   (vector x,float h,vector d,int n,vector q);

static float fmu    (vector x,float mu,int n,int r,
                     int s,float (*f)(vector x,int n),
                     void (*p)(vector x,int n,int r,vector y),
                     void (*q)(vector x,int n,int s,vector y));

static int  gradfmu (vector x,float mu,int n,int r,int s,
                     vector df,float (*f)(vector x,int n),
                     void (*p)(vector x,int n,int r,vector y),
                     void (*q)(vector x,int n,int s,vector y));

static void  q_limit (vector x,int n,int s,vector y);

/* -------------------------------------------------------------------- */
int	conjgrad (vector x,float eps,int n,int m,int v, 
                  int *ev,float (*f)(vector x,int n),
                  void (*g)(vector x,int t,vector df)) {
/* -------------------------------------------------------------------- */

/* Description: Use the Fletcher-Reeves version of the conjugate
                gradient method to solve the following n-dimensional
                unconstrained optimization problem:

                   minimize: f(x)

   On entry:    x   = n by 1 vector containing initial guess
                eps = error tolerance used to terminate search (eps >= 0)
                n   = number of independent variables (n >= 1)
                m   = maximum number of iterations (m >= 1)
                v   = order of method:

                         v        method
                         ---------------------------
                         1   steepest descent
                         n   full conjugate gradient
                         ---------------------------  

                f   = name of objective function: f(x)
                g   = name of objective gradient function: df(x)/dx

                      The prototypes of f and g are:

                         float f (vector x,int n);
                         void  g (vector x,int n,vector df);

                      When f is called with the n by 1 vector x, it must
                      return the objective value f(x). When g is called
                      it must compute the n by 1 gradient vector: df[k] 
                      = df(x)/dx[k], 1 <= k <= n;  The argument g is 
                      optional in the sense that conjgrad can be called
                      with g set to the symbolic constant NULL in
                      <stdio.h>.  When this is done, the gradient of f of
                      is approximated numerically using central
                      differences in which case a user-supplied function
                      is NOT required.

   On exit:      x   = n by 1 solution vector
                 *ev = number of scalar function evaluations

   Returned:     k = number of iterations performed. If it is less than
                     the user-specified maximum, m, then the following
                     convergence criterion was satisfied where em 
                     denotes the machine epsilon and h is the step
                     length:

                        (||df(x)/dx|| < eps) or (h*||x|| < em) 

   Notes:       If f(x) is quadratiac, conjgrad should converge in at
                most n iterations. */

   vector	d,g1;			
   float	h = 1,			/* step length */
                a,b,c,                  /* three-point pattern */
                gamma,                  /* norm of gradient */
                delta,			/* norm of direction d */
                e,em,r,beta,dx;
   int		j = 0,			/* iteration counter */
                k = 0,
                eval = 0,		/* function evaluations */
                err,			/* descent direction code */ 
                i;
   
/* Initialize */

   if      (eps < 0)
      nerror (100,"conjgrad",2,eps,"");
   else if (n < 1)
      nerror (100,"conjgrad",3,(float) n,"");
   else if (m < 1)
      nerror (100,"conjgrad",4,(float) m,"");
   else if ((v < 1) || (v > n))
      nerror (100,"conjgrad",5,(float) v,"");
   em = epsilon();   
   e = (float) (2*sqrt(em));   
   d = vec (n,"");        
   g1 = vec (n,"");
      
   do {

/* Compute gradient and its norm */

      r = inner (g1,g1,n);
      if (g != NULL) {
         (*g)(x,n,g1);
         eval += n; }
      else   
         eval += gradfmu (x,0,n,0,0,g1,f,NULL,NULL);
      gamma = vecnorm (g1,n);
      
/* Compute search direction */
         
      if (k < 1)
         for (i = 1; i <= n; i++)
            d[i] = -g1[i];
      else {
         beta = inner(g1,g1,n)/r;	
         for (i = 1; i <= n; i++)
            d[i] = -g1[i] + beta*d[i]; }
      delta = vecnorm (d,n);
            
/* Perform a line search along direction d */

      eval += bracket (delta,x,d,0,n,0,0,&a,&b,&c,&err,f,NULL,NULL);
      if (!err) {

         if (g != NULL)
            eval += bisectd (x,d,a,c,e,0,n,0,0,&h,f,g,NULL,NULL);
         else
            eval += golden (x,d,a,c,e,0,n,0,0,&h,f,NULL,NULL);
         dx = h*vecnorm(d,n);
         step (x,h,d,n,x);

/* Update indices, check for restart */
   
         j++;
         k++;
         if (k == v) 
            k = 0; } }

   while ((j <= m) && (gamma > eps) && (dx > em*vecnorm(x,n)) && !err);
        
/* Finalize */

   *ev = eval;
   freevec (d);
   freevec (g1);
   return j; }

/* -------------------------------------------------------------------- */
int	dfp  (vector x,float eps,int n,int m,int v,
              int *ev,float (*f)(vector x,int n),
              void (*g)(vector x,int t,vector df)) {
/* -------------------------------------------------------------------- */

/* Description: Use the Davidon-Fletcher-Powell method with self-scaling
                to solve the following n-dimensional unconstrained
                optimization problem:

                   minimize: f(x)

   On entry:    x   = n by 1 vector containing initial guess
                eps = error tolerance used to terminate search (eps >= 0)
                n   = number of independent variables (n >= 1)
                m   = maximum number of iterations (m >= 1)
                v   = order of method:

                         v       method
                         --------------------
                         1   steepest descent
                         n   full DFP method
                         --------------------  

                f   = name of objective function: f(x)
                g   = name of objective gradient function: df(x)/dx

                      The prototypes of f and g are:

                         float f (vector x,int n);
                         void  g (vector x,int n,vector df);

                      When f is called with the n by 1 vector x, it must
                      return the objective value f(x). When g is called
                      it must compute the n by 1 gradient vector: df[k]
                      = df(x)/dx[k], 1 <= k <= n;  The argument g is
                      optional in the sense that dfp can be called with
                      g set to the symbolic constant NULL in <stdio.h>.
                      When this is done, the gradient of f of is
                      approximated numerically using central differences
                      in which case a user-supplied function is NOT
                      required.

   On exit:      x   = n by 1 solution vector
                 *ev = number of scalar function evaluations

   Returned:     k = number of iterations performed. If it is less than
                     the user-specified maximum, m, then the following
                     convergence criterion was satisfied where em 
                     denotes the machine epsilon and h is the step
                     length:

                        (||df(x)/dx|| < eps) or (h*||x|| < em) 

   Notes:       If f(x) is quadratiac, dfp should converge in at most
                n iterations. */

   matrix 	Q;			/* inverse Hessian */ 
   vector	dx,dg,g1,d,r;
   float	h = 1,			/* step length */
   		a,b,c,			/* three-point pattern */
		beta,			/* scale factor */
		gamma,			/* norm of gradient */
		delta,			/* norm of direction */
   		e,em,b1,b2,s;
   int		j = 0,			/* iteration counter */
   		k = 0,
   		eval = 0,		/* function evaluations */
                err,			/* descent direction code */
		i,p;
   
/* Initialize */

   if      (eps < 0)
      nerror (100,"dfp",2,eps,"");
   else if (n < 1)
      nerror (100,"dfp",3,(float) n,"");
   else if (m < 1)
      nerror (100,"dfp",4,(float) m,"");
   else if ((v < 1) || (v > n))
      nerror (100,"dfp",5,(float) v,"");
   em = epsilon();
   e = (float) (2*sqrt(em));
   dx = vec (n,"");
   dg = vec (n,"");
   g1 = vec (n,"");
   d = vec (n,"");
   r = vec (n,"");
   Q = mat (n,n,"");
   for (i = 1; i <= n; i++)
      Q[i][i] = 1;
   if (g != NULL) {
      (*g)(x,n,g1);
      eval += n; }
   else 
     eval += gradfmu (x,0,n,0,0,g1,f,NULL,NULL);
  
   do {

/* Perform line search along d = -Qg */

      zerovec (d,n);
      for (i = 1; i <= n; i++)
         for (p = 1; p <= n; p++)
            d[i] -= Q[i][p]*g1[p];
      delta = vecnorm (d,n);
      eval += bracket (delta,x,d,0,n,0,0,&a,&b,&c,&err,f,NULL,NULL);
      if (!err) {

         if (g != NULL)
            eval += bisectd (x,d,a,c,e,0,n,0,0,&h,f,g,NULL,NULL);
         else
            eval += golden (x,d,a,c,e,0,n,0,0,&h,f,NULL,NULL);
         step (x,h,d,n,x);
         for (i = 1; i <= n; i++)
            dx[i] = h*d[i];
         s = vecnorm (dx,n);
         
/* Compute new gradient */

         copyvec (g1,n,dg);
         if (g != NULL) {
            (*g)(x,n,g1);
            eval += n; }
         else 
            eval += gradfmu (x,0,n,0,0,g1,f,NULL,NULL);
         gamma = vecnorm (g1,n);
         for (i = 1; i <= n; i++)
            dg[i] = g1[i] - dg[i];

/* Update estimate Q of inverse of Hessian matrix */

         zerovec (r,n);
         for (i = 1; i <= n; i++)
            for (p = 1; p <= n; p++)
               r[i] += Q[i][p]*dg[p];
         b1 = inner (r,dg,n);
         b2 = inner (dx,dg,n);
         beta = b2/b1;				/* self-scaling */
         for (i = 1; i <= n; i++)
            for (p = 1; p <= n; p++)
               Q[i][p] = beta*(Q[i][p] - r[i]*r[p]/b1) + dx[i]*dx[p]/b2;  
     
/* Update indices, check for restart */
   
         j++;
         k++;
         if (k == v) {
      	   k = 0;
      	   zeromat (Q,n,n);
           for (i = 1; i <= n; i++)
              Q[i][i] = 1; } } }

   while ((j <= m) && (gamma > eps) && (s > em*vecnorm(x,n)) && !err);
        
/* Finalize */

   *ev = eval;
   freevec (dx);
   freevec (dg);
   freevec (g1);
   freevec (d);
   freevec (r);
   freemat (Q,n);
   return j; }


/* -------------------------------------------------------------------- */
int	penalty  (vector x,float eps,float mu,int n,int r,int s,
                  int m,int *ev,float (*f)(vector x,int n),
                  void (*p)(vector x,int n,int r,vector y),
                  void (*q)(vector x,int n,int s,vector y)) {
/* -------------------------------------------------------------------- */

/* Description: Use the self-scaled DFP method method with penalty
                functions to solve the following n-dimensional
                constrained optimization problem:

                   minimize:   f(x)
                   subject to: p(x) = 0
                               q(x) >= 0

   On entry:    x   = n by 1 vector containing initial guess
                eps = error tolerance used to terminate search (eps >= 0)
                mu  = penalty parameter (mu >= 0)
                n   = number of independent variables (n >= 1)
                r   = number of inequality constraints (r >= 0)
                s   = number of inequality constraints (s >= 0) 
                m   = maximum number of iterations (m >= 1)
                f   = name of objective function: f(x)
                p   = name of equality constraint function: p(x) = 0
                q   = name of equality constraint function: q(x) >= 0

                      The prototypes of f, p, and q are:

                         float f (vector x,int n);
                         void  p (vector x,int n,int r,vector y);
                         void  q (vector x,int n,int s,vector y);

                      When f is called, it must return the value of the
                      objective function f(x). When p is called, it must
                      take the n by 1 vector x and compute the r by 1
                      equality constraint vector y = p(x).  When q is,
                      called, it must take the n by 1 vector x and
                      compute the s by 1 inequality constraint vector
                      y = q(x).  If r = 0, then penalty should be called
                      with p replaced by the symbolic constant NULL in
                      <stdio.h>.  A similar remark holds for the
                      argument q when s = 0.  Consequently, penalty can
                      be used to solve the following special cases:

                         ---------------------------------------------
                         r = 0, p = NULL =>  no equality constraints
                         s = 0, q = NULL =>  no inequality constraints
                         ---------------------------------------------
         
   On exit:      x   = n by 1 solution vector
                 *ev = number of scalar function evaluations

   Returned:     k = number of iterations performed. If it is less than
                     the user-specified maximum, m, then the following
                     convergence criterion was satisfied where em 
                     denotes the machine epsilon, and h is the step
                     length:

                        (||dF(x)/dx|| < eps) or (h*||x|| < em) 
 
                     Here F(x) = f(x) = mu*(P(x) + Q(x)) is the
                     generalized objective function.   P(x) is the
                     penalty function associated with the equality
                     constraint, p(x) = 0, and Q(x) is the penalty
                     function associated with the inequality constraint,
                     q(x) >= 0. */
   
   matrix 	Q;			/* estimate of inverse Hessian */ 
   vector	dx,dg,g1,d,g2;
   float	h = 1,			/* step length */
                a,b,c,			/* three-point pattern */
                beta,			/* scale factor */
                gamma,			/* norm of gradient */
                delta,			/* norm of direction d */
                e,em,b1,b2;
   int		j = 0,			/* iteration counter */
                 k = 0,
                eval = 0,		/* function evaluations */  
                err,			/* descent direction code */
                i,t;
   #if defined(DISPLAY_OPTIM)
      float y;
   #endif

   
/* Initialize */

   if      (eps < 0)
      nerror (100,"penalty",2,eps,"");
   else if (mu < 0)
      nerror (100,"penalty",3,mu,"");
   else if (n < 1)
      nerror (100,"penalty",4,(float) n,"");
   else if (r < 0)
      nerror (100,"penalty",5,(float) r,"");
   else if (s < 0)
      nerror (100,"penalty",6,(float) s,"");
   else if (m < 1)
      nerror (100,"penalty",7,(float) m,"");
   em = epsilon();
   e = (float) (2*sqrt(em));
   dx = vec (n,"");
   dg = vec (n,"");
   g1 = vec (n,"");
   g2 = vec (n,"");
   d = vec (n,"");
   Q = mat (n,n,"");
   for (i = 1; i <= n; i++)
      Q[i][i] = 1;
   eval += gradfmu (x,mu,n,r,s,g1,f,p,q);
   
   do {

/* Perform line search along d = -Qg */

      zerovec (d,n);
      for (i = 1; i <= n; i++)
         for (t = 1; t <= n; t++)
            d[i] -= Q[i][t]*g1[t];
      delta = vecnorm (d,n);
            
/* Perform a line search along direction d */

      eval += bracket (delta,x,d,mu,n,r,s,&a,&b,&c,&err,f,p,q);
      if (!err) {

         eval += golden (x,d,a,c,e,mu,n,r,s,&h,f,p,q); 
         #if defined(DISPLAY_OPTIM)
            y = fmu (x,mu,n,r,s,f,p,q);
         #endif
         step (x,h,d,n,x);
         for (i = 1; i <= n; i++)
            dx[i] = h*d[i]; 
         
/* Compute new gradient */

         copyvec (g1,n,dg);
         eval += gradfmu (x,mu,n,r,s,g1,f,p,q);
         gamma = vecnorm (g1,n);
         #if defined(DISPLAY_OPTIM)
            printf ("\n(k h gamma df) = (%5i %10.3g %10.3g %10.3g)",
                     k,h,gamma,fmu(x,mu,n,r,s,f,p,q)-y);
         #endif
         for (i = 1; i <= n; i++)
            dg[i] = g1[i] - dg[i];

/* Update estimate Q of inverse of Hessian matrix */

         zerovec (g2,n);
         for (i = 1; i <= n; i++)
            for (t = 1; t <= n; t++)
               g2[i] += Q[i][t]*dg[t];
         b1 = inner (g2,dg,n);
         b2 = inner (dx,dg,n);
         beta = b2/b1;
         for (i = 1; i <= n; i++)
            for (t = 1; t <= n; t++)
               Q[i][t] = beta*(Q[i][t] - g2[i]*g2[t]/b1) +
                         dx[i]*dx[t]/b2; 
 
/* Update indices, check for restart */
   
         j++;
         k++;
         if (k == n) {
            k = 0;
      	    zeromat (Q,n,n);
            for (i = 1; i <= n; i++)
               Q[i][i] = 1; } } }
 
   while ((j < m) && (gamma > eps) && !err);
        
/* Finalize */

   *ev = eval;
   freevec (dx);
   freevec (dg);
   freevec (g1);
   freevec (d);
   freevec (g2);
   freemat (Q,n);
   return j; }

/* -------------------------------------------------------------------- */
int	anneal   (vector x,float f0,float f1,float eps,float eta,
                  float gamma,float mu,int n,int m,vector a,vector b,
                  int *ev,float (*f)(vector x,int n)) {
/* -------------------------------------------------------------------- */

/* Description: Use the simulated annealing method to solve the
                following n-dimensional constrained optimization
                problem:

                   minimize:   f(x)
                   subject to: a <= x <= b 
         
                The function anneal first performs a global search using 
                a simulated annealing method, and then performs a local
                search using the function dfp.

   On entry:    x     = n by 1 vector containing initial guess
                f0    = lower bound on f(x)
                f1    = upper bound on f(x) (f1 > f0)
                eps   = global error tolerance (eps >= 0)
                eta   = local error tolerance (eta >= 0)
                gamma = localizaton parameter (0 < gamma < 1)  
                mu    = penalty parameter (mu >= 0)
                n     = number of independent variables (n >= 1)
                m     = maximum number of iterations (m >= 1)
                a     = n by 1 vector of lower bounds on x
                b     = n by 1 vector of upper bounds on x (b[k] >= a[k])
                f     = name of objective function: f(x)
                        The prototypes of f is:

                           float f (vector x,int n);

                        When f is called, it must return the value of the
                        objective function f(x).

   On exit:      x   = n by 1 solution vector
                 *ev = number of scalar function evaluations

   Returned:     k = number of iterations performed. Iterations for the
                 initial global search continue until (f(x) - f0) <
                 eps*(f1 - f0) or m iterations are performed.  Iterations
                 for the final local search continue until m additional
                 iterations are performed or the following criterion is
                 satisfied where h is the step length and em is the
                 machine epsilon:

                    (||df(x)/dx|| < eta) or (h*||x|| < em)

                 Normally, anneal is called with:

                     0 < eta << eps << 1. */

   vector	g1,			/* gradient of f */
                x1,			/* new x */
                x2;			/* optimal x so far */
   float	df,			/* change in f(x) */
                p,			/* probability of using x1 */
                s,			/* standard deviation of dx */
                z,F0,F1,F2,
                Df,alpha,delta,
                d = 0;
   int		j = 0,			/* iteration counter */
                eval = 0,		/* function evaluations */
    		i,k;

#if defined(DISPLAY_OPTIM)    		
   matrix	X = mat (2,5000,"");
   X[1][1] = x[1];
   X[2][1] = x[2];
#endif
    		
/* Initialize */

   if      (f1 <= f0)
      nerror (100,"anneal",3,f1,""); 
   else if (eps < 0)
      nerror (100,"anneal",4,eps,"");
   else if (eta < 0)
      nerror (100,"anneal",5,eta,"");
   else if ((gamma <= 0) || (gamma >= 1))
      nerror (100,"anneal",6,gamma,"");
   else if (mu < 0)
      nerror (100,"anneal",7,mu,"");   
   else if (n < 1)
      nerror (100,"anneal",8,(float) n,"");
   else if (m < 1)
      nerror (100,"anneal",9,(float) m,"");
   for (i = 1; i <= n; i++)
      if (b[i] < a[i])
         nerror (100,"anneal",11,b[i],"");

   g1 = vec (n,"");
   x1 = vec (n,"");
   x2 = vec (n,"");
   a_limit = vec (n,"");
   b_limit = vec (n,"");
   alpha = (float) log(2)/gamma;
   F0 = (*f)(x,n);
   eval++;
   copyvec (x,n,x2);
   F2 = F0;
   Df = F0 - f0;
   for (i = 1; i <= n; i++) {
      a_limit[i] = a[i];
      b_limit[i] = b[i];	
      d = MAX (d,b[i]-a[i]); }
    
   do {
      
/* Generate a random step */

         s = gamma*d*Df/(f1 - f0);
         for (i = 1; i <= n; i++) {
            x1[i] = x[i] + randg (0,s);
            x1[i] = MIN(b[i],MAX(a[i],x1[i])); }
         F1 = (*f)(x1,n);
         eval++; 
         df = F1 - F0;
      
/* Should we use it? */

         if (df < 0) {				/* a descent */
            copyvec (x1,n,x);
            F0 = F1;
            if (F1 < F2) {
               copyvec (x1,n,x2);
               F2 = F1; }
            j++; }
         else {
            p = (float) exp(-alpha*df/Df);
            z = randu (0,1);
            if (p > z) {
               copyvec (x1,n,x);
               F0 = F1;
               j++; } } 

/* Check termination criterion */

      #if defined(DISPLAY_OPTIM)
         X[1][j+1] = x[1];
         X[2][j+1] = x[2];  
      #endif
      
      Df = F0 - f0;
      delta = Df/(f1 - f0); }

   while ((delta> eps) && (j < m));

/* Perform a local search */

   copyvec (x2,n,x);
   i = penalty  (x,eta,mu,n,0,2*n,m,&k,f,NULL,q_limit);
   eval += k; 
   
   #if defined(DISPLAY_OPTIM)
      printf ("\n\nj,x = %i,[%.7f,%.7f]",j,x2[1],x2[2]); 
      printf ("\nf(x) = %.7f",(*f)(x2,n));
      X[1][j+2] = x[1];
      X[2][j+2] = x[2];
      wait();
      graphxy (X[1],X[2],j+2,
              "Simulated Annealing Trajectory with \\gamma = 0.1", 
              "x_1","x_2","fig684");
      freemat (X,2);
   #endif
      
/* Finalize */

   *ev = eval;
   freevec (g1);
   freevec (x1);
   freevec (x2);
   freevec (a_limit);
   freevec (b_limit);
   return (j+i); }


   

/* Local Functions */
   
/* -------------------------------------------------------------------- */
static void step (vector x,float h,vector d,int n,vector q) {
/* -------------------------------------------------------------------- */

/* Description: Take a step from x along direction d a distance h.
                Store the result in the n by 1 vector q.

   On entry:    x = n by 1 start vector             
                h = step size
                d = n by 1 direction vector
                n = number of components (n >= 1) 

   On exit	q = x + h*d */              

   int		i;
   for (i = 1; i <= n; i++)
      q[i] = x[i] + h*d[i]; }
  
/* -------------------------------------------------------------------- */
static float fmu (vector x,float mu,int n,int r,
                  int s,float (*f)(vector x,int n),
                  void (*p)(vector x,int n,int r,vector y),
                  void (*q)(vector x,int n,int s,vector y)) {
/* -------------------------------------------------------------------- */

/* Description: Compute the generalized objective function:

                   F(x) = f(x) + mu*(P(x) + Q(x))

   On entry:    x  = n by 1 vector
                mu = penalty paramter (mu >= 0)
                n  = dimension of x
                r  = number of equality constratains (r >= 0)
                s  = number of inequality constraints (s >= 0)
                f  = name of objective function: minimize: f(x)
                p  = name of equality constraint function: p(x) = 0
                q  = name of inequality constraint function: q(x) >= 0

                     The prototypes of f, p, and q are:

                        float f (vector x,int n);
                        void p (vector x,int n,int r,vector y);
                        void q (vector x,int n,int r,vector y);

                     When f is called with n by 1 vector x, it must
                     return f(x).  When p is called with n by 1 vector
                     x, it must compute r by 1 vector y = p(x). When
                     q is called with n by 1 vector x, it must
                     compute s by 1 vector y = q(x).

   Returned:    F(x) = f(x) + mu*(P(x) + Q(x)) where P(x) and Q(x)
                are the penalty terms: 

                     P(x) = p'(x)p(x)
                     Q(x) = min(0,q(x))'min(0,q(x)) */
  
   vector	y;             
   float	F;
   int		i;
   
   F = (*f)(x,n);			/* objective function */
   if (mu <= 0)
      return F;
   
   if (r > 0) {				/* equality constraints */ 
      y = vec(r,"");
      (*p)(x,n,r,y);
      if (vecnorm(y,r) < sqrt(INFINITY)/(r*mu)) 
         for (i = 1; i <= r; i++)
            F += mu*y[i]*y[i]; 
      else {
         F = INFINITY/r;
         freevec (y);
         return F; }   
      freevec (y); }

   if (s > 0) {				/* inequality constraints */
      y = vec(s,"");
      (*q)(x,n,s,y);
      if (vecnorm(y,s) < sqrt(INFINITY)/(s*mu)) 
         for (i = 1; i <= s; i++)
            F += mu*MIN(0,y[i])*MIN(0,y[i]);
      else 
         F = INFINITY/s;
      freevec (y); }

   return F; }   
      
/* -------------------------------------------------------------------- */
static int  gradfmu (vector x,float mu,int n,int r,int s,
                     vector df,float (*f)(vector x,int n),
                     void (*p)(vector x,int n,int r,vector y),
                     void (*q)(vector x,int n,int s,vector y)) {
/* -------------------------------------------------------------------- */

/* Description: Numerically approximate the gradient of the generalized
                objective function F(x) using central differences.

                   F(x) = f(x) + mu*(P(x) + Q(x))

   On entry:    x   = n by 1 vector
                mu = penalty paramter (mu >= 0)
                n  = dimension of x
                r  = number of equality constratains (r >= 0)
                s  = number of inequality constraints (s >= 0)
                f  = name of objective function: minimize: f(x)
                p  = name of equality constraint function: p(x) = 0
                q  = name of inequality constraint function: q(x) >= 0

                     The prototypes of f, p, and q are:

                        float f (vector x,int n);
                        void p (vector x,int n,int r,vector y);
                        void q (vector x,int n,int r,vector y);

                     When f is called with n by 1 vector x, it must
                     return f(x).  When p is called with n by 1 vector
                     x, it must compute r by 1 vector y = p(x). When
                     q is called with n by 1 vector x, it must
                     compute s by 1 vector y = q(x).

   On exit:	df = n by 1 gradient vector: df[k] = df(x)/dx[k]
    
   Returned:    k = number of scalar function evalutions */

   int		i;
   float	eps;
   eps = (float) pow (epsilon(),1.0/3.0);
   for (i = 1; i <= n; i++) {
      x[i] += eps;
      df[i] = fmu (x,mu,n,r,s,f,p,q);
      x[i] -= 2*eps;
      df[i] = (df[i] - fmu (x,mu,n,r,s,f,p,q))/(2*eps);
      x[i] += eps; }
   return 2*n; }

/* -------------------------------------------------------------------- */
int  bracket (float h,vector x,vector d,float mu,int n,int r,
              int s,float *a,float *b,float *c,int *err,
              float (*f)(vector x,int n),
              void (*p)(vector x,int n,int r,vector y),
              void (*q)(vector x,int n,int s,vector y)) {
/* -------------------------------------------------------------------- */

/* Description:  Construct a three-point pattern {a,b,c} such that
                 a < b < c, F(b) < F(a), and F(b) <= F(c) where:

                    F(y) = f(x+y*d) + mu*(P(x+y*d) + Q(x+y*d))

                 The interval [a,c] then contains the minimum of a
                 unimodal function F(y).  Here P(x) is a penalty
                 function associated with the r equality constraints,
                 p(x) = 0, and Q(x) is a penalty function associated
                 with the s inequality constraints, q(x) >= 0.        

   On entry:     h  = initial estimate of step size (h > 0)
                 x  = n by 1 start vector
                 d  = n by 1 search direction vector
                 mu = penalty paramter (mu >= 0)
                 n  = dimension of x
                 r  = number of equality constratains (r >= 0)
                 s  = number of inequality constraints (s >= 0)
                 f  = name of objective function: minimize: f(x)
                 p  = name of equality constraint function: p(x) = 0
                 q  = name of inequality constraint function: q(x) >= 0

                      The prototypes of f, p, and q are:

                         float f (vector x,int n);
                         void p (vector x,int n,int r,vector y);
                         void q (vector x,int n,int r,vector y);

                      When f is called with n by 1 vector x, it must
                      return f(x).  When p is called with n by 1 vector
                      x, it must compute r by 1 vector y = p(x). When
                      q is called with n by 1 vector x, it must
                      compute s by 1 vector y = q(x).

   On exit:	 *a   = lower limit of three-point pattern
                 *b   = interior point of three-point pattern
                 *c   = upper limit of three-point pattern
                 *err = an error code.  If *err = 0, the search
                        was successful.  Otherwise, a descent
                        direction was NOT found using the minimum 
                        step length.
    
   Returned:     k = number of scalar function evalutions */

   vector	u;   
   float	F0,F1,F2,e,
   		hmin,hmax;
   int		eval = 2;		/* function evaluations */
   		
/* Initialize */
      
   if      (h <= 0) {
      *err = 1;
      return 0; }
   else if (mu < 0)
      nerror (100,"bracket",4,mu,"");
   else if (n < 1)
      nerror (100,"bracket",5,(float) n,"");
   else if (r < 0)
      nerror (100,"bracket",6,(float) r,"");
   else if (s < 0)
      nerror (100,"bracket",7,(float) s,"");

   e = epsilon();
   u = vec (n,"");   
   hmin = e*MAX(e,vecnorm(x,n));
   hmax = 1/(e*e);
   #if defined(DISPLAY_OPTIM)
      printf ("\nbracket range: %g to %g",hmin,hmax);
   #endif
   *a = 0;
   *b = h;
   *c = 2*h;
   F0 = fmu (x,mu,n,r,s,f,p,q);
   step (x,*b,d,n,u);
   F1 = fmu (u,mu,n,r,s,f,p,q);
   
/* Find b by halving candidate interval */

   while ((F1 >= F0) && (*b > hmin)) {
      *c = *b;
      *b /= 2; 	
      #if defined(DISPLAY_OPTIM)
         F2 = F1;
      #endif
      step (x,*b,d,n,u);
      F1 = fmu(u,mu,n,r,s,f,p,q);
      eval++;
      #if defined(DISPLAY_OPTIM)
         printf ("\n f(%g) = %g, f(%g) = %g, f(%g) = %g",
                 *a,F0,*b,F1,*c,F2);  
      #endif
 }

/* Check for problems finding b */

   if (F1 >= F0) {
      *err = 1;
      #if defined(DISPLAY_OPTIM)	
         printf ("\nUnable to find a descent in bracket using a step");
         printf ("\nof length %g. Initial step length = %g. ",*b,h);
         printf ("\nf(%g) = %g, f(%g) = %g, f(%g) = %g",
                 *a,F0,*b,F1,*c,F2);  
         wait();
      #endif   
      freevec(u);
      return eval; }
      
/* Find c by doubling candidate interval */

   if (eval <= 2) {
      step (x,*c,d,n,u);
      F2 = fmu (u,mu,n,r,s,f,p,q);
      eval++;
      #if defined(DISPLAY_OPTIM)
         printf ("\n  f(%g) = %g, f(%g) = %g, f(%g) = %g",
                 *a,F0,*b,F1,*c,F2);  
      #endif
      while ((F2 < F1) && (*c < hmax)) {
         *a = *b;
         *b = *c;
         *c *= 2;
         #if defined(DISPLAY_OPTIM)
           F0 = F1;
         #endif
         F1 = F2;
         step (x,*c,d,n,u);
         F2 = fmu (u,mu,n,r,s,f,p,q);
         eval++;  
         #if defined(DISPLAY_OPTIM)
            printf ("\n  f(%g) = %g, f(%g) = %g, f(%g) = %g",
                    *a,F0,*b,F1,*c,F2);  
         #endif
      } }
      
/* Check for problems finding c */

   #if defined(DISPLAY_OPTIM)
      if (F2 < F1) {
         printf ("\nUnable to reach minimum in bracket using a step");
         printf ("\nof length %g. Initial step length = %g. ",*c,h);
         printf ("\nf(%g) = %g, f(%g) = %g, f(%g) = %g",
                 *a,F0,*b,F1,*c,F2);  
         wait(); }
   #endif         

/* Finalize */

   #if defined(DISPLAY_OPTIM)
      if (getchar() == 'x') exit(1);
    #endif
   *err = 0;
   freevec (u);        
   return eval; }

/* -------------------------------------------------------------------- */
int  golden (vector x,vector d,float a,float c,float eps,
             float mu,int n,int r,int s,float *h,
             float (*f)(vector x,int n),
             void (*p)(vector x,int n,int r,vector y),
             void (*q)(vector x,int n,int s,vector y)) {
/* -------------------------------------------------------------------- */

/* Description:  Use the golden section search to solve the following
                 one-dimensional optimization problem:

                    minimize:   F(y) = f(x+y*d) + mu*(P(x) + Q(x))
                    subject to: a < y < c

                 It is assumed that F(y) has a minimum in the interval
                 [a,c]. The interval [a,c] is obtained by a call to
                 function bracket.  Here P(x) is a penalty function
                 associated with the r equality constraints, p(x) = 0,
                 and Q(x) is a penalty function associated with the s
                 inequality constraints, q(x) >= 0.

   On entry:     x   = n by 1 start vector
                 d   = n by 1 search direction vector
                 a   = lower limit on step length
                 c   = upper limit on step length
                 eps = upper bound on size of interval of uncertainty
                       (eps > 0)   
                 mu  = penalty paramter (mu >= 0)
                 n   = dimension of x
                 r   = number of equality constratains (r >= 0)
                 s   = number of inequality constraints (s >= 0)
                 f   = name of objective function: minimize: f(x)
                 p   = name of equality constraint function: p(x) = 0
                 q   = name of inequality constraint function: q(x) >= 0

                     The prototypes of f, p, and q are:

                        float f (vector x,int n);
                        void p (vector x,int n,int r,vector y);
                        void q (vector x,int n,int r,vector y);

                     When f is called with n by 1 vector x, it must
                     return f(x).  When p is called with n by 1 vector
                     x, it must compute r by 1 vector y = p(x). When
                     q is called with n by 1 vector x, it must
                     compute s by 1 vector y = q(x).  If r = 0, set
                     p = NULL and if s = 0, set q = NULL. 

   On exit:	*h   = optimal step length

   Returned:    k = number of scalar function evalutions 

   Notes:       If F(y) is a unimodal function, then the returned value
                is within eps*|z| of the exact solution z.  It is
                recommended that eps be no smaller than the square
                root of the machine epsilon. */

   vector	u;   
   float	b,b1,b2,e,F1,F2,g1,g2;
   int		eval = 2;
		
/* Initialize */

   if      (eps <= 0) 
      nerror (100,"golden",5,eps,"");
   else if (mu < 0)
      nerror (100,"golden",6,mu,"");
   else if (n < 1)
      nerror (100,"golden",7,(float) n,"");
   else if (r < 0)
      nerror (100,"golden",8,(float) r,"");
   else if (s < 0)
      nerror (100,"golden",9,(float) s,"");
   e = MAX(eps,(float) sqrt(epsilon()));
   u = vec (n,"");
       
/* Reduce the length of the interval of uncertainty */

   g1 = (float) (3 - sqrt(5))/2;
   g2 = 1 - g1;
   b1 = a + g1*(c - a);
   b2 = a + g2*(c - a);
   step (x,b1,d,n,u);
   F1 = fmu (u,mu,n,r,s,f,p,q);
   step (x,b2,d,n,u);
   F2 = fmu (u,mu,n,r,s,f,p,q);
   b = (a + c)/2;
   #if defined(DISPLAY_OPTIM)
      printf ("\ngolden section search in [%g, %g]",a,c); 
   #endif
   
   while ((c - a) > 2*e*b) {
      if (F1 < F2) {
       	 c = b2;
       	 b2 = b1; 
      	 F2 = F1;
         b1 = a + g1*(c - a);
         step (x,b1,d,n,u);
         F1 = fmu (u,mu,n,r,s,f,p,q); }
      else {
         a = b1;
         b1 = b2; 
         F1 = F2;
         b2 = a + g2*(c - a);
         step (x,b2,d,n,u);
         F2 = fmu (u,mu,n,r,s,f,p,q); }

      #if defined(DISPLAY_OPTIM)
         printf ("\n [a b1 b2 c] = [%g %g %g %g]",a,b1,b2,c);
      #endif
      b = (a + c)/2;   
      eval++; }
 
/* Finalize */

   #if defined(DISPLAY_OPTIM)
      if (getchar() == 'x') exit(1);
   #endif
   freevec (u);
   *h = b;
   return eval; }
   
/* -------------------------------------------------------------------- */
int      bisectd (vector x,vector d,float a,float c,float eps,
                  float mu,int n,int r,int s,float *h,
                  float (*f)(vector x,int n),
                  void (*g)(vector x,int n,vector df),
                  void (*p)(vector x,int n,int r,vector y),
                  void (*q)(vector x,int n,int s,vector y)) {
/* -------------------------------------------------------------------- */

/* Description:  Use the bisection with derivative method to solve the
                 following one-dimensional optimization problem:

                    minimize:   F(y) = f(x+y*d) + mu*(P(x) + Q(x))
                    subject to: a < y < c

                 It is assumed that F(y) has a minimum in the interval
                 [a,c]. The interval [a,c] is obtained by a call to
                 function bracket.  Here P(x) is a penalty function
                 associated with the r equality constraints, p(x) = 0,
                 and Q(x) is a penalty function associated with the s
                 inequality constraints, q(x) >= 0.

   On entry:     x   = n by 1 start vector
                 d   = n by 1 search direction vector
                 a   = lower limit on step length
                 c   = upper limit on step length
                 eps = upper bound on size of interval of uncertainty
                       (eps > 0)   
                 mu  = penalty paramter (mu >= 0)
                 n   = dimension of x
                 r   = number of equality constratains (r >= 0)
                 s   = number of inequality constraints (s >= 0)
                 f   = name of objective function: f(x)
                 g   = name of objective gradient function: df(x)/dx 
                 p   = name of equality constraint function: p(x) = 0
                 q   = name of inequality constraint function: q(x) >= 0

                     The prototypes of f, p, and q are:

                        float f (vector x,int n);
                        float g (vector x,int n,vector y);
                        void p (vector x,int n,int r,vector y);
                        void q (vector x,int n,int r,vector y);

                     When f is called with n by 1 vector x, it must
                     return f(x).  When g is called it must take the
                     n by 1 vector x and compute the n by 1 gradient
                     vector y[k] = df(x)/dx[k]. When p is called
                     with n by 1 vector x, it must compute r by 1
                     vector y = p(x). When q is called with n by 1
                     vector x, it must compute s by 1 vector y = q(x).
                     The argument g is optional in the sense that
                     bisectd can be called with g replaced by the
                     symbolic constant NULL in <stdio.h>.  When this
                     is done, the gradient is estimated numerically
                     using central differences in which case a
                     user-supplied function is NOT required. If r = 0,
                     set p = NULL and if s = 0, set q = NULL. 

   On exit:	*h   = optimal step length

   Returned:    k = number of scalar function evalutions 

   Notes:       If F(y) is a unimodal function, then the returned value
                is within eps*|z| of the exact solution z.  It is
                recommended that eps be no smaller than the square
                root of the machine epsilon. */

   vector 	u,df;    
   float	b,e,slope;
   int		eval = 0;
		
/* Initialize */

   if      (eps <= 0) 
      nerror (100,"bisectd",5,eps,"");
   else if (mu < 0)
      nerror (100,"bisectd",6,mu,"");
   else if (n < 1)
      nerror (100,"bisectd",7,(float) n,"");
   else if (r < 0)
      nerror (100,"bisectd",8,(float) r,"");
   else if (s < 0)
      nerror (100,"bisectd",9,(float) s,"");
   e = MAX (eps,(float) sqrt(epsilon()));
   u = vec (n,"");
   df = vec (n,"");   
   b = (a + c)/2;
   #if defined(DISPLAY_OPTIM)
      printf ("\nbisection search in [%g, %g]",a,c); 
   #endif
       
/* Reduce the length of the interval of uncertainty */

   while ((c - a) > 2*e*b) {

      if (g == NULL) {
         step (x,b,d,n,u);
         eval += gradfmu (u,mu,n,r,s,df,f,p,q); }
      else {
      	 step (x,b,d,n,u);
         (*g)(u,n,df);
         eval += n; }
     slope = inner (df,d,n);
     if (slope > 0)
         c = b;
      else
         a = b;
      b = (a + c)/2; 
      #if defined(DISPLAY_OPTIM)
         printf ("\n [a b c] = [%g %g %g]",a,b,c);
      #endif
   }
 
/* Finalize */

   #if defined(DISPLAY_OPTIM)
      if (getchar() == 'x') exit(1);
   #endif
   freevec (u);
   freevec (df);
   *h = b;
   return eval; }

static void q_limit (vector x,int n,int s,vector y) {

/* Description: Compute the inequality constraint function y = q(x)
                for the following type of inequality constratint:

                a_limit[k] <= x[k] <= b_limit[k] , 1 <= k <= n

                Here a_limit and b_limit are global variables. 
                

   On entry:    x = n by 1 vector of independent variables
   		n = number of independent variables (n >= 1)
		s = number of inequality constraints (s = 2*n)

   On exit:     y = s by 1 vector containing inequality constraint
                    function a evaluated at x.

   Notes:       This function is used by anneal. */
                
   int		i;

   s = 0;  				/* kill warning */
   for (i = 1; i <= n; i++) {
      y[i]   = x[i] - a_limit[i] + s;
      y[n+i] = b_limit[i] - x[i]; } }                 




