/* -------------------------------------------------------------------- */
/* File:     pde.c                                                      

   Contents: Partial differential equation functions

   Uses:     nlib.c */
/* -------------------------------------------------------------------- */
                 

/* -------------------------------------------------------------------- */
int poisson (float a,float b,int m,int n,int q,float eps,float *alpha,
             vector x,vector y,matrix U,float (*f)(float x,float y),
             float (*g)(float x,float y)) {
/* -------------------------------------------------------------------- */

/* Description: Use the successive over relaxation (SOR) method and
                three-point central differences to solve the following
                partial differential equation called the Poisson
                equation:

                   (d/dx)(d/dx)(x,y) + (d/dy)(d/dy)u(x,y) = f(x,y)

                in the rectangular region:

                   R = {(x,y): 0 < x < a, 0 < y < b}

                using the Dirichlet type boundary conditions:

                    u(x,y) = g(x,y)

   On entry:    a   = upper limit of x (a > 0)
                b   = upper limit of y (b > 0)
                m   = number of steps in x (m >= 1)
                n   = number of steps in y (n >= 1)
                q   = upper bound on the number of SOR iterations
                      (q >= 1)
                eps = upper bound on the magnitude of the difference
                      between successive solution estimates (eps >= 0)
                f   = name of user-supplied right-hand side function
                g   = name of user-supplied boundary condition
                      function. The prototypes of f and g are:

                         float f (float x,float y);
                         float g (float x,float y);

                      When f is called, it must return the value f(x,y).
                      When g is called with (x,y) on the boundary of R,
                      it must return the boundary value g(x,y). The
                      arguments f and g are optional in the sense that
                      either one can be replaced by the symbolic
                      constant NULL defined in <stdio.h>.  When this is
                      done, the corresponding function is assumed to be
                      zero which means that a user-supplied function is
                      NOT required in this case.

   On exit:     alpha = optimal relaxation parameter
                x     = m by 1 vector of x grid points
                y     = n by 1 vector of y grid points 
                U     = m by n matrix containing the estimated solution
                        where U[k][j] = u(x[k],y[j]) with:

                        x[k] = k*a/(m+1)  ,  1 <= k <= m
                        y[j] = j*b/(n+1)  ,  1 <= j <= n

   Returned:    r = number of SOR iterations. If it is less than the
                    user specified maximum, q, then the following
                    termination criterion was satisfied where u(i)
                    denotes the ith solution estimatae and ||u|| is
                    the infinity norm:

                       ||u(i+1) - u(i)|| < eps  

   Notes:      When f(x,y) = 0, Poisson's equation reduces to Laplace's
               equation which is also called the potential equation. */

   float	u0 = 0,
		beta,rho,dx,dy,dx2,dy2,d,v,du;
   int		i = 0,
		k,j;
   
/* Initialize */
   
   if      (a <= 0)
      nerror (100,"poisson",1,a,"");
   else if (b <= 0)
      nerror (100,"poisson",2,b,"");
   else if (m < 1)
      nerror (100,"poisson",3,(float) m,"");
   else if (n < 1)
      nerror (100,"poisson",4,(float) n,"");
   else if (q < 1)
      nerror (100,"poisson",5,(float) q,"");
   else if (eps < 0)
      nerror (100,"poisson",6,eps,"");
      
/* Compute optimal relaxation parameter */

   dx = a/(m + 1);
   dy = b/(n + 1);
   dx2 = dx*dx;
   dy2 = dy*dy;
   d = 2*(dx2 + dy2);
   rho = (float) ((cos(PI/m) + (dx2/dy2)*cos(PI/n))/(1 + dx2/dy2));
   *alpha = (float) (2/(1 + sqrt(1 - rho*rho)));
   beta = *alpha;
   
/* Compute initial guess using average of boundary values */

   for (k = 1; k <= m; k++)
      x[k] = k*dx;
   for (j = 1; j <= n; j++)
      y[j] = j*dy;
   if (g != NULL) {  
      u0 = (*g)(0,0) + (*g)(0,b) + (*g)(a,0) + (*g)(a,b);   
      for (k = 1; k <= m; k++)
         u0 += (*g)(x[k],0) + (*g)(x[k],b);   
      for (j = 1; j <= n; j++)
         u0 += (*g)(0,y[j]) + (*g)(a,y[j]); 
      u0 /= (2.0f*(n + m) + 4.0f); }

   for (k = 1; k <= m; k++)
     for (j = 1; j <= n; j++)
         U[k][j] = u0;

/* Iterate using SOR method */

   do {
      du = 0;
      i++;
   
/* Lower left corner: (k,j) = (1,1) */      

      v = beta*((dy2*U[2][1] + dx2*U[1][2])/d - U[1][1]);
      if (g != NULL)
         v += beta*(dy2*(*g)(0,y[1]) + dx2*(*g)(x[1],0))/d;
      if (f != NULL)
         v -= beta*dx2*dy2*(*f)(x[1],y[1])/d;
      U[1][1] += v;
      du = MAX (du,(float) fabs(v));

/* Interior left edge: k = 1, 1 < j < n */      

      for (j = 2; j < n; j++) {
         v = beta*((dy2*U[2][j] + dx2*(U[1][j+1]
             + U[1][j-1]))/d - U[1][j]);
         if (g != NULL)
            v += beta*dy2*(*g)(0,y[j])/d;    
         if (f != NULL) 
            v -= beta*dx2*dy2*(*f)(x[1],y[j])/d;
            U[1][j] += v;
         du = MAX (du,(float) fabs(v)); }

/* Upper left corner: (k,j) = (1,n) */      

      v = beta*((dy2*U[2][n] + dx2*U[1][n-1])/d - U[1][n]);
      if (g != NULL)
         v += beta*(dy2*(*g)(0,y[n]) + dx2*(*g)(x[1],b))/d;     
      if (f != NULL)
         v -= beta*dx2*dy2*(*f)(x[1],y[n])/d;
      U[1][n] += v;
      du = MAX (du,(float) fabs(v));

   
      for (k = 2; k < m; k++) {

/* Interior bottom: 1 < k < m, j = 1 */

         v = beta*((dy2*(U[k+1][1] + U[k-1][1]) + dx2*U[k][2])/d -
             U[k][1]);
         if (g != NULL)
            v += beta*dx2*(*g)(x[k],0)/d;    
         if (f != NULL)    
            v -= beta*dx2*dy2*(*f)(x[k],y[1])/d;
         U[k][1] += v;
         du = MAX (du,(float) fabs(v));
 
/* Interior: 1 < k < m, 1 < j < n */

         for (j = 2; j < n; j++) {
            v = beta*((dy2*(U[k+1][j] + U[k-1][j]) + dx2*(U[k][j+1]
                + U[k][j-1]))/d - U[k][j]);
            if (f != NULL)
               v -= beta*dx2*dy2*(*f)(x[k],y[j])/d;
            U[k][j] += v;
            du = MAX (du,(float) fabs(v)); }
 
/* Interior top: 1 < k < m, j = n */

         v = beta*((dy2*(U[k+1][n] + U[k-1][n]) + dx2*U[k][n-1])/d -
             U[k][n]);
         if (g != NULL)
            v += beta*dx2*(*g)(x[k],b)/d; 
         if (f != NULL)    
            v -= beta*dx2*dy2*(*f)(x[k],y[n])/d;     
         U[k][n] += v;
         du = MAX (du,(float) fabs(v)); }

/* Lower right corner: (k,j) = (m,1) */
      
      v = beta*((dy2*U[m-1][1] + dx2*U[m][2])/d - U[m][1]);
      if (g != NULL)
         v += beta*(dy2*(*g)(a,y[1]) + dx2*(*g)(x[m],0))/d;     
      if (f != NULL)    
         v -= beta*dx2*dy2*(*f)(x[m],y[1])/d;     
      U[m][1] += v;
      du = MAX (du,(float) fabs(v));

/* Interior right edge: k = m, 1 < j < n */      
     
      for (j = 2; j < n; j++) {
         v = beta*((dy2*U[m-1][j] + dx2*(U[m][j+1] + U[m][j-1]))/d -
             U[m][j]);
         if (g != NULL)
            v += beta*dy2*(*g)(a,y[j])/d;     
         if (f != NULL)    
            v -= beta*dx2*dy2*(*f)(x[m],y[j])/d;     
         U[m][j] += v;
         du = MAX (du,(float) fabs(v)); }
    
/* Upper right corner: (k,j) = (m,n) */      
     
      v = beta*((dy2*U[m-1][n] + dx2*U[m][n-1])/d - U[m][n]);
      if (g != NULL)
         v += beta*(dy2*(*g)(a,y[n]) + dx2*(*g)(x[m],b))/d;    
      if (f != NULL)    
         v -= beta*dx2*dy2*(*f)(x[m],y[n])/d; 
      U[m][n] += v;
      du = MAX (du,(float) fabs(v)); }

   while ((du >= eps) && (i < q));
      
/* Finalize */

   return i; }

/* -------------------------------------------------------------------- */
void heat1   (float T,float a,int m,int n,float beta,float c1,float c2,
              vector t,vector x,matrix U,
              float (*f)(float x),float (*g)(float x,int k)) {
/* -------------------------------------------------------------------- */
                 	
/* Description: Use the Crank-Nicolson method to solve the following
                partial differential equation called the one-dimensional
                heat equation (also the diffusion equation):

                   (d/dt)u(t,x) = beta*(d/dx)*(d/dx)u(t,x)

                The heat equation is to be solved over the region:

                   0 <= t <= T,
                   0 < x < a

                subject to the following initial and boundary
                conditions:

                   u(0,x) = f(x)
                   (1 - c1)*u(t,0) - c1*(d/dt)u(t,0) = g(t,1)
                   (1 - c2)*u(t,a) + c2*(d/dt)u(t,a) = g(t,2)
  
                where c1 and c2 are in the range [0,1].

   On entry:    T    = final solution time (T > 0)
                a    = upper boundary of x (a > 0)
                m    = number steps in t (m >= 1)
                n    = number of steps in x (n >= 1)
                beta = heat equation coefficient (beta > 0)
                c1   = boundary condition parameter (0 <= c1 <= 1)
                c2   = boundary condition parameter (0 <= c1 <= 1)
                       The boundary conditon type is selected
                       as follows:

                          0 => Dirichlet
                          1 => Neumann

                       Otherwise it is a mixed boundary condition.
                f    = name of user-supplied initial condtion function
                g    = name of user-supplied boundary condtion function
                       The prototypes of f and g are:
 
                          float f (float x);
                          float g (float t,int k);

                       When f is called, it must return the initial
                       value u(0,x).  When g is called, it must return
                       the boundary value g(t,k). The functions f and
                       g are optional in the sense that either can be
                       replaced by the symbolic constant NULL in
                       <stdio.h>. When this is done, the corresponding
                       function is assumed to be zero which means that
                       a user-supplied function is NOT required in this
                       case.

   On exit:     t = (m+1) by 1 vector of t grid points
                x = n by 1 vector of x grid points
                U = (m+1) by n matrix containing the estimated solution
                    with U[k][j] = u[t[k],x[j]) where:

                    t[k] = (k-1)*T/m  ,  1 <= k <= m+1               
                    x[j] = j*a/(n+1)  ,  1 <= j <= n

   Notes:       1.  The function heat1 uses the functions trifac and
                    trisub to solve a tridiagonal linear algebraic
                    system at each time step.

                2.  The Crank-Nicolson method is a stable method that is
                    second-order accurate in both time and space. */

   matrix	A,Q;
   vector	b;
   double	delta; 
   float	dt,                     /* time step */ 			
                dx,                     /* space step */
	        gamma,                  /* gain */
	        b1,b2,g0;
   int		i,j,k;
   
/* Initialize */

   if      (T <= 0)
      nerror (100,"heat1",1,T,"");
   else if (a <= 0)
      nerror (100,"heat1",2,a,"");
   else if (m < 1)
      nerror (100,"heat1",3,(float) m,"");
   else if (n < 1)
      nerror (100,"heat1",4,(float) n,"");
   else if (beta <= 0)
      nerror (100,"heat1",5,beta,"");
   else if ((c1 < 0) || (c1 > 1))
      nerror (100,"heat1",6,c1,"");
   else if ((c2 < 0) || (c2 > 1))
      nerror (100,"heat1",7,c2,"");
      
   A = mat (3,n,"");
   Q = mat (3,n,"");
   b = vec (n,"");
   dt = T/m;
   dx = a/(n+1);
   for (i = 1; i <= m+1 ; i++)
      t[i] = (i-1)*dt;
   for (j = 1; j <= n; j++)
      x[j] = j*dx;   
   gamma = beta*dt/(dx*dx);   
   if (f != NULL)
      for (j = 1; j <= n; j++)
         U[1][j] = (*f)(j*dx);
   else
      for (j = 1; j <= n; j++)
         U[1][j] = 0;
         
/* Compute coefficient matrix of u(k+1) */

   b1 = c1 + (1 - c1)*dx;
   b2 = c2 + (1 - c2)*dx;
   g0 = 2*(1 + gamma);
   for (k = 1; k <= n; k++) {
      A[1][k] = -gamma;
      A[2][k] =  g0;
      A[3][k] = -gamma; }
   A[2][1] -= gamma*c1/b1;
   A[2][n] -= gamma*c2/b2;
   delta = trifac(A,n,Q);
   if (fabs(delta) < epsilon()/100) 
      nerror (312,"heat1",0,0,""); 
   
/* Solve the PDE */

   g0 = 2*(1 - gamma);
   for (k = 1; k <= m; k++) {
      b[1] = (g0 + (c1/b1)*gamma)*U[k][1] + gamma*U[k][2];
      for (j = 2; j < n; j++)
         b[j] = gamma*U[k][j-1] + g0*U[k][j] + gamma*U[k][j+1];
      b[n] = gamma*U[k][n-1] + (g0 + (c2/b2)*gamma)*U[k][n];
      if (g != NULL) {
         b[1] += (gamma*dx/b1)*((*g)(k*dt,1) + (*g)((k+1)*dt,1));
         b[n] += (gamma*dx/b2)*((*g)(k*dt,2) + (*g)((k+1)*dt,2)); }
      trisub (Q,b,n,U[k+1]); }
   
/* Finalize */

   freemat (A,3);
   freemat (Q,3);
   freevec (b); }

/* -------------------------------------------------------------------- */
void	heat2   (float T,float a,float b,int m,int n,int p,
                 float beta,vector x,vector y,matrix U,
                 float (*f)(float x,float y),
                 float (*g)(float t,float x,float y)) {
/* -------------------------------------------------------------------- */
                 	
/* Description: Use the alternating direction implicit (ADI) method to
                solve the following partial differential equation called
                the two-dimensional heat equation (also the diffusion
                equation):

                   (d/dt)u(t,x,y) = beta*[(d/dx)(d/dx)u(t,x,y)
                                    + (d/dy)(d/dy)u(t,x,y)]

                The heat equation is to be solved over the region:

                   0 < t <= T,
                   0 < x < a,
                   0 < y < b
         
                subject to the initial and boundary conditions:

                   u(0,x,y) = f(x,y),
                   u(t,0,y) = g(t,x,y)
         
   On entry:    T    = final solution time (T >= 0)
                a    = upper boundary of x (a > 0)
                b    = upper boundary of y (b > 0)
                m    = number of steps in t (m >= 1)
                n    = number of steps in x (n >= 1)
                p    = number of steps in y (p >= 1)
                beta = heat equation coefficient (beta > 0)
                f    = name of user-supplied initial condition function
                g    = name of user-supplied boundary condition
                       function. The prototypes of f and g are:

                          float f (float x,float y);
                          float g (float t,float x,float y);

                       When f is called it must return the value
                       u(0,x,y). When g is called with (x,y) on the
                       boundary, it must return the boundary value
                       u(t,x,y). The functions f and g are optional in
                       the sense that either can be replaced by the
                       symbolic constant NULL in <stdio.h>. When this
                       is done, the corresponding function is assumed
                       to be zero which means that a user-supplied
                       function is NOT required in this case.

   On exit:      x = n by 1 vector of x grid points
                 y = p by 1 vector of y grid points
                 U = n by p matrix containing final solution estimates
                     U[i][j] = u(T,x[i],y[j]) where:

                     x[i] = i*a/(n+1)  ,  1 <= i <= n
                     y[j] = j*b/(p+1)  ,  1 <= j <= p

   Notes:        1. The function heat2 uses the functions trifac and
                    trisub to solve n+p tridiagonal linear algebraic
                    systems at each time step.

                 2. The ADI method is a stable method. */
      
   matrix	A,Q1,Q2,W;
   vector	c,v;
   double	delta;
   float	dx,dy,dt,               /* x,y,t steps */
		gx,gy,                  /* x,y gains */
                g0,t0,tm,t1;
   int		i,j,k,q;
   
/* Initialize */

   if      (T < 0)
      nerror (100,"heat2",1,T,"");
   else if (a <= 0)
      nerror (100,"heat2",2,a,"");
   else if (b <= 0)
      nerror (100,"heat2",3,b,"");
   else if (m < 1)
      nerror (100,"heat2",4,(float) m,"");
   else if (n < 1)
      nerror (100,"heat2",5,(float) n,"");
   else if (p < 1)
      nerror (100,"heat2",6,(float) p,"");
   else if (beta <= 0)
      nerror (100,"heat2",7,beta,"");

   q = MAX (n,p);
   A = mat (3,q,"");
   Q1 = mat (3,p,"");
   Q2 = mat (3,n,"");
   W = mat (n,p,"");
   c = vec (q,"");
   v = vec (q,"");

   dt = T/m;
   dx = a/(n+1);
   dy = b/(p+1);
   gx = beta*(dt/2)/(dx*dx);   
   gy = beta*(dt/2)/(dy*dy);
   for (i = 1; i <= n; i++)
      x[i] = i*dx;
   for (j = 1; j <= p; j++)
      y[j] = j*dy;   
   
/* Compute coefficient matrices */

      g0 = 1 + 2*gy;
      for (i = 1; i <= p; i++) {
         A[1][i] = -gy;
         A[2][i] =  g0;
         A[3][i] = -gy; }
      delta = trifac (A,p,Q1);
      if (fabs(delta) < epsilon()/100) 
         nerror (312,"heat2",0,0,""); 
      g0 = 1 + 2*gx;
      for (i = 1; i <= n; i++) {
         A[1][i] = -gx;
         A[2][i] =  g0;
         A[3][i] = -gx; }
      delta = trifac (A,n,Q2);
      if (fabs(delta) < epsilon()/100) 
         nerror (312,"heat2",0,0,""); 

/* Initialize solution */

   if (f != NULL) 
      for (i = 1; i <= n; i++)
         for (j = 1; j <= p; j++) 
            U[i][j] = (*f)(x[i],y[j]); 
   else
      zeromat (U,n,p); 

/* Solve heat equation */ 
         
   for (k = 0; k < m; k++) {

      t0 = k*dt;
      t1 = (k+1)*dt;
      tm = (t0 + t1)/2;
   
/* First pass: i = 1 */

      g0 = 1 - 2*gx;
      for (j = 1; j <= p; j++) {
         c[j] = g0*U[1][j] + gx*U[2][j];
         if (g != NULL) 
            c[j] += gx*(*g)(t0,0,y[j]); }
      if (g != NULL) {      
         c[1] += gy*(*g)(tm,x[1],0);
         c[p] += gy*(*g)(tm,x[1],b); }
      trisub (Q1,c,p,W[1]);
   
/* First pass: 1 < i < n */

      for (i = 2; i < n; i++) { 
         for (j = 1; j <= p; j++)
            c[j] = gx*U[i-1][j] + g0*U[i][j] + gx*U[i+1][j];
         if (g != NULL) {   
            c[1] += gy*(*g)(tm,x[i],0);
            c[p] += gy*(*g)(tm,x[i],b); }
         trisub (Q1,c,p,W[i]); }
      
/* First pass: i = n */

      for (j = 1; j <= p; j++) {
         c[j] = gx*U[n-1][j] + g0*U[n][j];
         if (g != NULL)
            c[j] += gx*(*g)(t0,a,y[j]); }
      if (g != NULL) {      
         c[1] += gy*(*g)(tm,x[n],0);
         c[p] += gy*(*g)(tm,x[n],b); }
      trisub (Q1,c,p,W[n]);
   
/* Second pass: j = 1 */

      g0 = 1 - 2*gy;
      for (i = 1; i <= n; i++) {
         c[i] = g0*W[i][1] + gx*W[i][2];
         if (g != NULL)
            c[i] += gy*(*g)(t0,x[i],0); }
      if (g != NULL) { 
         c[1] += gx*(*g)(t1,0,y[1]);
         c[n] += gx*(*g)(t1,a,y[1]); }
      trisub (Q2,c,n,v);
      for (i = 1; i <= n; i++)
         U[i][1] = v[i];

/* Second pass: 1 < j < p */

      for (j = 2; j < p; j++) { 
         for (i = 1; i <= n; i++)
            c[i] = gy*W[i][j-1] + g0*W[i][j] + gy*W[i][j+1];
         if (g != NULL) {
            c[1] += gx*(*g)(t1,0,y[j]);
            c[n] += gx*(*g)(t1,a,y[j]); }
         trisub (Q2,c,n,v);
         for (i = 1; i <= n; i++)
            U[i][j] = v[i]; }
      
/* Second pass: j = p */

      for (i = 1; i <= n; i++) {
         c[i] = gy*W[i][p-1] + g0*W[i][p];
         if (g != NULL) 
            c[i] += gx*(*g)(tm,x[i],b); }
      if (g != NULL) { 	
         c[1] += gx*(*g)(t1,0,y[p]);
         c[n] += gx*(*g)(t1,a,y[p]); }
      trisub (Q2,c,n,v);
      for (i = 1; i <= n; i++)
            U[i][p] = v[i]; }
   
/* Finalize */

   freemat (A,3);
   freemat (Q1,3);
   freemat (Q2,3);
   freemat (W,n);
   freevec (c);
   freevec (v); }
   
/* -------------------------------------------------------------------- */
void wave1   (float T,float a,int m,int n,float beta,vector t,vector x,
              matrix U,float (*f)(float x),float (*g)(float x)) {
/* -------------------------------------------------------------------- */

/* Description: Use the explicit central difference method to solve the
                following partial differential equation called the
                one-dimensional wave equation:

                   (d/dt)(d/dt)u(t,x) = beta*(d/dx)(d/dx)u(t,x)

                The wave equation is to be solved over the region:

                   0 <= t <= T,
                   0 < x < a

                The boundary conditions are:

                   u(t,0) = 0,
                   u(t,a) = 0,

                The initial conditions are:

                   u(0,x)       = f(x)
                   (d/dt)u(0,x) = g(x)

   On entry:    T    = final solution time (T > 0)
                a    = upper boundary of x (a > 0)
                m    = number steps of t (m >= 1)
                n    = number of steps of x (n >= 1)
                beta = wave equation coefficient (beta > 0)
                f    = name of user-supplied initial function
                g    = name of user-supplied initial function
                       The prototypes of f and g are:

                          float f (float x);
                          float g (float x);

                       When f is called it must return the initial
                       value u(0,x). When g is called it must return
                       (d/dt)u(t,x) evaluated at t = 0.  The functions
                       f and g are optional in the sense that either
                       argument can be replaced by the symbolic
                       constant NULL in <stdio.h>. When this is done,
                       the corresponding function is assumed to be zero
                       which means a user-supplied function is NOT
                       required in this case.

   On exit:     t = (m+1) by 1 vector of t grid points
                x = n by 1 vector of x grid points
                U = (m+1) by n matrix containing the estimated solution
                    with U[k][j] = u(t[k],x[j]) where:

                       t[k] = (k-1)*T/m  ,  1 <= k <= m+1               
                       x[j] = j*a/(n+1)  ,  1 <= j <= n

   Notes:       The central difference method is stable for gamma <= 1
                where the gain gamma is defined:

                   gamma = beta*pow(T*(n+1)/(a*m),2)

                It is the responsibility of the user to pick T, a, n,
                and m to ensure that gamma <= 1.  */

   float	dt,                     /* time step */ 			
		dx,                     /* space step */
		gamma,                  /* gain */
		g0,tmax;
   int		i,j;
   
/* Initialize */
   
   if      (T <= 0)
      nerror (100,"wave1",1,T,"");
   else if (a <= 0)
      nerror (100,"wave1",2,a,"");
   else if (m < 1)
      nerror (100,"wave1",3,(float) m,"");
   else if (n < 1)
      nerror (100,"wave1",4,(float) n,"");
   else if (beta <= 0)
      nerror (100,"wave1",5,beta,"");

/* Check for stability */

   dt = T/m;
   dx = a/(n+1);
   for (i = 1; i <= m+1 ; i++)
      t[i] = (i-1)*dt;
   for (j = 1; j <= n; j++)
      x[j] = j*dx;   
   gamma = (float) (beta*pow(dt/dx,2));
   tmax = (float) (m*dx/sqrt(beta));
   if (T > tmax) {
      printf
      ("\n\n--------------------------------------------------------");
      printf
      ("\nThe solution time T in the call to function wave1 is too");
      printf ("\nlarge for a stable solution.  It must satisfy:");
      printf ("\n\n    pow(T/(m+1),2) <= pow(a/(n+1),2)/beta");
      printf
      ("\n\nThe maximum stable value is T = %.3f. You must decrease",
       tmax);
      printf
      ("\nT or modify m, a or n as needed to ensure stability.");
      printf
      ("\n--------------------------------------------------------\n");
      exit(1); }
   
/* Initialize U[1] and U[2] using initial condition functions */

   g0 = 2*(1 - gamma);
   if (f != NULL) {
      for (i = 1; i <= n; i++)
         U[1][i] = (*f)(x[i]);
      U[2][1]  = (g0*(*f)(x[1]) + gamma*(*f)(x[2]))/2;
      for (i = 2; i < n; i++) {
         U[2][i] = (gamma*(*f)(x[i-1]) + g0*(*f)(x[i]) +
                    gamma*(*f)(x[i+1]))/2; }
      U[2][n]  = (gamma*(*f)(x[n-1]) + g0*(*f)(x[n]))/2; }
   else {
      zerovec (U[1],n);
      zerovec (U[2],n); }
   if (g != NULL) 
      for (i = 1; i <= n; i++) 
         U[2][i] += dt*(*g)(i*dx); 
      
/* Compute remaining rows of U */
             
   for (i = 2; i <= m; i++) {
      U[i+1][1]  = g0*U[i][1] + gamma*U[i][2] - U[i-1][1];
      for (j = 2; j < n; j++) {
       U[i+1][j] = gamma*U[i][j-1] + g0*U[i][j] + gamma*U[i][j+1]
                   - U[i-1][j];
       U[i+1][n]  = gamma*U[i][n-1] + g0*U[i][n] - U[i-1][n]; } } }

/* -------------------------------------------------------------------- */
void	wave2   (float T,float a,float b,int m,int n,int p, 
                 float beta,vector x,vector y,matrix U,
                 float (*f)(float x,float y),
                 float (*g)(float x,float y)) {
/* -------------------------------------------------------------------- */
                 	
/* Description: Use the explicit central difference method to solve the
                following partial differential equation called the two-
                dimensional wave equation:

                   (d/dt)(d/dt)u(t,x,y) = beta*[(d/dx)(d/dx)u(t,x,y) +
                                          (d/dy)(d/dy)u(t,x,y)]

                The wave equation is to be solved over the region:

                   0 < t <= T,
                   0 < x < a,
                   0 < y < b
         
                subject to the initial and boundary conditions:

                   u(0,x,y)       = f(x,y),
                   (d/dt)u(0,x,y) = g(x,y),
                   u(t,x,y)       = 0

   On entry:    T    = final solution time (T >= 0)
                a    = upper boundary of x (a > 0)
                b    = upper boundary of y (b > 0)
                m    = number of steps in t (m >= 2)
                n    = number of steps in x (n >= 1)
                p    = number of steps in y (p >= 1)
                beta = wave equation coefficient (beta > 0)
                f    = name of user-supplied initial condition function
                g    = name of user-supplied initial condition function
                       The prototypes of f and g are:

                          float f (float x,float y);
                          float g (float x,float y);

                       When f is called it must return the value
                       u(0,x,y). When g is called it must return
                       (d/dt)u(t,x,y) evaluated at t = 0$. The
                       functions f and g are optional in the sense that
                       either argument can be replaced with the symbolic
                       constant NULL in <stdio.h>. When this is done,
                       the corresponding function is assumed to be zero
                       which means that a user-supplied function is NOT
                       required in this case.

   On exit:     x = n by 1 vector of x grid points
                y = p by 1 vector of y grid points
                U = n by p matrix containing final solution estimate 
                    with U[i][j] = u(T,x[i],y[j]) where:

                        x[i] = i*a/(n+1)  ,  1 <= i <= n
                        y[j] = j*b/(p+1)  ,  1 <= j <= p

   Notes:       The central difference method is stable when T
                satisfies the following criterion:

                   pow(T/m,2) <= (pow(a/(n+1),2) + pow(b/(p+1),2))/
                                 (4*beta)

                It is the responsibility of the user to pick T to
                ensure that the method is stable. */

   matrix	U0,U1;
   float	dt,dx,dy,               /* t,x,y steps */
                gx,gy,gz,               /* gains */
                Dt;    			/* maximum stable step */
   int		i,j,k;

/* Initialize */
   
   if      (T < 0)
      nerror (100,"wave2",1,T,"");
   else if (a <= 0)
      nerror (100,"wave2",2,a,"");
   else if (b <= 0)
      nerror (100,"wave2",3,b,"");
   else if (m < 2)
      nerror (100,"wave2",4,(float) m,"");
   else if (n < 1)
      nerror (100,"wave2",5,(float) n,"");
   else if (p < 1)
      nerror (100,"wave2",6,(float) p,"");
   else if (beta <= 0)
      nerror (100,"wave2",7,beta,"");
      
/* Check for stability */

   dt = T/m;
   dx = a/(n+1);
   dy = b/(p+1);
   Dt = (float) sqrt((dx*dx + dy*dy)/(4*beta));
   if (dt > Dt) {
    printf
    ("\n\n------------------------------------------------------------");
    printf
    ("\nThe solution time T in the call to function wave2 is too");
    printf ("\nlarge for a stable solution.  It must satisfy:");
    printf
    ("\n\n  pow(T/m,2) <= (pow(a/(n+1),2) + pow(b/(p+1),2))/(4*beta)");
    printf
    ("\n\nThe maximum stable value is T = %.3f. You must decrease T",
     m*Dt);
    printf
    ("\nor modify m, a, b, n, or p as needed to ensure stability.");
    printf
    ("\n------------------------------------------------------------\n");
    exit(1); }

/* Compute x, y, U0 */

    U0 = mat (n,p,"");
    U1 = mat (n,p,"");
    for (i = 1; i <= n; i++) { 
      x[i] = i*dx;	
      for (j = 1; j <= p; j++) {
      	 y[j] = j*dy;
      	 if (f != NULL)
            U0[i][j] = (*f)(x[i],y[j]);
         else
            U0[i][j] = 0; } }

/* Compute: U1[1] */

   gx = (float) (beta*pow(dt/dx,2));   
   gy = (float) (beta*pow(dt/dy,2));
   gz = 2*(1 - gx - gy);
   if (f != NULL) {
      U1[1][1] = (gx*(*f)(x[2],y[1]) + gz*(*f)(x[1],y[1])
                 + gy*(*f)(x[1],y[2]))/2;
      for (j = 2; j < p; j++)
         U1[1][j] = (gx*(*f)(x[2],y[j]) + gz*(*f)(x[1],y[j])
                    + gy*((*f)(x[1],y[j-1]) + (*f)(x[1],y[j+1])))/2;
      U1[1][p] = (gx*(*f)(x[2],y[p]) + gz*(*f)(x[1],y[p])
                 + gy*(*f)(x[1],y[p-1]))/2; }
   else
      zerovec (U1[1],p);
   if (g != NULL) 
      for (j = 1; j <= p; j++)
         U1[1][j] += dt*(*g)(x[1],y[j]); 	

/* Compute U1[i], 1 < i < n */

   for (i = 2; i < n; i++) {
      if (f != NULL) {
         U1[i][1] = (gx*((*f)(x[i-1],y[1]) + (*f)(x[i+1],y[1]))
                    + gz*(*f)(x[i],y[1]) + gy*(*f)(x[i],y[2]))/2; 
         for (j = 2; j < p; j++)
            U1[i][j] = (gx*((*f)(x[i-1],y[j]) + (*f)(x[i+1],y[j]))
                       + gz*(*f)(x[i],y[j]) + gy*((*f)(x[i],y[j-1])
                       + (*f)(x[i],y[j+1])))/2;
         U1[i][p] = (gx*((*f)(x[i-1],y[p]) + (*f)(x[i+1],y[p]))
                    + gz*(*f)(x[i],y[p]) + gy*(*f)(x[i],y[p-1]))/2; }
      else
         zerovec (U1[i],p);
      if (g != NULL)
         for (j = 1; j <= p; j++)
            U1[i][j] += dt*(*g)(x[i],y[j]); }
   
/* Compute: U1[n] */

   if (f != NULL) {
      U1[n][1] = (gx*(*f)(x[n-1],y[1]) + gz*(*f)(x[n],y[1])
                 + gy*(*f)(x[n],y[2]))/2;
      for (j = 2; j < p; j++)
         U1[n][j] = (gx*(*f)(x[n-1],y[j]) + gz*(*f)(x[n],y[j])
                    + gy*((*f)(x[n],y[j-1]) + (*f)(x[n],y[j+1])))/2;
      U1[n][p] = (gx*(*f)(x[n-1],y[p]) + gz*(*f)(x[n],y[p])
                 + gy*(*f)(x[n],y[p-1]))/2; }
   else
      zerovec (U1[n],p);
   if (g != NULL)
      for (j = 1; j <= p; j++)
         U1[n][j] += dt*(*g)(x[n],y[j]); 


   for (k = 1; k < m; k++) {
   
/* Compute: U[1] */

      U[1][1] = gx*U1[2][1] + gz*U1[1][1] + gy*U1[1][2]
                - U0[1][1];
      for (j = 2; j < p; j++)
         U[1][j] = gx*U1[2][j] + gz*U1[1][j]
                   + gy*(U1[1][j-1] + U1[1][j+1])
                   - U0[1][j];
      U[1][p] = gx*U1[2][p] + gz*U1[1][p] + gy*U1[1][p-1]
                - U0[1][p];

/* Compute U[i], 1 < i < n */

      for (i = 2; i < n; i++) {
         U[i][1] = gx*(U1[i-1][1] + U1[i+1][1]) + gz*U1[i][1]
                   + gy*U1[i][2] - U0[i][1];
         for (j = 2; j < p; j++)
            U[i][j] = gx*(U1[i-1][j] + U1[i+1][j])+ gz*U1[i][j]
                      + gy*(U1[i][j-1] + U1[i][j+1]) - U0[i][j];
         U[i][p] = gx*(U1[i-1][p] + U1[i+1][p]) + gz*U1[i][p]
                   + gy*U1[i][p-1] - U0[i][p]; }

/* Compute: U[n] */

      U[n][1] = gx*U1[n-1][1] + gz*U1[n][1] + gy*U1[n][2]
                - U0[n][1];
      for (j = 2; j < p; j++)
         U[n][j] = gx*U1[n-1][j] + gz*U1[n][j]
                   + gy*(U1[n][j-1] + U1[n][j+1])
                   - U0[n][j];
      U[n][p] = gx*U1[n-1][p] + gz*U1[n][p] + gy*U1[n][p-1]
                - U0[n][p];

      copymat (U1,n,p,U0);
      copymat (U,n,p,U1); }

/* Finalize */

   freemat (U0,n);
   freemat (U1,n);  }
   
/* ------------------------------------------------------------------- */
double trifac (matrix T,int n,matrix Q) {
/* ------------------------------------------------------------------- */

/* Descripton: Factor an n by n tridiagonal coefficient matrix A into
               lower and upper-triangular factors.

   On entry:   T = 3 by n matrix containing the nonzero entries of A:

                   row                  contents
                   ---------------------------------------------------
                    1     A[1][2], A[2][3], ... , A[n-1][n],   ---    
                    1     A[1][1], A[2][2], ... , A[n-1][n-1], A[n][n]    
                    3     A[2][1], A[3][2], ... , A[n][n-1],   ---    
                   ---------------------------------------------------

               n = number of unknowns (n >= 3)

   On exit:    Q = 3 by n matrix containing lower and upper-triangular
               factors of A.  The matrix Q is used by the function
               trisub which solves Ax = b by forward and backward
               substitution.

   Returned:   The determinant of A.  If it is nonzero, then A has been
               successfully factored. */
               
   int 		k;            
   float	eps = epsilon()/100;
   double	delta = 1;

/* Initilialize */

   if (n < 3)
      nerror (100,"trifac",2,(float) n,"");   

/* Factor T into L and U */
     
   zeromat (Q,3,n); 
   copyvec (T[3],n-1,Q[3]);
   Q[2][1] = T[2][1];
   for (k = 1; k < n; k++) {
      delta *= Q[2][k];	
      if (fabs(Q[2][k]) < eps) {
         freemat (Q,3);
         return 0; }
      else {
         Q[1][k] = T[1][k]/Q[2][k];
         Q[2][k+1] = T[2][k+1] - T[3][k]*Q[1][k]; } }
   
   delta *= Q[2][n];
   return delta; }

/* ------------------------------------------------------------------- */
double trisub (matrix Q,vector b,int n,vector x) {
/* ------------------------------------------------------------------- */

/* Descripton: Solve the factored tridiagonal linear algebraic system
               Ax = b using forward and back substitution.

   On entry:   Q = 3 by n matrix containing lower and upper-triangular
                   factors of A.  Q is obtained by a previous call to
                   function trifac. 
               b = n by 1 right-hand side vector
               n = number of unknowns (n >= 3)

   On exit:    x = n by 1 solution vector

   Returned:   The determinant of A.  If it is nonzero, then x has
               been successfully computed. */

   int 		k;            
   float	eps = epsilon()/100;
   double	delta = 1;
   vector	y;

/* Initilialize */

   if (n < 3)
      nerror (100,"trisub",3,(float) n,"");   
   for (k = 1; k <= n; k++) {
      delta *= Q[2][k];
      if (fabs(delta)< eps)
         return 0; }
      
/* Perform forward substitution to find y */
 
   y = vec (n,"");
   y[1] = b[1]/Q[2][1];
   for (k = 2; k <= n; k++) 
      y[k] = (b[k] - Q[3][k-1]*y[k-1])/Q[2][k];
    
/* Perform back substitution to find x */

   x[n] = y[n];
   for (k = n-1; k >= 1; k--) 
      x[k] = y[k] - Q[1][k]*x[k+1];

/* Finalize */

   freevec (y);
   return delta; }


