/* -------------------------------------------------------------------- */
/* File:     roots.c 

   Contents: Roots of nonlinear equations

   Uses:     nlib.c, linear.c */     
/* -------------------------------------------------------------------- */


/* -------------------------------------------------------------------- */
int 	bisect (float x0,float x1,float eps,int m,float *x, 
                float (*f)(float x)) {
/* -------------------------------------------------------------------- */

/* Description: Apply the bisection method to find a root of:

                   f(x) = 0

   On entry:    x0  = lower limit of search
                x1  = upper limit of search (x1 > x0)
                eps = error tolerance used to terminate search (eps >= 0)
                m   = maximum number of iterations (m >= 1)
                f   = name of user-supplied function whose root is to
                      found.  The prototype of f is:

                         float f (float x);

                      When f is called, it must return the value f(x).

   On exit:     *x = estimated root of f(x).

   Returned:    the number of iterations performed.  If it is less than
                the user-specified maximum, m, then the following
                converence criterion was satisfied:

                   |f(*x)| < eps

   Notes:       1. x0 and x1 must be selected such that f(x0)*f(x1) < 0 
                   to ensure that there is a root in [x0,x1]

                2. bisect should always converge for a continuous
                   f if eps and m are sufficiently large. */ 
                     
   int		k = 1;
   float	x2,f0,f2,y;

/* Initialize */
   
   if (x1 <= x0)
      nerror (100,"bisect",2,x1,"");
   else if (eps < 0)
      nerror (100,"bisect",3,eps,"");
   else if (m < 1)
      nerror (100,"bisect",4,(float) m,"");
   f0 = (*f)(x0); 
   f2 = (*f)(x1);
   if (f0*f2 >= 0)
      nerror (305,"bisect",0,0,"");

/* Find root */

   do {
      x2 = (x0 + x1)/2;
      f2 = (*f)(x2);
      if (f0*f2 < 0)
         x1 = x2;
      else {
         x0 = x2;
         f0 = f2; }
      y = (float) fabs(f2);
      #if defined(DISPLAY_ROOTS)
         printf ("\n%2i & %10.7f & %10.7f & %10.7f \\\\",k,x2,x1-x0,y);
      #endif	   
      k++; }
   while ((y > eps) && (k <= m));

/* Finalize */

   *x = x2;   
   return k-1; }
   
/* -------------------------------------------------------------------- */
int 	secant (float x0,float x1,float eps,int m,float *x, 
                float (*f)(float x)) {
/* -------------------------------------------------------------------- */

/* Description: Apply the secant method to find a root of:

                   f(x) = 0

   On entry:    x0  = first initial guess
                x1  = second initial guess (x1 <> x0)  
                eps = error tolerance used to terminate search (eps >= 0)
                m   = maximum number of iterations (m >= 1)
                f   = name of user-supplied function whose root is to
                      found.  The prototype of f is:

                         float f (float x);

                      When f is called, it must return the value f(x).

   On exit:     *x = estimated root of f(x).

   Returned:    the number of iterations performed.  If it is less than
                the user-specified maximum, m, then the following
                converence criterion was satisfied:

                   |f(*x)| < eps */

   int		k = 1; 
   float	em = epsilon(),
		x2,f0,f1,y;

/* Initialize */

   if      (fabs(x1-x0) < em)
      nerror (310,"secant",0,0,"");
   else if (eps < 0)
      nerror (100,"secant",3,eps,"");
   else if (m < 1)
      nerror (100,"secant",4,(float) m,"");
   f0 = (*f)(x0);
   f1 = (*f)(x1);
   #if defined(DISPLAY_ROOTS)
      printf ("\n 0 & %10.7f & %10.7f \\\\",x0,fabs(f0)); 
      printf ("\n 1 & %10.7f & %10.7f \\\\",x1,fabs(f1)); 
      printf ("\n \\hline");
   #endif

/* Find root */

   do {
      x2 = x1 - (x1-x0)*f1/(f1-f0);
      x0 = x1;
      f0 = f1;
      x1 = x2;
      f1 = (*f)(x2);
      y = (float) fabs(f1);
      #if defined(DISPLAY_ROOTS)
         printf ("\n%2i & %10.7f & %10.7f \\\\",k+1,x1,y); 
      #endif
      k++; }
   while ((y > eps) && (k <= m));

/* Finalize */

   *x = x2;   
   return k-1; }

/* -------------------------------------------------------------------- */
int 	muller    (complex x0,complex x1,complex x2,float eps,
                   int m,complex *x,complex (*f)(complex x)) {
/* -------------------------------------------------------------------- */

/* Description: Apply Muller's method to find a root of:

                   f(x) = 0

   On entry:    x0  = first complex initial guess
                x1  = second complex initial guess (x1 <> x0)
                x2  = third complex initial guess (x2 <> x1, x2 <> x0) 
                eps = error tolerance used to terminate search (eps >= 0)
                m   = maximum number of iterations (m >= 1)
                f   = name of user-supplied function whose root is to
                      found.  The prototype of f is:

                         complex f (complex x);

                      When f is called it must return f(x) 

   On exit:     *x = estimated root of f(x).

   Returned:    the number of iterations performed.  If it is less than
                the user-specified maximum, m, then the following
                converence criterion was satisfied:

                   |f(*x)| < eps */

   int		k = 1; 
   float	em = epsilon(),
		y;	
   complex	x3,f0,f1,f2,h0,h1,d,c0,c1,c2,ca,cb,c;

/* Initialize */
   
   if      (cmag(csub(x1,x0)) < em)
      nerror (310,"muller",0,0,"");
   else if ((cmag(csub(x2,x0)) < em) || (cmag(csub(x2,x1)) < em))
      nerror (310,"muller",0,0,"");
   else if (eps < 0)      
      nerror (100,"muller",4,eps,"");
   else if (m < 1)
      nerror (100,"muller",5,(float) m,"");
   f0 = (*f)(x0);
   f1 = (*f)(x1);
   f2 = (*f)(x2);
   #if defined(DISPLAY_ROOTS)
      printf ("\n 0 & %10.7f %+10.7fj & %10.7f \\\\",x0.x,x0.y,cmag(f0));
      printf ("\n 1 & %10.7f %+10.7fj & %10.7f \\\\",x1.x,x1.y,cmag(f1));
      printf ("\n 2 & %10.7f %+10.7fj & %10.7f \\\\",x2.x,x2.y,cmag(f2));
      printf ("\n \\hline");
   #endif

/* Find root */
   
   do {
      h0 = csub (x0,x2);
      h1 = csub (x1,x2);
      d = cmul(cmul(h0,h1),csub(h1,h0));
      c0 = f2;

      ca = cmul(cmul(h1,h1),csub(f0,f2));
      cb = cmul(cmul(h0,h0),csub(f1,f2));
      c1 = cdiv(csub(ca,cb),d);

      ca = cmul(h0,csub(f1,f2));
      cb = cmul(h1,csub(f0,f2));
      c2 = cdiv(csub(ca,cb),d);

      c.x = 4*c0.x;
      c.y = 4*c0.y;
      c = csqrt(csub(cmul(c1,c1),cmul(c,c2)));
      ca = cadd(c1,c);
      cb = csub(c1,c);
      if (cmag(cb) > cmag(ca))
         ca = cb;
      c.x = 2*c0.x;
      c.y = 2*c0.y;
      cb = cdiv(c,ca);
      x3 = csub(x2,cb);   
      
      x0 = x1;
      f0 = f1;
      x1 = x2;
      f1 = f2;
      x2 = x3;
      f2 = (*f)(x3);
      y = cmag(f2);
      #if defined(DISPLAY_ROOTS)
        printf ("\n%2i & %10.7f %+10.7fj & %10.7f \\\\",k+2,x2.x,x2.y,y);
      #endif
      k++; }
   while ((y > eps) && (k <= m));

/* Finalize */

   *x = x2;   
   return k-1; }

/* -------------------------------------------------------------------- */
int 	laguerre  (complex x0,float eps,int m,vector a,int n,complex *x) {
/* -------------------------------------------------------------------- */

/* Description: Apply Lauerre's method to find a root an nth degree
                polynomial:

                   f(x) = a[1] + a[2]*x + ... + a[n+1]*pow(x,n)

   On entry:    x0  = complex initial guess
                eps = error tolerance used to terminate search (eps >= 0)
                m   = maximum number of iterations (m >= 1)
                a   = (n+1) by 1 vector of polynomial coefficients
                n   = degree of polynomial (n >= 1)

   On exit:     *x = estimated root of f(x).

   Returned:    the number of iterations performed.  If it is less than
                the user-specified maximum, m, then the following
                converence criterion was satisfied:

                   |f(*x)| < eps */

   int		k = 0,
		i; 
   float	y;	
   complex	f0,f1,f2,g,d1,d2,z,x1;
   vector	a1,a2;
   cvector      w;
   
/* Initialize */

   if      (eps < 0)
      nerror (100,"laguerre",2,eps,"");
   else if (m < 1)
      nerror (100,"laguerre",3,(float) m,"");
   else if (n < 1)
      nerror (100,"laguerre",5,(float) n,"");
   switch (n) {

/* Linear polynomial */     

      case 1:			
         x->x = -a[1]/a[2];
         x->y = 0;
         return 0;

/* Quadratic polynomial */     

      case 2:
         w = cvec (2,"");
         quadratic (a,w);
         *x = w[1];
         freecvec (w);
         return 0; }
        
/* Compute coefficients of derivative polynomials */

   a1 = vec (n,"");
   a2 = vec (n-1,"");
   for (i = 1; i < n; i++) {
      a1[i] = i*a[i+1];
      a2[i] = i*(i+1)*a[i+2]; }
   a1[n] = n*a[n+1];

/* Find a root */
     
   f0 = cpoly (x0,a,n);

   #if defined(DISPLAY_ROOTS)
      printf ("\n 0 & %10.7f %+10.7fj & %10.7f \\\\",x0.x,x0.y,cmag(f0));
      printf ("\n \\hline"); 
   #endif

   do {
      f1 = cpoly (x0,a1,n-1);
      f2 = cpoly (x0,a2,n-2);
      g = cmul(f0,f2);
      g.x = n*g.x/(n-1);
      g.y = n*g.y/(n-1);
      z = cmul(f1,f1);
      g = csub(z,g);
      g = csqrt(g);      
      g.x = (n-1)*g.x;
      g.y = (n-1)*g.y;
      d1 = cadd(f1,g);
      d2 = csub(f1,g);
      if ((d1.x*d1.x + d1.y*d1.y) >= (d2.x*d2.x + d2.y*d2.y)) 
         x1 = cdiv(f0,d1);
      else
         x1 = cdiv(f0,d2);
      x1.x = n*x1.x;
      x1.y = n*x1.y; 
      x1 = csub(x0,x1);
      
      x0 = x1;
      f0 = cpoly(x0,a,n);
      y = cmag(f0);
      k++;
      #if defined(DISPLAY_ROOTS)
         printf ("\n%2i & %10.7f %+10.7fj & %10.7f \\\\",k,x0.x,x0.y,y);
      #endif
   }
   while ((y >= eps) && (k < m));

/* Finalize */

   freevec (a1);
   freevec (a2);
   *x = x0;
   return k; }

/* -------------------------------------------------------------------- */
int 	polyroot (vector a,float eps,int n,int m,cvector x) {
/* -------------------------------------------------------------------- */

/* Description: Apply Lauerre's method with deflation to find the n
                roots of the following nth degree polynomial:

                   f(x) = a[1] + a[2]*x + ... + a[n+1]*pow(x,n)

   On entry:    a = (n+1) by 1 vector of polynomial coefficients
                eps = error tolerance used to terminate search (eps >= 0)
                n   = degree of polynomial (n >= 1)
                m   = maximum number of iterations per root (m >= 1)

   On exit:     x = n by 1 complex vector containing the n roots.

   Returned:    the number of iterations performed.  If it is less than
                the user-specified maximum, m, then the following
                converence criterion was satisfied:

                   |f(x[k])| < eps , 1 <= k <= n

   Notes:       The roots in x are sorted into decreasing magnitudes. */

   int      k = 1,
            t = 0,
	    j,h;
   float    em = epsilon(),
	    d,r,s,dx;
   complex  z;
   vector   b,c;
   #if defined(DISPLAY_ROOTS)
      int i;
      float y;
   #endif


/* Initialize */

   if      (eps < 0)
      nerror (100,"polyroot",2,eps,"");
   else if (n < 1)
      nerror (100,"polyroot",3,(float) n,"");
   else if (m < 1)
      nerror (100,"polyroot",4,(float) m,"");
   else if (fabs(a[n+1]) < em)
      nerror (304,"polyroot",n,0,"");  
   dx = (float) sqrt(em);
   b = vec (n+1,"");
   c = vec (n+1,"");
   copyvec (a,n+1,b);
   
/* Find roots */

   do {

/* Estimate root */
   
      j = n - k + 1;
      x[k].x = randu (-dx,dx);
      x[k].y = randu (-dx,dx);
      h = laguerre (x[k],eps,m,b,j,&z);
      x[k] = z;
      t = MAX(t,h);
      
/* Polish it */
            
       
      if ((k > 1) && (n > 2)) {
         h = laguerre (x[k],eps,m,a,n,&z);
         x[k] = z;
         t = MAX(t,h); }
      if (fabs(x[k].y) < 100*em) {

/* Deflate by linear factor */

         if (j > 1) {
            syndiv (b,x[k].x,j,c);
            copyvec (c,j,b);
            #if defined(DISPLAY_ROOTS) 
               printf ("\nLinear deflation by %g",x[k].x);
               wait();
            #endif
            }
            k += 1; } 
          
      else {

/* Deflate by quadratic factor */
      
         x[k+1].x =  x[k].x;
         x[k+1].y = -x[k].y;
         if (j > 2) {
            d = x[k].x*x[k].x + x[k].y*x[k].y;
            syndiv2 (b,2*x[k].x,-d,j,c,&r,&s);
            copyvec (c,j-1,b);
            #if defined(DISPLAY_ROOTS) 
               printf ("\nQuadratic deflation by %g +/- %gj",x[k].x,
                        x[k].y);
               wait();
            #endif
            }
         k += 2; } }

   while (k <= n);

   #if defined(DISPLAY_ROOTS)
      printf ("\n");
      for (i = 1; i <= n; i++) {
         z = cpoly (x[i],a,n);
         y = sqrt(z.x*z.x + z.y*z.y);
         printf ("\n%2i & %10.7f %+10.7fj & %10.7f \\\\",i,
                  x[i].x,x[i].y,y); }
   #endif               

/* Finalize */
               
   csort (x,n);
   freevec (c);
   freevec (b);
   return t; }

/* -------------------------------------------------------------------- */
int	newton   (vector x,float eps,int n,int m,
                  void (*f)(vector x,int n,vector y),
                  void (*g)(vector x,int n,matrix J)) {
/* -------------------------------------------------------------------- */

/* Description: Apply Newton's method to find a root of the n dimensional
                nonlinear algebraic system: 

                   f(x) = 0

   On entry:    x   = n by 1 vector containing initial guess
                eps = error tolerance used to terminate search (eps >= 0)
                n   = number of unknowns (n >= 1)
                m   = maximum number of iterations (m >= 1)
                f   = name of user-supplied function whose root is to
                      found.  The prototype of f is:

                         void f (vector x,int n,vector y);

                      When f is called, it takes the n by 1 vector x
                      and computes the n by 1 vector y = f(x).

                g   = name of optional user-supplied function which
                      computes the n by n Jacobian matrix J(x).  The
                      prototype of g is:
                      
                         void g (vector x,int n,matrix J);

                      When g is called it takes the n by 1 vector x
                      and computes the n by n Jacobian matrix J =
                      df(x)/dx.  That is J[k][j] = df[k]/dx[j].  The
                      function g is optional in the sense that newton
                      can be called with the argument g set to the
                      symbolic constant NULL defined in the header file
                      stdio.h.  When this is done, newton approximates
                      the Jacobain matrix numerically using central
                      differences in which case the user does not have
                      to supply a function.

   On exit:     x   = n by 1 vector containing estimated root of f(x). 

   Returned:    the number of iterations performed.  If it is less than
                the user-specified maximum, m, then the following
                converence criterion was satisfied:

                   ||f(x)|| < eps */

   int		k = 1,
		i;
   float	r;
   vector	y,dx;
   matrix 	J;
         
/* Initialize */

   if      (eps < 0)
      nerror (100,"newton",2,eps,"");
   else if (n < 1)
      nerror (100,"newton",3,(float) n,"");
   else if (m < 1)
      nerror (100,"newton",4,(float) m,"");
   y = vec (n,"");
   dx = vec (n,"");
   J = mat (n,n,"");
   (*f) (x,n,y);
   #if defined(DISPLAY_ROOTS)
      printf ("\n%2i & ",0);
      for (i = 1; i <= n; i++)
         printf ("%10.7f & ",x[i]);
      printf ("%10.7f \\\\",vecnorm(y,n)); 
      printf ("\n \\hline");
   #endif
   
/* Find root */

   do {

      if (g == NULL)
         Jacobian (x,n,J,f);
      else   
         (*g) (x,n,J);
      ludec (J,y,n,dx);
      for (i = 1; i <= n; i++)
         x[i] -= dx[i];
      (*f) (x,n,y);
      r = vecnorm (y,n);
      #if defined(DISPLAY_ROOTS)      
         printf ("\n%2i & ",k);
         for (i = 1; i <= n; i++)
            printf ("%10.7f & ",x[i]);
         printf ("%10.7f \\\\",r);
      #endif         
      k++; }
      
  while ((r > eps) && (k <= m));

/* Finalize */

   freemat (J,n);
   freevec (dx);
   freevec (y);
   return k-1; }

/* -------------------------------------------------------------------- */
/* Local Functions */ 
/* -------------------------------------------------------------------- */

/* -------------------------------------------------------------------- */
float syndiv (vector a,float c,int n,vector b) {
/* -------------------------------------------------------------------- */

/* Description: Divide an nth degree polynomial f(x) by a linear factor
                (x - c) to produce a quotient polynomial g(x) and a
                remainder d.

                   f(x) = (x-c)g(x) + d

    On entry:   a = (n+1) by 1 vector containing coefficeints of the
                    polynomial:

                    f(x) = a[1] + a[2]*x + ... + a[n+1]*pow(x,n)
                    
                c = root of divisor polynomial
                n = degree of polynoial f(x) (n \ge 2)

    On exit:    b = n by 1 vector containing coefficients of quotient
                    polynomial:

                    g(x) = b[1] + b[2]*x + ... + b[n]*pow(x,n-1)
 
    Returned:   The remainder d = f(c). */

   int		k;
   if (n < 2)
      nerror (100,"syndiv",3,(float) n,"");
   b[n] = a[n+1];
   for (k = 1; k < n; k++)
      b[n-k] = a[n-k+1] + c*b[n-k+1];
   return (a[1] + c*b[1]); }
 
/* -------------------------------------------------------------------- */
void syndiv2 (vector a,float p,float q,int n,vector b,float *r,
              float *s) {
/* -------------------------------------------------------------------- */

/* Description: Divide an nth degree polynomial f(x) by a quadratic factor
                (x*x - p*x - q) to produce a quotient polynomial g(x) and 
                a remainder rx + s.

                   f(x) = (x*x - px - q)g(x) + rx + s

    On entry:   a = (n+1) by 1 vector containing coefficeints of the
                    polynomial:

                    f(x) = a[1] + a[2]*x + ... + a[n+1]*pow(x,n)
                    
                p = linear coefficient of divisor polynomial
                q = constant coefficient of divisor polynomial
                n = degree of polynomial f(x) (n >= 3)
                
    On exit:    b = (n-1) by 1 vector containing coefficients of
                    quotient polynomial:

                    g(x) = b[1] + b[2]*x + ... + b[n-1]*pow(x,n-2)

                *r = coefficient of linear term of remainder
                *q = coefficeint of constant term of remainder */

   int		k;
   if (n < 3)
      nerror (100,"syndiv2",4,(float) n,"");
   b[n-1] = a[n+1];
   b[n-2] = a[n] + p*b[n-1];
   for (k = 3; k < n; k++)
      b[n-k] = a[n-k+2] + p*b[n-k+1] + q*b[n-k+2];
   *r = a[2] + p*b[1] + q*b[2];
   *s = a[1] + q*b[1]; }
 
/* -------------------------------------------------------------------- */
void Jacobian (vector x,int n,matrix J,
               void (*f)(vector x,int n,vector y)) {
/* -------------------------------------------------------------------- */

/* Description: Estimate the Jacobain matrix, J = df(x)/dx, numerically
                using central differences.

   On entry:    x = n by 1 vector at which J is to be evaluated
                n = dimension of x (n >= 1)
                f = name of user-defined function which specifies
                    f(x).  The prototype of f is:

                       void f (vector x,int n,vector y);
                       
                    When f is called, it must take the n by 1 vector x
                    and compute the n by 1 vector y = f(x).

   On exit:     J = n by n Jacobian matrix of f(x):

                       J[k][j] = df[k]/x[j] , 1 <= k,j <= n */
                       
   int		i,j;
   float	h = (float) sqrt(epsilon());
   vector	x0 = vec (n,""),
                x1 = vec (n,""), 
                x2 = vec (n,""); 
   copyvec (x,n,x0);
   for (j = 1; j <= n; j++) {
      x0[j] = x[j] + h;  	
      (*f)(x0,n,x1);
      x0[j] = x[j] - h;
      (*f)(x0,n,x2);
      x0[j] = x[j];
       for (i = 1; i <= n; i++)
         J[i][j] = (x1[i] - x2[i])/(2*h); }
   freevec (x2);
   freevec (x1);
   freevec (x0); }
   
/* -------------------------------------------------------------------- */
void quadratic (vector a, cvector x) {
/* -------------------------------------------------------------------- */

/* Description: Compute the roots of the quadratic polynomial:

                   f(x) = a[1] + a[2]*x + a[3]*pow(x,2)

   On entry:    a = 3 by 1 vector containing polynomial coefficients

   On exit:     x = 1 by 1 complex vector containing roots

   Notes:       If a[3] = 0, issues an error message. */

   float	em = epsilon(),
		d;

/* Check polynomial degree */
   
   if (fabs(a[3]) < em)
      nerror (304,"quadratic",2,0,"");
      
/* Find roots */

      d = a[2]*a[2] - 4*a[1]*a[3];
      if (d >= 0) {

/* Real roots */

         d = (float) sqrt(d);
         x[1].y = 0;
         x[2].y = 0; 
         x[1].x = (-a[2] - SGN(a[2])*d)/(2*a[3]);
         if (fabs(x[1].x) > em)
            x[2].x = a[1]/(a[3]*x[1].x);
         else
            x[2].x = (-a[2] + SGN(a[2])*d)/(2*a[3]); }
      else {

/* Complex conjugate roots */

         x[1].x = (float) (-a[2]/(2*a[3]));
         x[1].y = (float) sqrt(-d)/(2*a[3]);
         x[2].x = x[1].x;
         x[2].y = -x[1].y; } }
      
/* -------------------------------------------------------------------- */
int 	falsepos (float x0,float x1,float eps,int m,float *x, 
                  float (*f)(float x)) {
/* -------------------------------------------------------------------- */

/* Description: Apply the false position method to find a root of:

                   f(x) = 0

   On entry:    x0  = lower limit of search
                x1  = upper limit of search (x1 > x0)
                eps = error tolerance used to terminate search (eps >= 0)
                m   = maximum number of iterations (m >= 1)
                f   = name of user-supplied function whose root is to
                      found.  The prototype of f is:

                         float f (float x);

                      When f is called, it must return the value f(x).

   On exit:     *x = estimated root of f(x).

   Returned:    the number of iterations performed.  If it is less than
                the user-specified maximum, m, then the following
                converence criterion was satisfied:

                   |f(*x)| < eps

   Notes:       1. x0 and x1 must be selected such that f(x0)*f(x1) < 0 to
                   ensure that there is a root in [x0,x1]

                2. falsepos should always converge for a continuous
                   f if eps and m are sufficiently large. */ 

   int		k = 1; 
   float	x2,f0,f1,f2,y;

/* Initialize */
   
   if (x1 <= x0)
      nerror (100,"falsepos",2,x1,"");
   else if (eps < 0)
      nerror (100,"falsepos",3,eps,"");
   else if (m < 1)
      nerror (100,"falsepos",4,(float) m,"");
   f0 = (*f)(x0); 
   f1 = (*f)(x1);
   if (f0*f1 >= 0)
      nerror (305,"falsepos",0,0,"");

/* Find root */
     
   do {
      x2 = x1 - (x1-x0)*f1/(f1-f0);
      f2 = (*f)(x2);
      if (f0*f2 < 0) {
         x1 = x2;
         f1 = f2; }
      else {
         x0 = x2;
         f0 = f2; }
      y = (float) fabs(f2);
      #if defined(DISPLAY_ROOTS)
         printf ("\n%2i & %10.7f & %10.7f & %10.7f \\\\",k,x2,x1-x0,y);
      #endif
      k++; }
   while ((y > eps) && (k <= m));

/* Finalize */

   *x = x2;   
   return k-1; }

/* -------------------------------------------------------------------- */
int 	aitken (float x0,float b,float eps,int m,int ex,float *x, 
                float (*f)(float x)) {
/* -------------------------------------------------------------------- */

/* Description: Apply the contraction mapping method with Aitken
                extrapolation to find a root of:

                   f(x) = 0

   On entry:    x0  = initial guess 
                b   = gain which ensures that g(x) = x - bf(x) is
                      a contration.  The gain b must satisfy:

                         0 < bf'(x) < 2

                eps = error tolerance used to terminate search (eps >= 0)
                m   = maximum number of iterations (m >= 1)
                ex  = extrapolation switch:

                         0 = contraction mapping
                         1 = contraction mapping with Aitken
                             extrapolation

                f   = name of user-supplied function whose root is to
                      found.  The prototype of f is:

                         float f (float x);

                      When f is called, it must return the value f(x).

   On exit:     *x = estimated root of f(x).

   Returned:    the number of iterations performed.  If it is less than
                the user-specified maximum, m, then the following
                converence criterion was satisfied:

                   |f(*x)| < eps */

   int		k = 1; 
   float	
		x1,x2,x3,x4,f2,y;

/* Initialize */

   if      (eps < 0)
      nerror (100,"aitken",3,eps,"");
   else if (m < 1)
      nerror (100,"aitken",4,(float) m,"");
   else if ((ex < 0) || (ex > 1))
      nerror (100,"aitken",5,(float) ex,"");
   x1 = 0;
   x2 = x0;
   f2 = (*f)(x2);
   #if defined(DISPLAY_ROOTS) 
      printf ("\n 0 & %10.7f & %10.7f \\\\",x0,fabs((*f)(x0)));
      printf ("\n \\hline");
   #endif
      
/* Find root */

   do {
      x3 = x2 - b*f2;
      x0 = x1;
      x1 = x2;
      x2 = x3;
      f2 = (*f)(x3);
      if (ex && (k > 2)) {
       	 x4 = (x0*x2 - x1*x1)/(x2 - 2*x1 + x0);
         y = (float) fabs((*f)(x4));
         #if defined(DISPLAY_ROOTS)
            printf ("\n%2i & %10.7f & %10.7f \\\\",k,x4,y); 
         #endif
         }
      else {
         y = (float) fabs(f2);
         #if defined(DISPLAY_ROOTS)
            printf ("\n%2i & %10.7f & %10.7f \\\\",k,x2,y); 
         #endif
         }
      k++; }
   while ((y > eps) && (k <= m) );

/* Finalize */

   if (ex)
      *x = x4;
   else
      *x = x2;   
   return k-1; }

/* -------------------------------------------------------------------- */
int 	Newton1   (float x0,float eps,int m,float *x,
                   float (*f)(float x,int k)) {
/* -------------------------------------------------------------------- */

/* Description: Apply the Newton-Raphson method to find a root of:

                   f(x) = 0

   On entry:    x0  = initial guess
                eps = error tolerance used to terminate search (eps >= 0)
                m   = maximum number of iterations (m >= 1)
                f   = name of user-supplied function whose root is to
                      found.  The prototype of f is:

                         float f (float x,int k);

                      When f is called with k = 0 or k = 1, it must 
                      return the kth derivative of f(x).  That is, f
                      returns f(x) when k = 0 and f'(x) when k = 1.

   On exit:     *x = estimated root of f(x).

   Returned:    the number of iterations performed.  If it is less than
                the user-specified maximum, m, then the following
                converence criterion was satisfied:

                   |f(*x)| < eps */

   int		k = 1; 
   float	x1,f0,y,q;

/* Initialize */

   if      (eps < 0)
      nerror (100,"Newton1",2,eps,"");
   else if (m < 1)
      nerror (100,"Newton1",3,(float) m,"");
   f0 = (*f)(x0,0);
   #if defined(DISPLAY_ROOTS)
      printf ("\n 0 & %10.7f & %10.7f \\\\",x0,fabs(f0));
      printf ("\n \\hline");
   #endif
      
/* Find root */

   do {
      q = (*f)(x0,1);
      x1 = x0 - f0/q;
      x0 = x1;
      f0 = (*f)(x0,0);
      y = (float) fabs(f0);
      #if defined(DISPLAY_ROOTS) 
         printf ("\n%2i & %10.7f & %10.7f \\\\",k,x0,y);
      #endif
      k++; }
   while ((y > eps) && (k <= m));

/* Finalize */

   *x = x1;   
   return k-1; }




