/*------------------------------------------------------------------*/
/* Example 8.5.1: Predictor-Corrector Method                        */
/*------------------------------------------------------------------*/

#include "c:\nlib\util.h"

void f (float t,vector x,int n,vector dx)
{

/* Description: Two-dimensional system with solution
                x(t) = [t,1-t]*exp(-t) when x(0) = [0,1]. */

   dx[1] = x[2];
   dx[2] = -x[1] - 2*x[2];
}

/* ---------------------------------------------------------------------- */
float    Adams (matrix X,float t0,float t1,int n,
                void (*f)(float t,vector x,int n,vector dx)) 
/* ---------------------------------------------------------------------- */
{
/* Description: Perform one step of Adam's fourth-order predictor-corrector
                method to solve an n-dimensional system of ordinary
                differential equations:

                   dx(t)/dt = f[t,x(t)]

   On entry:    X  = 4 by n matrix X containing past solutions where
                     h = t1 - t0:
                    
                        ---------------------------
                        Row    On entry    On exit
                        ---------------------------
                         1     x(t0)       x(t0+h)
                         2     x(t0-h)     x(t0)
                         3     x(t0-2h)    x(t0-h)
                         4     x(t0-3h)    x(t0-2h)
                        ---------------------------

                t0 = initial time
                t1 = final time
                n  = number of equations (n >= 1)
                f  = name of user-supplied function which defines the
                     system of equations to be solved. The prototype of
                     f is:

                        void f (float t,vector x,int n,vector dx);

                     When f is called, it must evaluate the right-hand
                     side of the system of differential equations and
                     return the result in the n by 1 vector dx.

   On exit:     X  = 4 by n matrix X updated to include current and
                     past solutions (see above).

   Returned:         The infinity norm of the truncation error. */
 
   vector	dx,xp,xc;
   float	h = t1-t0,
		predictor[4] = {55,-59,37,-9},
		corrector[4] = {9,19,-5,1},
		t,E;	
   int		i,j;

/* Initialize */

   if (n < 1)
      nerror (100,"Adams",4,n,"");
   dx = vec (n,"");
   xp = vec (n,"");
   xc = vec (n,"");
   
/* Compute fourth-order Adams-Bashforth predictor: xp */


   copyvec (X[1],n,xp);
   for (i = 1; i <= 4; i++)
   {
      t = t0 - (i-1)*h; 	
      (*f) (t,X[i],n,dx);
      for (j = 1; j <= n; j++)
         xp[j] += h*predictor[i-1]*dx[j]/24;
   }
         
/* Compute fourth-order Adams-Moulton corrector: xc */

   copyvec (X[1],n,xc);
   for (i = 1; i <= 4; i++)
   {
      t = t0 - (i-1)*h;  
      if (i == 1)   	
         (*f) (t+h,xp,n,dx);
      else
         (*f) (t+h,X[i-1],n,dx);
      for (j = 1; j <= n; j++)
         xc[j] += h*corrector[i-1]*dx[j]/24;
   }
   
/* Compute local truncation error */

   for (j = 1; j <= n; j++) 
      dx[j] = 19*(xp[j] - xc[j])/270;
   E = vecnorm(dx,n);   

/* Update X */

   for (i = 4; i > 1; i--) 
      for (j = 1; j <= n; j++)
         X[i][j] = X[i-1][j];
   for (j = 1; j <= n; j++)
      X[1][j] = xc[j] + dx[j];
      
/* Finalize */

   freevec (xc);
   freevec (xp);
   freevec (dx);
   return E;
}

int main (void)
{
   int		n     	= 2,
		m     	= 11,
		k,j;
   float	alpha	= 0,	
   		beta  	= 2,
		h	= (beta-alpha)/(m-1);
   vector	x0    	= vec (n,"0,1"),
   		x1	= vec (n,""),
   		x	= vec (n,""),
   		e	= vec (n,""),
   		t	= vec (m,"");
   matrix	E     	= mat (m,5,""),
		X	= mat (n,m,""),
		X0	= mat (4,n,"");	

/* Exact solution */

   printf ("\nExample 8.5.1: Predictor-Corrector Method\n");
   for (k = 1; k <= m; k++)
   {
      t[k] = (k - 1)*h;
      X[1][k] = t[k]*exp(-t[k]);
      X[2][k] = (1-t[k])*exp(-t[k]);
   }
   graphxm (t,X,2,m,"Solution of Linear System","t","x","fig851");  
   addlabel (0.3,0.60,"x_2","fig851");
   addlabel (0.3,0.15,"x_1","fig851");

 
/* Euler solution */
   
   copyvec (x0,n,x);
   for (k = 1; k < m; k++)
   {
      f(t[k],x,n,x1);
      for (j = 1; j <= n; j++)
      {
         x[j] += h*x1[j];
         e[j] = x[j] - X[j][k+1];
      }
      E[k][1] = vecnorm (e,n);
   }

/* Runge-Kutta solution */
   
   copyvec (x0,n,x);
   for (k = 1; k < m; k++)
   {
      E[k][2] = rkfstep (x,t[k],t[k+1],n,x1,f);
      copyvec (x1,n,x);
      for (j = 1; j <= n; j++)
         e[j] = x[j] - X[j][k+1];
      E[k][3] = vecnorm (e,n);
   }

/* Adams predictor-corrector solution */
   
   for (k = 1; k <= 4; k++)
   {
      X0[k][1] = t[5-k]*exp(-t[5-k]);
      X0[k][2] = (1-t[5-k])*exp(-t[5-k]);
   }
   for (k = 4; k < m; k++)
   {
      E[k][4] = Adams (X0,t[k],t[k+1],n,f);
      for (j = 1; j <= n; j++)
         e[j] = X0[1][j] - X[j][k+1];
      E[k][5] = vecnorm (e,n);
   }

/*   showmat ("E = ",E,m-1,5,0); */
   
   for (k = 1; k < m; k++)
      printf ("\n %.1f & %12.7f & %12.7e & %12.7e & %12.7e & %12.7e \\\\",
               t[k+1],E[k][1],E[k][2],E[k][3],E[k][4],E[k][5]);
   wait();
   return 0;
}
/*------------------------------------------------------------------*/

   

               
   


