function [x,k] = jac (x,A,b,tol,n,m)
%----------------------------------------------------------------
% Usage:       [x,k] = jac (x,A,b,tol,n,m)
%
% Description: Solve linear algebraic system, Ax = b, iteratively
%              using the Jacobi's iterative method. 
%
% Inputs:      x     = n by 1 vector containing initial guess
%              A     = n by n coefficient matrix (nonzero diagonal
%                      elements)  
%              b     = n by 1 right-hand side vector
%              tol   = error tolerance used to terminate search
%              n     = number of components of x
%              m     = maximum number of iterations (m >= 1)
%
% Outputs:     x = n by 1 solution vector
%              k = number of iterations performed.  If 0 < k < m, 
%                  then the following convergence criterion was 
%                  satisfied where r = b - Ax is the residual error
%                  vector:
%
%                  ||r|| < tol 
%
%                  Zero is returned if one of a diagonal element 
%                  of A was found to be zero.  In this case, the 
%                  equations must be reordered.
%----------------------------------------------------------------

% Initialize

   k = 0;
   tol   = args (tol,0,tol,4,'jac');   
   n     = args (n,1,n,5,'jac');
   m     = args (m,1,m,6,'jac');
   n = length(x);
   for i = 1 : n
      if abs(A(i,i)) < eps
          return;
      end
   end

% Iterate

   while ((k < m) & (residual(A,b,x) >= tol))
      for i = 1 : n;
         y = b(i);
         for j = 1 : n
            if j ~= i
               y = y - A(i,j)*x(j); 
            end
         end
         x(i) = y/A(i,i);
      end
      k = k + 1;
   end
%----------------------------------------------------------------
