/* -------------------------------------------------------------------- */
/* File:        show.c  

   Description: This file contains NLIB console support functions to
                interactively display scalars, vectors, and matrices
                on the console screen.  It uses platform-specific 
                functions (not ANSI C) that are contained in the
                console configuration file "console.c".  An appropriate
                version of console.c must be copied from the console
                subdirectory to the NLIB directory. 

   Note:        The environment variable PRINTER is defined in
                autoexec.bat.  If it is not defined, then "prn" is
                used. */
                
#define PRINTER "prn"		/* hard copy device */
#define ROWS     22             /* rows per screen */

/* -------------------------------------------------------------------- */
/* Local Functions */
/* -------------------------------------------------------------------- */

static   void   savemat   (char *s,matrix B,int m,int n,char *c,int mode);
static   void   savecmat  (char *s,cmatrix B,int m,int n,char *c,int mode);
static   FILE  *getprint  (void);

/* -------------------------------------------------------------------- */
static   void    savemat (char *s,matrix B,int m,int n,char *c,int mode) 
/* -------------------------------------------------------------------- */
{
/* Description: Save the m by n real matrix B.

   On entry:    s    = the default file name
                B    = the m by n real matrix to be saved
                m    = the number of rows of B
                n    = the number of columns of B
		c    = variable name used for B
                mode = write mode: 0 = append, 1 = overwrite */

   char 	d[81];
   
   printf ("\nEnter a file name <%s>: ",s);
   gets (d);
   if (!strlen(d))
      strcpy (d,s);
   putmat (B,m,n,c,d,mode);
   strcpy (s,d);
}


   
/* -------------------------------------------------------------------- */
static   void   savecmat (char *s,cmatrix B,int m,int n,char *c,int mode) 
/* -------------------------------------------------------------------- */
{
/* Description: Save the m by n complex matrix B.

   On entry:    s    = the default file name
                B    = the m by n complex matrix to be saved
                m    = the number of rows of B
                n    = the number of columns of B
                c    = variable name used for B
                mode = write mode: 0 = append, 1 = overwrite */

   char 	d[81];
   
   printf ("\nEnter a file name <%s>: ",s);
   gets (d); 
   if (!strlen(d))
      strcpy (d,s);
   putcmat (B,m,n,c,d,mode);
   strcpy (s,d);
}

/* -------------------------------------------------------------------- */
static FILE  *getprint (void) {
/* -------------------------------------------------------------------- */

/* Description: Open the printer device for hard copy output.

   Returned:    A pointer to the printer device.

   Notes:       The symbolic constant PRINTER in denotes the
                printer device. */

   static FILE   *p;
   p = fopen (PRINTER,"w");
   return p; }	

   
/* ------------------------------------------------------------------- */
float	prompt (char *s,float a,float b,float df ) {
/* ------------------------------------------------------------------- */

/* Description: Prompt the user for a real number in the interval
                [a,b] using the prompt string s and default df.


   On entry:    s  = prompt string to be displayed
                a  = lower limit for a valid response
                b  = upper limit for a valid response (b >= a)
                df = default response (a to b)

   Returned:    The first valid user response

   Notes:       The lower limit can be suppressed if a is replaced
                by -INFINITY and the upper limit can be suppressed if
                b is replaced by INFINITY.  INFINITY is a symbolic
                constant defined in the NLIB header file nlib.h. */

   char		d[21];
   float	x;
   
   if (b < a)
      nerror (100,"prompt",3,b,"");
   if ((df < a) || (df > b))
      nerror (100,"prompt",4,df,"");

/* Display prompt */
      
   do {
      if      ((a > -INFINITY) && (b < INFINITY))
         printf ("\n%s (%g to %g, default: %g): ",s,a,b,df);
      else if ((a > -INFINITY) && (b >= INFINITY))
         printf ("\n%s (down to %g, default: %g): ",s,a,df);
      else if ((a <= -INFINITY) && (b < INFINITY))
         printf ("\n%s (up to %g, default: %g): ",s,b,df);
      else if ((a <= -INFINITY) && (b >= INFINITY))
         printf ("\n%s (default: %g): ",s,df);

/* Get response */
         
      fgets (d,20,stdin);
      if (d[0] != '\n') 
         x = (float) atof(d);
      else
         x = df;
   }      
   while ((x < a) || (x > b));
   return x;
}

/* ------------------------------------------------------------------- */
int pause (char *s) {
/* ------------------------------------------------------------------- */

/* Description: Display a message to the user, and then wait for a
                key to be entered.

   On entry:    s = prompt string to be displayed.
                
   Returned:    The key entered.

   Note:        If s = "", then the following default string
                is displayed:

                "Press Enter to continue ... "  */

   char		keys[21];	
   int		c;

   if (strlen(s) == 0)
      printf ("\nPress Enter to continue ... ");
   else   
      printf ("\n%s",s);
   fgets (keys,20,stdin);
   c = toupper ((int) keys[0]);
   return c; }
         

/* ------------------------------------------------------------------- */
void wait (void) 
/* ------------------------------------------------------------------- */
{
/* Description: Display a standard message to the user and then
                wait for a key to be entered. */

   pause ("");
}             

/* -------------------------------------------------------------------- */
void 	shownum	(char *s,float q) {
/* -------------------------------------------------------------------- */

/* Description: Display the real scalar q on the screen and label
                it with the string s.

   On entry:    s = string used to label the real scalar
                q = real scalar to be displayed

   Notes:       shownum can be called with q replaced by an
                integer or a float. */

   matrix	A;

   A = mat (1,1,"");
   A[1][1] = q;
   showmat (s,A,1,1,0);
   freemat (A,1); }

/* -------------------------------------------------------------------- */
void 	showvec (char *s,vector x,int n,int t) {
/* -------------------------------------------------------------------- */

/* Description: Display the n by 1 real vector x on the screen, enclose
                it in brackets, and label it with the string s.

   On entry:    s = string used to label the real vector
                x = n by 1 real vector to be displayed
                n = number of components of x
                t = transpose code.  If t is nonzero, display the
                    transposed version of x. */

   int		i;
   matrix	A;         

   if (n < 1)
      nerror (100,"showvec",3,(float) n,"");
   A = mat (n,1,"");
   for (i = 1; i <= n; i++) 
      A[i][1] = x[i];
   showmat (s,A,n,1,t);
   freemat (A,n); }

/* -------------------------------------------------------------------- */
void	showmat	(char *s,matrix A,int m,int n,int t) {
/* -------------------------------------------------------------------- */

/* Description: Display the m by n real matrix A on the screen, enclose
                it in brackets, and label it with the string s.

   On entry:    s = variable name for the real matrix
                A = m by n real matrix to be displayed
                m = number of rows of A
                n = number of columns of A
                t = transpose code.  If t is odd, display the
                    transposed version of A. */


	int 	ch;
   static char  fname[81] = "temp.m";
   FILE		*p = getprint();
   
   listmat (stdout,s,A,m,n,t);
   ch = pause("Save, Append, Print, eXit, press Enter to continue ... ");
   switch (ch) {

      case 'S':					/* save matrix */
      
         savemat (fname,A,m,n,s,1);
         break;
      
      case 'A':					/* append matrix */
      
         savemat (fname,A,m,n,s,0);
         break;
      
      case 'P':					/* print matrix */

         listmat (p,s,A,m,n,0);
         break; 

      case 'X':					/* exit program */

         exit(0);
         break; } }

/* -------------------------------------------------------------------- */
void 	showcnum (char *s,complex u) {
/* -------------------------------------------------------------------- */

/* Description: Display the complex scalar u on the screen and label
                it with the string s.

   On entry:    s = string used to label the complex scalar
                q = complex scalar to be displayed */

   cmatrix	A;

   A = cmat (1,1,"");
   A[1][1] = u;
   showcmat (s,A,1,1,0);
   freecmat (A,1); }

/* -------------------------------------------------------------------- */
void 	showcvec (char *s,cvector x,int n,int t) {
/* -------------------------------------------------------------------- */

/* Description: Display the n by 1 complex vector x on the screen, 
                enclose it in brackets, and label it with the string s.

   On entry:    s = string used to label the complex vector
                x = n by 1 complex vector to be displayed
                n = number of components of x
                t = transpose code.  If t is nonzero, display the
                    transposed version of x. */

   int		i;
   cmatrix	A;         

   if (n < 1)
      nerror (100,"showcvec",3,(float) n,"");
   A = cmat (n,1,"");
   for (i = 1; i <= n; i++) 
      A[i][1] = x[i];
   showcmat (s,A,n,1,t);
   freecmat (A,n); }

/* -------------------------------------------------------------------- */
void	showcmat (char *s,cmatrix A,int m,int n,int t) {
/* -------------------------------------------------------------------- */

/* Description: Display the m by n complex matrix A on the screen,
                enclose it in brackets, and label it with the string s.

   On entry:    s = variable name for the complex matrix
                A = m by n complex matrix to be displayed
                m = number of rows of A
                n = number of columns of A
                t = transpose code.  If t is odd, display the
                    transposed version of A. */

   int 	ch;
   static char  fname[81] = "temp.m";
   FILE		*p = getprint();
   
   listcmat (stdout,s,A,m,n,t);
	ch = pause
             ("Save, Append, Print, eXit, press Enter to continue ... ");
   switch (ch) {

      case 'S':					/* save matrix */
      
         savecmat (fname,A,m,n,s,1);
         break;
      
      case 'A':					/* append matrix */
      
         savecmat (fname,A,m,n,s,0);
         break;
      
      case 'P':					/* print matrix */

         printf ("\nPrinting ... ");
         listcmat (p,s,A,m,n,0);
         printf ("\b\b\b\b\b\b\b\b\b\b\b\b\b");
         break; 

      case 'X':					/* exit program */

         exit(0);
         break;

      case 'C':					/* continue */

         break; } }


/* -------------------------------------------------------------------- */
void 	listnum	(char *s,float q) {
/* -------------------------------------------------------------------- */

/* Description: Output the real scalar q to the printer device, and 
                label it with the string s.

   On entry:    s = string used to label the real scalar
                q = real scalar to be printed

   Notes:       listnum can be called with q replaced by an
                integer or a float. */

   matrix	A;
   FILE		*p = getprint();   
   
   A = mat (1,1,"");
   A[1][1] = q;
   listmat (p,s,A,1,1,0);
   freemat (A,1); }
            
/* -------------------------------------------------------------------- */
void	listvec	(char *s,vector x,int n,int t) {
/* -------------------------------------------------------------------- */

/* Description: Output the n by 1 real vector x to the printer device,
                enclose it in brackets, and label it with the string s.

   On entry:    s = string used to label the real vector
                x = n by 1 real vector to be printed
                n = number of components of x
                t = transpose code.  If t is nonzero, print the
                    transposed version of x. */

   int          i;
   matrix	A;         
   FILE		*p = getprint();

   if (n < 1)
      nerror (100,"listvec",3,(float) n,"");
   A = mat (n,1,"");
   for (i = 1; i <= n; i++)
      A[i][1] = x[i];
   listmat (p,s,A,n,1,t);
   freemat (A,n); }

/* -------------------------------------------------------------------- */
void	listmat	(FILE *dev,char *s,matrix A,int m,int n,int t) {
/* -------------------------------------------------------------------- */

/* Description: Output the m by n real matrix A to device dev,
                enclose it in brackets, and label it with the string s.

   On entry:    dev = pointer to display device (console or printer)  
		s = variable name for the real matrix
                A = m by n real matrix to be printed
                m = number of rows of A
                n = number of columns of A
                t = transpose code.  If t is nonzero, print the
                    transposed version of A. */

   matrix	B;
   int          cols = 80,
                rlab = 5,
                i,j,k,c,y1,n0,n1,width,last=0,col;
   char		format[26],
                ne = ' ',se = ' ',nw = ' ',sw = ' ',
                bar = '|',dash = '-';

   
/* Transpose? */

   if (t) {
      B = mat (n,m,"");
      transpose (A,m,n,B);
      i = m;
      m = n;
      n = i; }
   else {
      B = mat (m,n,"");
      copymat (A,m,n,B); }   

/* Initialize */

   if (dev == stdout)
   {
      ne = (char) NE;
      se = (char) SE;
      nw = (char) NW;
      sw = (char) SW;
      bar = (char) VBAR;
      dash = (char) HBAR;
   }   
   col = 12;
   strcpy (format,"%");
   sprintf (format+1,"%i",col-1);
   strcat (format,".4lg ");
   c = strlen (s) + 3;
   width = (cols - 7 - c)/col;
   n0 = 1;
   n1 = MIN(n,width);
   y1 = (m+1)/2;

/* Scalar? */

  if (m*n == 1)
  {
    fprintf (dev,"\n%s = ",s);	
    fprintf (dev,"%.4lg\n",(double) B[1][1]);
    return;
  } 

  do {

/* Label columns */

      if (n > 1) {
      	 fprintf (dev,"\n");
         for (i = 0; i < (c+col/2); i++)
            fprintf (dev," ");
         for (i = n0; i <= n1; i++) {
            fprintf (dev,"%3i",i);
            if (i < n1) {
               for (j = 0; j < col-3; j++)
                  fprintf (dev," "); } } }

/* Print A */   
   
      k = n1 - n0 + 1;
      fprintf (dev,"\n");
      for (i = 0; i <= (m+1); i++) {
         if ((i == 0) || (i == (m+1)))
         {
            for (j = 0; j < c; j++)
               fprintf (dev," ");
            if (i == 0)    
               fprintf (dev,"%c",nw);
            else
               fprintf (dev,"%c",sw);
      	    for (j = 0; j < col*k; j++)
               fprintf (dev,"%c",dash);
            if (i == 0)    
               fprintf (dev,"%c\n",ne);
            else
               fprintf (dev,"%c\n",se);
         }
         else {
            if ((i+1) % ROWS == 0)
               pause ("Press Enter to continue ..."); 	
            if ((i == y1) && (n1 <= width))
               fprintf (dev,"%s = ",s);
            else
               for (j = 1; j <= c; j++)
                  fprintf (dev," ");
            fprintf (dev,"%c",bar);      
            for (j = n0; j <= n1; j++)
               fprintf (dev,format,(double) B[i][j]); 
            if ((m > 2*rlab) && (i % rlab == 0))
               fprintf (dev,"%c%3i\n",bar,i);
            else
               fprintf (dev,"%c\n",bar); } }
      if (n1 < n) {
         n0 = n1 + 1;
         n1 = MIN(n1+width,n); 
         pause ("Press Enter to continue ..."); } 	
      else
         last = 1; }
   while (!last);

/* Finalize */
   
   fflush (dev);
   freemat (B,m); }

/* -------------------------------------------------------------------- */
void 	listcnum (char *s,complex u) {
/* -------------------------------------------------------------------- */

/* Description: Output the complex scalar u to the printer device,
                and label it with the string s.

   On entry:    s = string used to label the complex scalar
                u = complex scalar to be printed. */

   cmatrix	A;
   FILE		*p = getprint();
   
   A = cmat (1,1,"");
   A[1][1] = u;
   listcmat (p,s,A,1,1,0);
   freecmat (A,1); }
            
/* -------------------------------------------------------------------- */
void	listcvec (char *s,cvector x,int n,int t) {
/* -------------------------------------------------------------------- */

/* Description: Output the n by 1 complex vector x to the printer device,
                enclose it in brackets, and label it with the string s.

   On entry:    s = string used to label the complex vector
                x = n by 1 complex vector to be printed
                n = number of components of x
                t = transpose code.  If t is nonzero, print the
                    transposed version of x. */

   int          i;
   cmatrix	A;
   FILE		*p = getprint();
   
   if (n < 1)
      nerror (100,"listcvec",3,(float) n,"");
   A = cmat (n,1,"");
   for (i = 1; i <= n; i++)
      A[i][1] = x[i];
   listcmat (p,s,A,n,1,t);
   freecmat (A,n); }

/* -------------------------------------------------------------------- */
void	listcmat (FILE *dev,char *s,cmatrix A,int m,int n,int t) {
/* -------------------------------------------------------------------- */

/* Description: Output the m by n complex matrix A to device dev,
                enclose it in brackets, and label it with the string s.

   On entry:    dev = display device (console or printer )
                s = variable name for the complex matrix
                A = m by n complex matrix to be printed
                m = number of rows of A
                n = number of columns of A
                t = transpose code.  If t is nonzero, print the
                    transposed version of A. */

   cmatrix	B;
   double	x,y;
   int          cols = 80,
		rlab = 5,
		i,j,k,c,y1,n0,n1,width,last=0,col;
   char		format[35],
                ne = ' ',se = ' ',nw = ' ',sw = ' ',
                bar = '|',dash = '-';
   
/* Transpose? */

   if (t) {
      B = cmat (n,m,"");
      for (i = 1; i <= n; i++)
         for (j = 1; j <= m; j++)
            B[i][j] = A[j][i];
      i = m;
      m = n;
      n = i; }
   else {
      B = cmat (m,n,"");
      for (i = 1; i <= m; i++)
         for (j = 1; j <= n; j++)
            B[i][j] = A[i][j]; }

/* Initialize */

   if (dev == stdout)
   {
      ne = (char) NE;
      se = (char) SE;
      nw = (char) NW;
      sw = (char) SW;
      bar = (char) VBAR;
      dash = (char) HBAR;
   }   
   col = 24;
   strcpy (format,"%");
   sprintf (format+1,"%i",col/2-1);
   strcat (format,".4lg%+");
   sprintf (format+strlen(format),"%i",col/2-1);
   strcat (format,".4lgj");
   c = strlen (s) + 3;
   width = (cols - 7 - c)/col;
   n0 = 1;
   n1 = MIN(n,width);
   y1 = (m+1)/2;

/* Scalar? */

  if (m*n == 1)
  {
    fprintf (dev,"\n%s = ",s);	
    fprintf (dev,"%.4lg%+.4lgj\n",(double) B[1][1].x,B[1][1].y);
    return;
  } 

   do {
   
/* Label columns */

      if (n > 1) {
      	 fprintf (dev,"\n");
         for (i = 1; i < (c+col/2); i++)
            fprintf (dev," ");
         for (i = n0; i <= n1; i++) {
            fprintf (dev,"%3i",i);
            if (i < n1) {
               for (j = 0; j < col-4; j++)
                  fprintf (dev," "); } } }

/* Print A */   

      k = n1 - n0 + 1;
      fprintf (dev,"\n");
      for (i = 0; i <= (m+1); i++) {
         if ((i == 0) || (i == (m+1)))
         {
            for (j = 0; j < c; j++)
               fprintf (dev," ");
            if (i == 0)    
               fprintf (dev,"%c",nw);
            else
               fprintf (dev,"%c",sw);
      	    for (j = k; j <= col*k; j++)
               fprintf (dev,"%c",dash);
            if (i == 0)    
               fprintf (dev,"%c\n",ne);
            else
               fprintf (dev,"%c\n",se);
         }   
         else {
            if ((i+1) % ROWS == 0)
               pause ("Press Enter to continue ..."); 	
            if ((i == y1) && (n1 <= width))
               fprintf (dev,"%s = ",s);
            else
               for (j = 1; j <= c; j++)
                  fprintf (dev," ");
            fprintf (dev,"%c",bar);      
            for (j = n0; j <= n1; j++) {
               x = B[i][j].x;
               y = B[i][j].y;	
               fprintf (dev,format,x,y); }
            if ((m > 2*rlab) && (i % rlab == 0))
               fprintf (dev," %c%3i\n",bar,i);
            else
               fprintf (dev," %c\n",bar); } }
      if (n1 < n) {
         n0 = n1 + 1;
         n1 = MIN(n1+width,n); 
         pause ("Press Enter to continue ..."); } 	
      else
         last = 1; }
   while (!last);

/* Finalize */
   
   fflush (dev);
   freecmat (B,m); }

#undef PRINTER

